@@ -25,6 +25,7 @@
 #define PPTXXMLDOCREADER_H
 
 #include <MsooXmlThemesReader.h>
+#include "PptxXmlSlideReader.h"
 
 class PptxImport;
 class PptxSlideProperties;
@@ -38,18 +39,17 @@ class MsooXmlRelationships;
 class PptxXmlDocumentReaderContext : public MSOOXML::MsooXmlReaderContext
 {
 public:
-    PptxXmlDocumentReaderContext(PptxImport& _import, const QMap<QString, MSOOXML::DrawingMLTheme*>& _themes,
-                                 const QString& _path, const QString& _file,
+    PptxXmlDocumentReaderContext(PptxImport& _import, const QString& _path, const QString& _file,
                                  MSOOXML::MsooXmlRelationships& _relationships);
     PptxImport *import;
-    const QMap<QString, MSOOXML::DrawingMLTheme*> *themes;
     const QString path;
     const QString file;
     MSOOXML::MsooXmlRelationships* relationships;
+    bool firstReadRound;
 };
 
 //! A class reading MSOOXML PPTX markup - presentation.xml part.
-class PptxXmlDocumentReader : public MSOOXML::MsooXmlReader
+class PptxXmlDocumentReader : public PptxXmlSlideReader
 {
 public:
     explicit PptxXmlDocumentReader(KoOdfWriters *writers);
@@ -68,12 +68,18 @@ protected:
     KoFilter::ConversionStatus read_sldIdLst();
     KoFilter::ConversionStatus read_sldId();
     KoFilter::ConversionStatus read_sldSz();
-    
+    KoFilter::ConversionStatus read_defaultTextStyle();
+
     // Locates slide layout informaitons for given slide. Caches the result.
     PptxSlideLayoutProperties* slideLayoutProperties(const QString& slidePath, const QString& slideFile);
 
+    KoOdfWriters *m_writers;
     PptxXmlDocumentReaderContext* m_context;
+
+    void initializeContext(PptxXmlSlideReaderContext& context, const MSOOXML::DrawingMLTheme& theme);
+
 private:
+
     void init();
 
     class Private;
