@@ -42,6 +42,7 @@
 #include "globalssubstreamhandler.h"
 #include "worksheetsubstreamhandler.h"
 #include "chartsubstreamhandler.h"
+#include "XlsRecordOutputStream.h"
 
 //#define SWINDER_XLS2RAW
 
@@ -417,121 +418,6 @@ void ColumnSpanInfo::setLastColumn(unsigned c)
     spaninfo->lastColumn = c;
 }
 
-// ========== BOF ==========
-
-const unsigned int BOFRecord::id = 0x0809;
-
-// helper class for BOFRecord
-class BOFRecord::Private
-{
-public:
-    unsigned version;  // 0x0500=Excel95, 0x0600=Excel97, and so on
-    unsigned type;
-    unsigned build;
-    unsigned year;
-    unsigned history;
-    unsigned rversion;
-};
-
-// constructor of BOFRecord
-BOFRecord::BOFRecord(Swinder::Workbook *book):
-        Record(book)
-{
-    d = new BOFRecord::Private();
-    d->version  = 0x600; // BIFF8;
-    d->type     = 0;
-    d->build    = 0;
-    d->year     = 0;
-    d->history  = 0;
-    d->rversion = 0;
-}
-
-// destructor of BOFRecord
-BOFRecord::~BOFRecord()
-{
-    delete d;
-}
-
-void BOFRecord::setData(unsigned size, const unsigned char* data, const unsigned int*)
-{
-    if (size < 4) return;
-
-    d->version  = readU16(data);
-    d->type     = readU16(data + 2);
-    if (size >= 8) {
-        d->build    = readU16(data + 4);
-        d->year     = readU16(data + 6);
-        if (size >= 16) {
-            d->history  = readU32(data + 8);
-            d->rversion = readU32(data + 12);
-        }
-    }
-}
-
-unsigned BOFRecord::version() const
-{
-    unsigned ver = UnknownExcel;
-    switch (d->version) {
-    case 0x0500 : ver = Excel95; break;
-    case 0x0600 : ver = Excel97; break;
-    default: break;
-    }
-    return ver;
-}
-
-const char* BOFRecord::versionAsString() const
-{
-    const char *result = "Unknown";
-    switch (version()) {
-    case Excel95 : result = "Excel95"; break;
-    case Excel97 : result = "Excel97"; break;
-    default: break;
-    }
-    return result;
-}
-
-unsigned BOFRecord::type() const
-{
-    unsigned result = UnknownType;
-    switch (d->type) {
-    case 0x005  : result = Workbook; break;
-    case 0x006  : result = VBModule; break;
-    case 0x010  : result = Worksheet; break;
-    case 0x020  : result = Chart; break;
-    case 0x040  : result = MacroSheet; break;
-    case 0x100  : result = Workspace; break;
-    default: break;
-    }
-    return result;
-}
-
-const char* BOFRecord::typeAsString() const
-{
-    const char *result = "Unknown";
-    switch (type()) {
-    case Workbook   : result = "Workbook"; break;
-    case VBModule   : result = "Visual Basic Module"; break;
-    case Worksheet  : result = "Worksheet"; break;
-    case Chart      : result = "Chart"; break;
-    case MacroSheet : result = "Macro Sheet"; break;
-    case Workspace  : result = "Workspace File"; break;
-    default: break;
-    }
-    return result;
-}
-
-void BOFRecord::dump(std::ostream& out) const
-{
-    out << "BOF" << std::endl;
-    out << "            Version : 0x" << std::hex << d->version << " (" << versionAsString() << ")" << std::endl;
-    out << "               Type : 0x" << d->type << " (" << typeAsString() << ")" << std::endl;
-    out << "              Build : 0x" << d->build << std::endl;
-    out << "               Year : " << std::dec << d->year << std::endl;
-    out << "            History : 0x" << std::hex << d->history << std::endl;
-    out << "           RVersion : 0x" << d->rversion << std::endl;
-    out << std::dec;
-}
-
 // ========== EXTERNBOOK ==========
 
 const unsigned int ExternBookRecord::id = 0x01ae;
@@ -571,6 +457,21 @@ void ExternBookRecord::setData(unsigned size, const unsigned char* data, const u
         d->name = QString(":");
     } else {
         d->name = EString::fromUnicodeString(data + 2, true, size - 2).str();
+        if (d->name.length() > 2 && d->name[0] == 0x0001) {
+            if (d->name[1] == 0x0001) {
+                // 'unc-volume'
+                d->name = "unc://" + d->name.mid(3).replace(0x0003, '/');
+            } else if (d->name[1] == 0x0002) {
+                // relative to drive volume
+                d->name = d->name.mid(2).replace(0x0003, '/');
+            } else if (d->name[1] == 0x0005) {
+                // full url
+                d->name = d->name.mid(3);
+            } else {
+                // TODO other options
+                d->name = d->name.mid(2).replace(0x0003, '/');
+            }
+        }
     }
 }
 
@@ -663,7 +564,7 @@ FormulaRecord::FormulaRecord(Workbook *book):
         Record(book)
 {
     d = new FormulaRecord::Private();
-    d->shared = true;
+    d->shared = false;
 }
 
 FormulaRecord::~FormulaRecord()
@@ -686,6 +587,11 @@ FormulaTokens FormulaRecord::tokens() const
     return d->tokens;
 }
 
+void FormulaRecord::addToken(const FormulaToken &token)
+{
+    d->tokens.push_back(token);
+}
+
 bool FormulaRecord::isShared() const
 {
     return d->shared;
@@ -738,6 +644,75 @@ void FormulaRecord::setData(unsigned size, const unsigned char* data, const unsi
     d->tokens = decoder.decodeFormula(size, 20, data, version());
 }
 
+void FormulaRecord::writeData(XlsRecordOutputStream &o) const
+{
+    o.writeUnsigned(16, row());
+    o.writeUnsigned(16, column());
+    o.writeUnsigned(16, xfIndex());
+    if (d->result.isNumber()) {
+        o.writeFloat(64, d->result.asFloat());
+    } else if (d->result.isString()) {
+        o.writeUnsigned(8, 0); // type
+        o.writeUnsigned(24, 0); // reserved
+        o.writeUnsigned(16, 0); // reserved
+        o.writeUnsigned(16, 0xFFFF);
+    } else if (d->result.isBoolean()) {
+        o.writeUnsigned(8, 1); // type
+        o.writeUnsigned(8, 0); // reserved
+        o.writeUnsigned(8, d->result.asBoolean() ? 1 : 0);
+        o.writeUnsigned(24, 0); // reserved
+        o.writeUnsigned(16, 0xFFFF);
+    } else if (d->result.isError()) {
+        o.writeUnsigned(8, 2); // type
+        o.writeUnsigned(8, 0); // reserved
+        Value v = d->result;
+        if (v == Value::errorNULL()) {
+            o.writeUnsigned(8, 0x00);
+        } else if (v == Value::errorDIV0()) {
+            o.writeUnsigned(8, 0x07);
+        } else if (v == Value::errorVALUE()) {
+            o.writeUnsigned(8, 0x0F);
+        } else if (v == Value::errorREF()) {
+            o.writeUnsigned(8, 0x17);
+        } else if (v == Value::errorNAME()) {
+            o.writeUnsigned(8, 0x1D);
+        } else if (v == Value::errorNUM()) {
+            o.writeUnsigned(8, 0x24);
+        } else if (v == Value::errorNA()) {
+            o.writeUnsigned(8, 0x2A);
+        } else {
+            o.writeUnsigned(8, 0x2A);
+        }
+        o.writeUnsigned(24, 0); // reserved
+        o.writeUnsigned(16, 0xFFFF);
+    } else {
+        o.writeUnsigned(8, 3); // type
+        o.writeUnsigned(24, 0); // reserved
+        o.writeUnsigned(16, 0); // reserved
+        o.writeUnsigned(16, 0xFFFF);
+    }
+
+    o.writeUnsigned(1, 1); // fAlwaysRecalc
+    o.writeUnsigned(1, 0); // reserved
+    o.writeUnsigned(1, 0); // fFill
+    o.writeUnsigned(1, d->shared ? 1 : 0);
+    o.writeUnsigned(1, 0); // reserved
+    o.writeUnsigned(1, 0); // fClearErrors
+    o.writeUnsigned(10, 0); // reserved
+    o.writeUnsigned(32, 0); // chn
+    // actual formula
+    unsigned totalSize = 0;
+    for (unsigned i = 0; i < d->tokens.size(); i++) {
+        totalSize += d->tokens[i].size() + 1;
+    }
+    o.writeUnsigned(16, totalSize);
+    for (unsigned i = 0; i < d->tokens.size(); i++) {
+        o.writeUnsigned(8, d->tokens[i].id()); // ptg
+        std::vector<unsigned char> data = d->tokens[i].data();
+        o.writeBlob(QByteArray::fromRawData(reinterpret_cast<char*>(data.data()), data.size()));
+    }
+}
+
 void FormulaRecord::dump(std::ostream& out) const
 {
     out << "FORMULA" << std::endl;
@@ -1248,9 +1223,9 @@ class SSTRecord::Private
 {
 public:
     unsigned total;
-    unsigned count;
     std::vector<QString> strings;
     std::vector<std::map<unsigned, unsigned> > formatRuns;
+    ExtSSTRecord* esst;
 };
 
 SSTRecord::SSTRecord(Workbook *book):
@@ -1258,7 +1233,7 @@ SSTRecord::SSTRecord(Workbook *book):
 {
     d = new SSTRecord::Private();
     d->total = 0;
-    d->count = 0;
+    d->esst = 0;
 }
 
 SSTRecord::~SSTRecord()
@@ -1281,14 +1256,14 @@ void SSTRecord::setData(unsigned size, const unsigned char* data, const unsigned
     if (size < 8) return;
 
     d->total = readU32(data);
-    d->count = readU32(data + 4);
+    unsigned count = readU32(data + 4);
 
     unsigned offset = 8;
     unsigned int nextContinuePosIdx = 0;
     unsigned int nextContinuePos = continuePositions[0];
 
     d->strings.clear();
-    for (unsigned i = 0; i < d->count; i++) {
+    for (unsigned i = 0; i < count; i++) {
         // check against size
         if (offset >= size) {
             std::cerr << "Warning: reached end of SST record, but not all strings have been read!" << std::endl;
@@ -1304,15 +1279,47 @@ void SSTRecord::setData(unsigned size, const unsigned char* data, const unsigned
 
 
     // sanity check, adjust to safer condition
-    if (d->count > d->strings.size()) {
-        std::cerr << "Warning: mismatch number of string in SST record, expected " << d->count << ", got " << d->strings.size() << "!" << std::endl;
-        d->count = d->strings.size();
+    if (count > d->strings.size()) {
+        std::cerr << "Warning: mismatch number of string in SST record, expected " << count << ", got " << d->strings.size() << "!" << std::endl;
+    }
+}
+
+void SSTRecord::writeData(XlsRecordOutputStream &out) const
+{
+    unsigned dsst = qMax<unsigned>(8, (count() / 128)+1);
+    if (d->esst) {
+        d->esst->setDsst(dsst);
+        d->esst->setGroupCount((count() + dsst-1) / dsst);
+    }
+    out.writeUnsigned(32, d->total);
+    out.writeUnsigned(32, count());
+    for (unsigned i = 0; i < count(); i++) {
+        if (i % dsst == 0 && d->esst) {
+            d->esst->setIb(i/dsst, out.pos());
+            d->esst->setCbOffset(i/dsst, out.recordPos() + 4);
+        }
+        out.writeUnicodeStringWithFlagsAndLength(stringAt(i));
     }
 }
 
 unsigned SSTRecord::count() const
 {
-    return d->count;
+    return d->strings.size();
+}
+
+unsigned SSTRecord::useCount() const
+{
+    return d->total;
+}
+
+void SSTRecord::setUseCount(unsigned count)
+{
+    d->total = count;
+}
+
+void SSTRecord::setExtSSTRecord(ExtSSTRecord *esst)
+{
+    d->esst = esst;
 }
 
 // why not just string() ? to avoid easy confusion with std::string
@@ -1328,6 +1335,12 @@ std::map<unsigned, unsigned> SSTRecord::formatRunsAt(unsigned index) const
     return d->formatRuns[ index ];
 }
 
+unsigned SSTRecord::addString(const QString &string)
+{
+    d->strings.push_back(string);
+    return d->strings.size()-1;
+}
+
 void SSTRecord::dump(std::ostream& out) const
 {
     out << "SST" << std::endl;
@@ -1406,7 +1419,6 @@ void ObjRecord::setData(unsigned size, const unsigned char* data, const unsigned
     switch (ot) {
     case Object::Group: // gmo
         printf("ObjRecord::setData group\n");
-        m_object = new GroupObject(id);
         startPict += 6;
         break;
     case Object::Picture: { // pictFormat and pictFlags
@@ -1518,11 +1530,7 @@ void ObjRecord::setData(unsigned size, const unsigned char* data, const unsigned
     case Object::Label: printf("ObjRecord::setData Label\n"); break;
     case Object::DialogBox: printf("ObjRecord::setData DialogBox\n"); break;
     case Object::GroupBox: printf("ObjRecord::setData GroupBox\n"); break;
-
-    case Object::OfficeArt:
-        printf("ObjRecord::setData OfficeArt\n");
-        m_object = new OfficeArtObject(id);
-        break;
+    case Object::OfficeArt: printf("ObjRecord::setData OfficeArt\n"); break;
 
     default:
         std::cerr << "ObjRecord::setData: Unexpected objecttype " << ot << " in ObjRecord" << std::endl;
@@ -1661,14 +1669,15 @@ TxORecord::~TxORecord() {}
 void TxORecord::dump(std::ostream& out) const
 {
     out << "TxO" << std::endl;
+    out << "   " << m_text << " " << halign << " " << valign;
 }
 
-void TxORecord::setData(unsigned size, const unsigned char* data, const unsigned* /* continuePositions */)
+void TxORecord::setData(unsigned size, const unsigned char* data, const unsigned* continuePositions)
 {
-    //const unsigned long opts1 = readU16(data);
+    const unsigned long opts1 = readU16(data);
     //const bool reserved1 = opts1 & 0x01;
-    //const unsigned int hAlignment = opts1 & 0x000e; // 3 bits
-    //const unsigned int vAlignment = opts1 & 0x0070; // 3 bits
+    halign = static_cast<HorizontalAlignment>((opts1 & 0x000e) >> 1); // 3 bits
+    valign = static_cast<VerticalAlignment>((opts1 & 0x0070) >> 4); // 3 bits
     //const unsigned long rot = readU16(data + 2);
     // 4 bytes reserved
 
@@ -1678,6 +1687,7 @@ void TxORecord::setData(unsigned size, const unsigned char* data, const unsigned
 
     const unsigned long cchText = readU16(data + 14);
     const unsigned char* startPict = data + 16;
+    const unsigned char* endPict = data + size;
     if(cchText > 0) {
         //const unsigned long cbRuns = readU16(startPict);
         const unsigned long cbFmla = readU16(startPict + 2); // fmla, ObjFmla structure
@@ -1685,6 +1695,9 @@ void TxORecord::setData(unsigned size, const unsigned char* data, const unsigned
     } else {
         //const unsigned long ifntEmpty = readU16(data + 18); // FontIndex
         startPict += 2;
+        const unsigned *endOffset = continuePositions;
+        while (data + *endOffset <= startPict && *endOffset < size) endOffset++;
+        endPict = data + *endOffset;
     }
 
     const unsigned opts = readU8(startPict);
@@ -1696,14 +1709,24 @@ void TxORecord::setData(unsigned size, const unsigned char* data, const unsigned
     // XLUnicodeStringNoCch
     m_text = QString();
     if(fHighByte) {
-        for (unsigned k = 1; k + 2 < size; k += 2) {
+        for (unsigned k = 1; startPict + k + 1 < endPict; k += 2) {
             unsigned zc = readU16(startPict + k);
-            m_text.append(QString(zc));
+            if (!zc) break;
+            if (!QChar(zc).isPrint() && zc != 10) {
+                m_text.clear();
+                break;
+            }
+            m_text.append(QChar(zc));
         }
     } else {
-        for (unsigned k = 1; k + 1 < size; k += 1) {
+        for (unsigned k = 1; startPict + k < endPict; k += 1) {
             unsigned char uc = readU8(startPict + k) + 0x0*256;
-            m_text.append(QString(uc));
+            if (!uc) break;
+            if (!QChar(uc).isPrint() && uc != 10) {
+                m_text.clear();
+                break;
+            }
+            m_text.append(QChar(uc));
         }
     }
 
@@ -1714,6 +1737,28 @@ void TxORecord::setData(unsigned size, const unsigned char* data, const unsigned
 
 const unsigned MsoDrawingRecord::id = 0xEC;
 
+class MsoDrawingRecord::Private
+{
+public:
+    MSO::OfficeArtDgContainer container;
+};
+
+MsoDrawingRecord::MsoDrawingRecord(Workbook *book)
+    : Record(book)
+{
+    d = new MsoDrawingRecord::Private();
+}
+
+MsoDrawingRecord::~MsoDrawingRecord()
+{
+    delete d;
+}
+
+const MSO::OfficeArtDgContainer& MsoDrawingRecord::dgContainer() const
+{
+    return d->container;
+}
+
 void MsoDrawingRecord::dump(std::ostream& out) const
 {
     out << "MsoDrawingRecord" << std::endl;
@@ -1721,11 +1766,6 @@ void MsoDrawingRecord::dump(std::ostream& out) const
 
 void MsoDrawingRecord::setData(unsigned size, const unsigned char* data, const unsigned* continuePositions)
 {
-    if(size < 24) {
-        setIsValid(false);
-        return;
-    }
-
     QByteArray byteArr = QByteArray::fromRawData(reinterpret_cast<const char*>(data), size);
     QBuffer buff(&byteArr);
     buff.open(QIODevice::ReadOnly);
@@ -1764,15 +1804,39 @@ void MsoDrawingRecord::setData(unsigned size, const unsigned char* data, const u
     }
     
     // Finally remember the container to be able to extract later content out of it.
-    m_container = container;
+    d->container = container;
 }
 
 // ========== MsoDrawingGroup ==========
 
 const unsigned MsoDrawingGroupRecord::id = 0xEB;
 
-MsoDrawingGroupRecord::MsoDrawingGroupRecord(Workbook *book) : Record(book) {}
-MsoDrawingGroupRecord::~MsoDrawingGroupRecord() {}
+class MsoDrawingGroupRecord::Private
+{
+public:
+    MSO::OfficeArtDggContainer container;
+    QList< MsoDrawingBlibItem* > items;
+};
+
+MsoDrawingGroupRecord::MsoDrawingGroupRecord(Workbook *book) : Record(book)
+{
+    d = new Private();
+}
+
+MsoDrawingGroupRecord::~MsoDrawingGroupRecord()
+{
+    delete d;
+}
+
+const MSO::OfficeArtDggContainer& MsoDrawingGroupRecord::dggContainer() const
+{
+    return d->container;
+}
+
+QList<MsoDrawingBlibItem*> MsoDrawingGroupRecord::blibItems() const
+{
+    return d->items;
+}
 
 void MsoDrawingGroupRecord::dump(std::ostream& out) const
 {
@@ -1782,7 +1846,7 @@ void MsoDrawingGroupRecord::dump(std::ostream& out) const
 void MsoDrawingGroupRecord::setData(unsigned size, const unsigned char* data, const unsigned* continuePositions)
 {
     printf("MsoDrawingGroupRecord::setData size=%i data=%i continuePositions=%i\n",size,*data,*continuePositions);
-    if(size < 32 || !m_workbook->store()) {
+    if(size < 32) {
         setIsValid(false);
         return;
     }
@@ -1792,559 +1856,28 @@ void MsoDrawingGroupRecord::setData(unsigned size, const unsigned char* data, co
     buff.open(QIODevice::ReadOnly);
     LEInputStream lei(&buff);
     
-    MSO::OfficeArtDggContainer container;
     try {
-        MSO::parseOfficeArtDggContainer(lei, container);
+        MSO::parseOfficeArtDggContainer(lei, d->container);
     } catch (const IOException& e) {
         std::cerr << "Invalid MsoDrawingGroup record:" << qPrintable(e.msg) << std::endl;
         setIsValid(false);
         return;
     }
     
-    if(!container.blipStore.data()) {
-        setIsValid(false);
-        return;
-    }
+    if(d->container.blipStore.data() && m_workbook->store()) {
+        m_workbook->store()->enterDirectory("Pictures");
     
-    m_workbook->store()->enterDirectory("Pictures");
-    
-    foreach(MSO::OfficeArtBStoreContainerFileBlock fb, container.blipStore->rgfb) {
-        PictureReference ref = savePicture(fb, m_workbook->store());
-        if (ref.name.length() == 0) {
-            std::cerr << "Empty name in picture reference for picture with uid=" << ref.uid << " mimetype=" << ref.mimetype << std::endl;
-            continue;
+        foreach(MSO::OfficeArtBStoreContainerFileBlock fb, d->container.blipStore->rgfb) {
+            PictureReference ref = savePicture(fb, m_workbook->store());
+            if (ref.name.length() == 0) {
+                std::cerr << "Empty name in picture reference for picture with uid=" << ref.uid << " mimetype=" << ref.mimetype << std::endl;
+                continue;
+            }
+            d->items << new MsoDrawingBlibItem(ref);
         }
-        m_items << new MsoDrawingBlibItem(ref);
-    }
     
-    m_workbook->store()->leaveDirectory();
-}
-
-// ========== XF ==========
-
-const unsigned int XFRecord::id = 0x00e0;
-
-class XFRecord::Private
-{
-public:
-    unsigned fontIndex;
-    unsigned formatIndex;
-    bool locked;
-    bool formulaHidden;
-    unsigned parentStyle;
-    unsigned horizontalAlignment;
-    unsigned verticalAlignment;
-    bool textWrap;
-    unsigned rotationAngle;
-    bool stackedLetters;
-    unsigned indentLevel;
-    bool shrinkContent;
-    unsigned leftBorderStyle;
-    unsigned leftBorderColor;
-    unsigned rightBorderStyle;
-    unsigned rightBorderColor;
-    unsigned topBorderStyle;
-    unsigned topBorderColor;
-    unsigned bottomBorderStyle;
-    unsigned bottomBorderColor;
-    bool diagonalTopLeft;
-    bool diagonalBottomLeft;
-    unsigned diagonalStyle;
-    unsigned diagonalColor;
-    unsigned fillPattern;
-    unsigned patternForeColor;
-    unsigned patternBackColor;
-};
-
-XFRecord::XFRecord(Workbook *book):  Record(book)
-{
-    d = new XFRecord::Private();
-    d->fontIndex           = 0;
-    d->formatIndex         = 0;
-    d->locked              = false;
-    d->formulaHidden       = false;
-    d->parentStyle         = 0;
-    d->horizontalAlignment = General;
-    d->verticalAlignment   = General;
-    d->textWrap            = false;
-    d->rotationAngle       = 0;
-    d->stackedLetters      = 0;
-    d->indentLevel         = 0;
-    d->shrinkContent       = 0;
-    d->leftBorderStyle     = 0;
-    d->leftBorderColor     = 0;
-    d->rightBorderStyle    = 0;
-    d->rightBorderColor    = 0;
-    d->topBorderStyle      = 0;
-    d->topBorderColor      = 0;
-    d->bottomBorderStyle   = 0;
-    d->bottomBorderColor   = 0;
-    d->diagonalTopLeft     = false;
-    d->diagonalBottomLeft  = false;
-    d->diagonalStyle       = 0;
-    d->diagonalColor       = 0;
-    d->fillPattern         = 0;
-    d->patternForeColor    = 0;
-    d->patternBackColor    = 0;
-}
-
-XFRecord::~XFRecord()
-{
-    delete d;
-}
-
-XFRecord::XFRecord(const XFRecord& xf):  Record(xf.m_workbook)
-{
-    d = new XFRecord::Private();
-    operator=(xf);
-}
-
-XFRecord& XFRecord::operator=(const XFRecord & xf)
-{
-    d->fontIndex           = xf.fontIndex();
-    d->formatIndex         = xf.formatIndex();
-    d->locked              = xf.locked();
-    d->formulaHidden       = xf.formulaHidden();
-    d->parentStyle         = xf.parentStyle();
-    d->horizontalAlignment = xf.horizontalAlignment();
-    d->verticalAlignment   = xf.verticalAlignment();
-    d->textWrap            = xf.textWrap();
-    d->rotationAngle       = xf.rotationAngle();
-    d->stackedLetters      = xf.stackedLetters();
-    d->indentLevel         = xf.indentLevel();
-    d->shrinkContent       = xf.shrinkContent();
-    d->leftBorderStyle     = xf.leftBorderStyle();
-    d->leftBorderColor     = xf.leftBorderColor();
-    d->rightBorderStyle    = xf.rightBorderStyle();
-    d->rightBorderColor    = xf.rightBorderColor();
-    d->topBorderStyle      = xf.topBorderStyle();
-    d->topBorderColor      = xf.topBorderColor();
-    d->bottomBorderStyle   = xf.bottomBorderStyle();
-    d->bottomBorderColor   = xf.bottomBorderColor();
-    d->diagonalTopLeft     = xf.diagonalTopLeft();
-    d->diagonalBottomLeft  = xf.diagonalBottomLeft();
-    d->diagonalStyle       = xf.diagonalStyle();
-    d->diagonalColor       = xf.diagonalColor();
-    d->fillPattern         = xf.fillPattern();
-    d->patternForeColor    = xf.patternForeColor();
-    d->patternBackColor    = xf.patternBackColor();
-    return *this;
-}
-
-unsigned XFRecord::fontIndex() const
-{
-    return d->fontIndex;
-}
-
-void XFRecord::setFontIndex(unsigned fi)
-{
-    d->fontIndex = fi;
-}
-
-unsigned XFRecord::formatIndex() const
-{
-    return d->formatIndex;
-}
-
-void XFRecord::setFormatIndex(unsigned fi)
-{
-    d->formatIndex = fi;
-}
-
-bool XFRecord::locked() const
-{
-    return d->locked;
-}
-
-void XFRecord::setLocked(bool l)
-{
-    d->locked = l;
-}
-
-bool XFRecord::formulaHidden() const
-{
-    return d->formulaHidden;
-}
-
-void XFRecord::setFormulaHidden(bool f)
-{
-    d->formulaHidden = f;
-}
-
-unsigned XFRecord::parentStyle() const
-{
-    return d->parentStyle;
-}
-
-void XFRecord::setParentStyle(unsigned p)
-{
-    d->parentStyle = p;
-}
-
-unsigned XFRecord::horizontalAlignment() const
-{
-    return d->horizontalAlignment;
-}
-
-void XFRecord::setHorizontalAlignment(unsigned ha)
-{
-    d->horizontalAlignment = ha;
-}
-
-const char* XFRecord::horizontalAlignmentAsString() const
-{
-    const char *result = "Unknown";
-    switch (horizontalAlignment()) {
-    case General:   result = "General"; break;
-    case Left:      result = "Left"; break;
-    case Centered:  result = "Centered"; break;
-    case Right:     result = "Right"; break;
-    case Justified: result = "Justified"; break;
-    case Filled:    result = "Filled"; break;
-    default: break;
+        m_workbook->store()->leaveDirectory();
     }
-    return result;
-}
-
-unsigned XFRecord::verticalAlignment() const
-{
-    return d->verticalAlignment;
-}
-
-void XFRecord::setVerticalAlignment(unsigned va)
-{
-    d->verticalAlignment = va;
-}
-
-const char* XFRecord::verticalAlignmentAsString() const
-{
-    const char *result = "Unknown";
-    switch (verticalAlignment()) {
-    case General:      result = "General"; break;
-    case Top:          result = "Top"; break;
-    case VCentered:    result = "Centered"; break;
-    case Bottom:       result = "Bottom"; break;
-    case VJustified:   result = "Justified"; break;
-    case VDistributed: result = "Distributed"; break;
-    default: break;
-    }
-    return result;
-}
-
-bool XFRecord::textWrap() const
-{
-    return d->textWrap;
-}
-
-void XFRecord::setTextWrap(bool wrap)
-{
-    d->textWrap = wrap;
-}
-
-unsigned XFRecord::rotationAngle() const
-{
-    return d->rotationAngle;
-}
-
-void XFRecord::setRotationAngle(unsigned angle)
-{
-    d->rotationAngle = angle;
-}
-
-bool XFRecord::stackedLetters() const
-{
-    return d->stackedLetters;
-}
-
-void XFRecord::setStackedLetters(bool stacked)
-{
-    d->stackedLetters = stacked;
-}
-
-unsigned XFRecord::indentLevel() const
-{
-    return d->indentLevel;
-}
-
-void XFRecord::setIndentLevel(unsigned i)
-{
-    d->indentLevel = i;
-}
-
-bool XFRecord::shrinkContent() const
-{
-    return d->shrinkContent;
-}
-
-void XFRecord::setShrinkContent(bool s)
-{
-    d->shrinkContent = s;
-}
-
-unsigned XFRecord::leftBorderStyle() const
-{
-    return d->leftBorderStyle;
-}
-
-void XFRecord::setLeftBorderStyle(unsigned style)
-{
-    d->leftBorderStyle = style;
-}
-
-unsigned XFRecord::leftBorderColor() const
-{
-    return d->leftBorderColor;
-}
-
-void XFRecord::setLeftBorderColor(unsigned color)
-{
-    d->leftBorderColor = color;
-}
-
-unsigned XFRecord::rightBorderStyle() const
-{
-    return d->rightBorderStyle;
-}
-
-void XFRecord::setRightBorderStyle(unsigned style)
-{
-    d->rightBorderStyle = style;
-}
-
-unsigned XFRecord::rightBorderColor() const
-{
-    return d->rightBorderColor;
-}
-
-void XFRecord::setRightBorderColor(unsigned color)
-{
-    d->rightBorderColor = color;
-}
-
-unsigned XFRecord::topBorderStyle() const
-{
-    return d->topBorderStyle;
-}
-
-void XFRecord::setTopBorderStyle(unsigned style)
-{
-    d->topBorderStyle = style;
-}
-
-unsigned XFRecord::topBorderColor() const
-{
-    return d->topBorderColor;
-}
-
-void XFRecord::setTopBorderColor(unsigned color)
-{
-    d->topBorderColor = color;
-}
-
-unsigned XFRecord::bottomBorderStyle() const
-{
-    return d->bottomBorderStyle;
-}
-
-void XFRecord::setBottomBorderStyle(unsigned style)
-{
-    d->bottomBorderStyle = style;
-}
-
-unsigned XFRecord::bottomBorderColor() const
-{
-    return d->bottomBorderColor;
-}
-
-void XFRecord::setBottomBorderColor(unsigned color)
-{
-    d->bottomBorderColor = color;
-}
-
-bool XFRecord::diagonalTopLeft() const
-{
-    return d->diagonalTopLeft;
-}
-
-void XFRecord::setDiagonalTopLeft(bool dd)
-{
-    d->diagonalTopLeft = dd;
-}
-
-bool XFRecord::diagonalBottomLeft() const
-{
-    return d->diagonalBottomLeft;
-}
-
-void XFRecord::setDiagonalBottomLeft(bool dd)
-{
-    d->diagonalBottomLeft = dd;
-}
-
-unsigned XFRecord::diagonalStyle() const
-{
-    return d->diagonalStyle;
-}
-
-void XFRecord::setDiagonalStyle(unsigned style)
-{
-    d->diagonalStyle = style;
-}
-
-unsigned XFRecord::diagonalColor() const
-{
-    return d->diagonalColor;
-}
-
-void XFRecord::setDiagonalColor(unsigned color)
-{
-    d->diagonalColor = color;
-}
-
-unsigned XFRecord::fillPattern() const
-{
-    return d->fillPattern;
-}
-
-void XFRecord::setFillPattern(unsigned pattern)
-{
-    d->fillPattern = pattern;
-}
-
-unsigned XFRecord::patternForeColor() const
-{
-    return d->patternForeColor;
-}
-
-void XFRecord::setPatternForeColor(unsigned color)
-{
-    d->patternForeColor = color;
-}
-
-unsigned XFRecord::patternBackColor() const
-{
-    return d->patternBackColor;
-}
-
-void XFRecord::setPatternBackColor(unsigned color)
-{
-    d->patternBackColor = color;
-}
-
-void XFRecord::setData(unsigned size, const unsigned char* data, const unsigned int*)
-{
-    unsigned recordSize = (version() == Excel97) ? 20 : 16;
-    if (size < recordSize) return;
-
-    setFontIndex(readU16(data));
-    setFormatIndex(readU16(data + 2));
-
-    unsigned protection = readU16(data + 4) & 7;
-    setLocked(protection & 1);
-    setFormulaHidden(protection & 2);
-
-    setParentStyle(readU16(data + 4) >> 4);
-
-    unsigned align = data[6];
-    setHorizontalAlignment(align & 0x07);
-    setVerticalAlignment((align >> 4) + 1/*enum starts at 1*/);
-    setTextWrap(align & 0x08);
-
-    if (version() == Excel97) {
-        unsigned angle = data[7];
-        //qDebug() << d->globalIndex << angle;
-        setRotationAngle((angle != 255) ? angle : 0);
-        setStackedLetters(angle == 255);
-
-        unsigned options = data[8];
-//     unsigned attributes = data[9];
-
-        setIndentLevel(options & 0x0f);
-        setShrinkContent(options & 0x10);
-
-        unsigned linestyle = readU16(data + 10);
-        unsigned color1 = readU16(data + 12);
-        unsigned color2 = readU16(data + 14);
-        unsigned flag = readU16(data + 16);
-        unsigned fill = readU16(data + 18);
-
-        setLeftBorderStyle(linestyle & 0xf);
-        setRightBorderStyle((linestyle >> 4) & 0xf);
-        setTopBorderStyle((linestyle >> 8) & 0xf);
-        setBottomBorderStyle((linestyle >> 12) & 0xf);
-
-        setLeftBorderColor(color1 & 0x7f);//127
-        setRightBorderColor((color1 >> 7) & 0x7f);
-        setTopBorderColor(color2 & 0x7f);
-        setBottomBorderColor((color2 >> 7) & 0x7f);
-
-        const unsigned grbitDiag = color1 >> 14;
-        setDiagonalTopLeft(grbitDiag == 0x1 || grbitDiag == 0x3);
-        setDiagonalBottomLeft(grbitDiag == 0x2 || grbitDiag == 0x3);
-        setDiagonalStyle((flag >> 4) & 0x1e);
-        setDiagonalColor(((flag & 0x1f) << 2) + ((color2 >> 14) & 3));
-
-        setFillPattern((flag >> 10) & 0x3f);
-        setPatternForeColor(fill & 0x7f);
-        setPatternBackColor((fill >> 7) & 0x7f);
-    } else {
-        unsigned orient = data[7] & 3;
-        switch (orient) {
-            case 0: break;
-            case 1: setStackedLetters(true); break;
-            case 2: setRotationAngle(90); break;
-            case 3: setRotationAngle(180); break; // 90 degrees clockwise
-        }
-        unsigned data1 = readU32(data + 8);
-        unsigned data2 = readU32(data + 12);
-
-        setPatternForeColor(data1 & 0x7f);
-        setPatternBackColor((data1 >> 7) & 0x7f);
-        setFillPattern((data1 >> 16) & 0x3f);
-
-        setBottomBorderStyle((data1 >> 22) & 0x07);
-        setBottomBorderColor((data1 >> 25) & 0x7f);
-
-        setTopBorderStyle(data2 & 0x07);
-        setLeftBorderStyle((data2 >> 3) & 0x07);
-        setRightBorderStyle((data2 >> 6) & 0x07);
-
-        setTopBorderColor((data2 >> 9) & 0x7f);
-        setLeftBorderColor((data2 >> 16) & 0x7f);
-        setRightBorderColor((data2 >> 23) & 0x7f);
-    }
-}
-
-void XFRecord::dump(std::ostream& out) const
-{
-    out << "XF" << std::endl;
-    out << "       Parent Style : " << parentStyle() << std::endl;
-    out << "         Font Index : " << fontIndex() << std::endl;
-    out << "       Format Index : " << formatIndex() << std::endl;
-    out << "             Locked : " << (locked() ? "Yes" : "No") << std::endl;
-    out << " Formula Visibility : " << (formulaHidden() ? "Hidden" : "Visible") << std::endl;
-    out << "   Horizontal Align : " << horizontalAlignmentAsString() << std::endl;
-    out << "     Vertical Align : " << verticalAlignmentAsString() << std::endl;
-    out << "          Text Wrap : " << (textWrap() ? "yes" : "no") << std::endl;
-    out << "          Rotation  : " << rotationAngle() << std::endl;
-    out << "    Stacked Letters : " << (stackedLetters() ? "yes" : "no") << std::endl;
-    out << "       Indent Level : " << indentLevel() << std::endl;
-    out << "      Shrink To Fit : " << (shrinkContent() ? "yes" : "no") << std::endl;
-    out << "        Left Border : Style " << leftBorderStyle();
-    out << " Color: " << leftBorderColor() << std::endl;
-    out << "       Right Border : Style " << rightBorderStyle();
-    out << " Color: " << rightBorderColor() << std::endl;
-    out << "         Top Border : Style " << topBorderStyle();
-    out << " Color: " << topBorderColor() << std::endl;
-    out << "      Bottom Border : Style " << bottomBorderStyle();
-    out << " Color: " << bottomBorderColor() << std::endl;
-    out << "     Diagonal Lines : " ;
-    if (diagonalTopLeft()) out << "TopLeft ";
-    if (diagonalBottomLeft()) out << "BottomLeft ";
-    out << "Style " << diagonalStyle() << " Color: " << diagonalColor() << std::endl;
-    out << "       Fill Pattern : " << fillPattern() << std::endl;
-    out << "         Fill Color : Fore " << patternForeColor() << " Back: "
-    << patternBackColor() << std::endl;
 }
 
 // ========== BkHimRecord ==========
@@ -2616,10 +2149,6 @@ static Record* createSSTRecord(Workbook *book)
 {
     return new SSTRecord(book);
 }
-static Record* createXFRecord(Workbook *book)
-{
-    return new XFRecord(book);
-}
 static Record* createObjRecord(Workbook *book)
 {
     return new ObjRecord(book);
@@ -2653,7 +2182,6 @@ static void registerAllRecordClasses()
     RecordRegistry::registerRecordClass(RKRecord::id, createRKRecord);
     RecordRegistry::registerRecordClass(RStringRecord::id, createRStringRecord);
     RecordRegistry::registerRecordClass(SSTRecord::id, createSSTRecord);
-    RecordRegistry::registerRecordClass(XFRecord::id, createXFRecord);
     RecordRegistry::registerRecordClass(ObjRecord::id, createObjRecord);
     RecordRegistry::registerRecordClass(TxORecord::id, createTxORecord);
     RecordRegistry::registerRecordClass(MsoDrawingRecord::id, createRecordMsoDrawingRecord);
