@@ -29,6 +29,7 @@
 #include "cell.h"
 #include "sheet.h"
 #include "workbook.h"
+#include "excel.h"
 
 #include <string>
 #include <iostream>
@@ -203,36 +204,22 @@ public:
     bool operator!=(const ChartObject &other) const { return ! (*this == other); }
 };
 
-/**
- * A group object.
- */
-class GroupObject : public Object
-{
-public:
-    explicit GroupObject(unsigned long id) : Object(Group, id) {}
-    virtual ~GroupObject() {}
-};
-
-/**
- * A oval object.
- */
-class OvalObject : public Object
-{
-public:
-    explicit OvalObject(unsigned long id) : Object(Oval, id) {}
-    virtual ~OvalObject() {}
-};
-
 /**
  * A OfficeArt object.
  */
-class OfficeArtObject : public Object
+class OfficeArtObject
 {
 public:
-    explicit OfficeArtObject(unsigned long id) : Object(OfficeArt, id) {}
-    virtual ~OfficeArtObject() {}
-    bool operator==(const OfficeArtObject &other) const { return this == &other; }
-    bool operator!=(const OfficeArtObject &other) const { return ! (*this == other); }
+    explicit OfficeArtObject(const MSO::OfficeArtSpContainer& object);
+    ~OfficeArtObject();
+    MSO::OfficeArtSpContainer object() const;
+    void setText(const TxORecord& text);
+    TxORecord text() const;
+    bool operator==(const OfficeArtObject& other) const { return this == &other; }
+    bool operator!=(const OfficeArtObject& other) const { return !(*this == other); }
+private:
+    MSO::OfficeArtSpContainer m_object;
+    TxORecord m_text;
 };
 
 } // namespace Swinder
