@@ -41,6 +41,9 @@ struct Field {
     bool isStringLength;
     bool isEnum;
     QList<Field> arrayFields;
+    QString defaultValue;
+    QString lengthFor;
+
     Field(QString name = QString(), QString type = QString()) : name(name), type(type), isArray(false), isArrayLength(false), isStringLength(false), isEnum(false) {}
 
     QString getterName() const {
@@ -75,9 +78,12 @@ static QMap<QString, Field> getFields(QDomElement record, bool* foundStrings = 0
 {
     QDomNodeList types = record.elementsByTagName("type");
     QMap<QString, QString> extraTypes;
+    QMap<QString, QString> extraTypesDefaults;
     for (int i = 0; i < types.size(); i++) {
         QDomElement e = types.at(i).toElement();
         extraTypes[e.attribute("name")] = e.attribute("type");
+        if (e.elementsByTagName("enum").size() > 0)
+            extraTypesDefaults[e.attribute("name")] = e.elementsByTagName("enum").at(0).toElement().attribute("name");
     }
 
     QDomNodeList fields = record.elementsByTagName("field");
@@ -94,19 +100,25 @@ static QMap<QString, Field> getFields(QDomElement record, bool* foundStrings = 0
             if (e.elementsByTagName("enum").size() > 0) {
                 map[name].isEnum = true;
                 map[name].type = ucFirst(name);
+                map[name].defaultValue = e.elementsByTagName("enum").at(0).toElement().attribute("name");
             }
             if (extraTypes.contains(e.attribute("type"))) {
                 map[name].isEnum = true;
                 map[name].type = e.attribute("type");
+                map[name].defaultValue = extraTypesDefaults[e.attribute("type")];
             }
+            if (e.hasAttribute("default"))
+                map[name].defaultValue = e.attribute("default");
         }
     }
     for (int i = 0; i < fields.size(); i++) {
         QDomElement e = fields.at(i).toElement();
         if (e.hasAttribute("length")) {
             QString name = e.attribute("length");
-            if (map.contains(name))
+            if (map.contains(name)) {
                 map[name].isStringLength = true;
+                map[name].lengthFor = e.attribute("name");
+            }
         }
     }
     QDomNodeList arrays = record.elementsByTagName("array");
@@ -162,7 +174,7 @@ void processRecordForHeader(QDomElement e, QTextStream& out)
     << "    virtual unsigned rtti() const { return this->id; }\n\n";
 
     // constructor and destructor
-    out << "    " << className << "(Workbook *book);\n    virtual ~" << className << "();\n\n";
+    out << "    " << className << "(Swinder::Workbook *book);\n    virtual ~" << className << "();\n\n";
 
     // copy and assignment
     out << "    " << className << "( const " << className << "& record );\n"
@@ -194,9 +206,31 @@ void processRecordForHeader(QDomElement e, QTextStream& out)
         out << "    " << type << " " << f.attribute("name") << "() const;\n\n";
     }
 
+    // array lengths
+    QDomNodeList arrayNodes = e.elementsByTagName("array");
+    for (int i = 0; i < arrayNodes.size(); i++) {
+        QDomElement f = arrayNodes.at(i).toElement();
+        if (f.hasAttribute("lengthField")) {
+            QString name = f.attribute("lengthField");
+            out << "    unsigned " << lcFirst(name) << "() const;\n";
+            out << "    void set" << ucFirst(name) << "( unsigned " << name << " );\n\n";
+        }
+    }
+
+    // optional groups
+    QDomNodeList optionalNodes = e.elementsByTagName("optional");
+    for (int i = 0; i < optionalNodes.size(); i++) {
+        QDomElement f = optionalNodes.at(i).toElement();
+        out << "    bool has" << ucFirst(f.attribute("name")) << "() const;\n\n";
+        out << "    void setHas" << ucFirst(f.attribute("name")) << "( bool value );\n\n";
+    }
+
     // setData method
     out << "    virtual void setData( unsigned size, const unsigned char* data, const unsigned* continuePositions );\n\n";
 
+    // writeData method
+    out << "    virtual void writeData( XlsRecordOutputStream& out ) const;\n\n";
+
     // name method
     out << "    virtual const char* name() const { return \"" << e.attribute("name") << "\"; }\n\n";
 
@@ -207,7 +241,18 @@ void processRecordForHeader(QDomElement e, QTextStream& out)
     out << "private:\n    class Private;\n    Private * const d;\n};\n\n";
 }
 
-static void sizeCheck(QString indent, QTextStream& out, QDomElement firstField, unsigned offset, bool dynamicOffset)
+static void invalidFound(QString indent, QTextStream& out, QString currentOptional)
+{
+    if (currentOptional.isEmpty()) {
+        out << indent << "setIsValid(false);\n";
+        out << indent << "return;\n";
+    } else {
+        out << indent << "d->has" << ucFirst(currentOptional) << " = false;\n";
+        out << indent << "goto " << lcFirst(currentOptional) << "Failed;\n";
+    }
+}
+
+static void sizeCheck(QString indent, QTextStream& out, QDomElement firstField, unsigned offset, bool dynamicOffset, QString currentOptional)
 {
     // find size of all fields until first unsized field or first if/array
     unsigned size = 0;
@@ -229,18 +274,16 @@ static void sizeCheck(QString indent, QTextStream& out, QDomElement firstField,
         out << indent << "if (size < ";
         if (dynamicOffset) out << "curOffset + ";
         if (offset) out << (offset / 8) << " + ";
-        out << (size / 8) << ") {\n"
-        << indent << "    setIsValid(false);\n"
-        << indent << "    return;\n"
-        << indent << "}\n";
+        out << (size / 8) << ") {\n";
+        invalidFound(indent + "    ", out, currentOptional);
+        out << indent << "}\n";
     }
 }
 
-static void processFieldElement(QString indent, QTextStream& out, QDomElement field, unsigned& offset, bool& dynamicOffset, QMap<QString, Field>& fieldsMap, QString setterArgs = QString())
+static void processFieldElement(QString indent, QTextStream& out, QDomElement field, unsigned& offset, bool& dynamicOffset, QMap<QString, Field>& fieldsMap, QString setterArgs = QString(), QString currentOptional = QString())
 {
     if (field.tagName() == "fail") {
-        out << indent << "setIsValid(false);\n";
-        out << indent << "return;\n";
+        invalidFound(indent, out, currentOptional);
     } else if (field.tagName() == "field") {
         unsigned bits = field.attribute("size").toUInt();
         QString name = field.attribute("name");
@@ -275,19 +318,42 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
                     out << "-1";
                 out << ", size - curOffset"
                 << ", &stringLengthError, &stringSize));\n";
-                out << indent << "if (stringLengthError) {\n"
-                << indent << "    setIsValid(false);\n"
-                << indent << "    return;\n"
-                << indent << "}\n";
+                out << indent << "if (stringLengthError) {\n";
+                invalidFound(indent + "    ", out, currentOptional);
+                out << indent << "}\n";
                 out << indent << "curOffset += stringSize;\n";
-                sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset);
+                sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset, currentOptional);
             } else if (f.type == "QByteArray") {
+                if (offset % 8 != 0)
+                    qFatal("Unaligned string");
+
+                if (field.hasAttribute("length")) {
+                    out << indent << "if (";
+                    if (dynamicOffset) out << "curOffset + ";
+                    if (offset) out << (offset / 8) << " + ";
+                    out << "(" << field.attribute("length") << ") > size) {\n";
+                    invalidFound(indent + "    ", out, currentOptional);
+                    out << indent << "}\n";
+                }
                 out << indent << f.setterName() << "(" << setterArgs;
                 out << "QByteArray(reinterpret_cast<const char*>(";
                 out << "data";
                 if (dynamicOffset) out << " + curOffset";
                 if (offset) out << " + " << (offset / 8);
-                out << "), " << (bits / 8) << "));\n";
+                out << "), ";
+                if (field.hasAttribute("length"))
+                    out << field.attribute("length");
+                else
+                    out << (bits / 8);
+                out << "));\n";
+                if (field.hasAttribute("length")) {
+                    if (!dynamicOffset)
+                        out << indent << "curOffset = " << (offset / 8);
+                    else out << indent << "curOffset += " << (offset / 8);
+                    out << " + " << field.attribute("length") << ";\n";
+                    dynamicOffset = true; offset = 0;
+                    sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset, currentOptional);
+                }
             } else if (f.type == "QUuid") {
                 out << indent << f.setterName() << "(" << setterArgs;
                 out << "readUuid(data";
@@ -323,7 +389,10 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
                 if (!f.isStringLength) out << ")";
                 out << ";\n";
             } else {
-                out << indent << f.setterName() << "(" << setterArgs;
+                if (f.isStringLength)
+                    out << indent << "unsigned " << name << " = ";
+                else
+                    out << indent << f.setterName() << "(" << setterArgs;
                 if (f.isEnum)
                     out << "static_cast<" << f.type << ">(";
                 unsigned firstByte = offset / 8;
@@ -342,7 +411,16 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
                 out << ") & 0x" << hex << mask << dec << ")";
                 if (field.attribute("type") == "bool") out << " != 0";
                 if (f.isEnum) out << ")";
-                out << ");\n";
+                if (!f.isStringLength) out << ")";
+                out << ";\n";
+            }
+            // evaluate constraints
+            for (QDomElement child  = field.firstChildElement(); !child.isNull(); child = child.nextSiblingElement()) {
+                if (child.tagName() == "constraint") {
+                    out << indent << "if (!(" << f.getterName() << "(" << setterArgs << ") " << child.attribute("expr") << ")) {\n";
+                    invalidFound(indent + "    ", out, currentOptional);
+                    out << indent << "}\n";
+                }
             }
         }
         offset += bits;
@@ -356,10 +434,10 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
         out << indent << "if (" << field.attribute("predicate") << ") {\n";
 
         offset = 0; dynamicOffset = true;
-        sizeCheck(indent + "    ", out, field.firstChildElement(), offset, dynamicOffset);
+        sizeCheck(indent + "    ", out, field.firstChildElement(), offset, dynamicOffset, currentOptional);
 
         for (QDomElement child = field.firstChildElement(); !child.isNull(); child = child.nextSiblingElement()) {
-            processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap);
+            processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap, setterArgs, currentOptional);
         }
 
         if (offset % 8 != 0)
@@ -368,7 +446,38 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
         if (offset) out << indent << "    curOffset += " << (offset / 8) << ";\n";
         out << indent << "}\n";
         offset = 0;
-        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset);
+        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset, currentOptional);
+    } else if (field.tagName() == "optional") {
+        if (offset % 8 != 0)
+            qFatal("Optionals should always be byte-aligned");
+        if (!dynamicOffset)
+            out << indent << "curOffset = " << (offset / 8) << ";\n";
+        else if (offset) out << indent << "curOffset += " << (offset / 8) << ";\n";
+        offset = 0; dynamicOffset = true;
+
+        QString name = ucFirst(field.attribute("name"));
+        QString label = field.attribute("name") + "Failed";
+        out << indent << "d->has" << name << " = true;\n";
+        out << indent << "unsigned savedOffset" << name << " = curOffset;\n";
+        out << indent << "{\n";
+
+        sizeCheck(indent + "    ", out, field.firstChildElement(), offset, dynamicOffset, name);
+        for (QDomElement child = field.firstChildElement(); !child.isNull(); child = child.nextSiblingElement()) {
+            processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap, setterArgs, name);
+        }
+
+        if (offset % 8 != 0)
+            qFatal("Optionals should contain an integer number of bytes");
+
+        if (offset) out << indent << "    curOffset += " << (offset / 8) << ";\n";
+
+        out << indent << "}\n";
+        out << label << ":\n";
+        out << indent << "if (!d->has" << name << ") {\n";
+        out << indent << "    curOffset = savedOffset" << name << ";\n";
+        out << indent << "}\n";
+        offset = 0;
+        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset, currentOptional);
     } else if (field.tagName() == "choose") {
         if (offset % 8 != 0)
             qFatal("Choose tags should always be byte-aligned");
@@ -392,10 +501,10 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
                 out << "if (" << childField.attribute("predicate") << ") ";
             out << "{\n";
 
-            sizeCheck(indent + "    ", out, childField.firstChildElement(), offset, dynamicOffset);
+            sizeCheck(indent + "    ", out, childField.firstChildElement(), offset, dynamicOffset, currentOptional);
 
             for (QDomElement child = childField.firstChildElement(); !child.isNull(); child = child.nextSiblingElement()) {
-                processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap);
+                processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap, setterArgs, currentOptional);
             }
 
             if (offset % 8 != 0)
@@ -406,7 +515,7 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
             offset = 0;
         }
         out << "\n";
-        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset);
+        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset, currentOptional);
     } else if (field.tagName() == "array") {
         if (offset % 8 != 0)
             qFatal("Arrays should always be byte-aligned");
@@ -431,10 +540,10 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
 
         out << indent << "for (unsigned i = 0, endi = " << length << "; i < endi; ++i) {\n";
         offset = 0; dynamicOffset = true;
-        sizeCheck(indent + "    ", out, field.firstChildElement(), offset, dynamicOffset);
+        sizeCheck(indent + "    ", out, field.firstChildElement(), offset, dynamicOffset, currentOptional);
 
         for (QDomElement child = field.firstChildElement(); !child.isNull(); child = child.nextSiblingElement()) {
-            processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap, setterArgs + "i, ");
+            processFieldElement(indent + "    ", out, child, offset, dynamicOffset, fieldsMap, setterArgs + "i, ", currentOptional);
         }
 
         if (offset % 8 != 0)
@@ -443,7 +552,87 @@ static void processFieldElement(QString indent, QTextStream& out, QDomElement fi
         if (offset) out << indent << "    curOffset += " << (offset / 8) << ";\n";
         out << indent << "}\n";
         offset = 0;
-        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset);
+        sizeCheck(indent, out, field.nextSiblingElement(), offset, dynamicOffset, currentOptional);
+    }
+}
+
+static void processFieldElementForWrite(QString indent, QTextStream& out, QDomElement field, const QMap<QString, Field>& fieldsMap, QString getterArgs = QString())
+{
+    if (field.tagName() == "field") {
+        QString name = field.attribute("name");
+        unsigned bits = field.attribute("size").toUInt();
+        if (!name.startsWith("reserved")) {
+            const Field& f = fieldsMap[name];
+            if (field.attribute("type") == "bytestring") {
+                out << indent << "out.writeByteString(";
+            } else if (field.attribute("type") == "unicodestring") {
+                out << indent << "out.writeUnicodeStringWithFlags(";
+            } else if (field.attribute("type") == "unicodechars") {
+                out << indent << "out.writeUnicodeString(";
+            } else if (f.type == "QByteArray") {
+                out << indent << "out.writeBlob(";
+            } else if (f.type == "QUuid") {
+                out << indent << "// TODO ";
+            } else if (field.attribute("type") == "bool" || field.attribute("type") == "unsigned" || f.isEnum) {
+                out << indent << "out.writeUnsigned(" << bits << ", ";
+            } else if (field.attribute("type") == "signed") {
+                out << indent << "out.writeSigned(" << bits << ", ";
+            } else if (field.attribute("type") == "float") {
+                out << indent << "out.writeFloat(" << bits << ", ";
+            } else if (field.attribute("type") == "fixed") {
+                out << indent << "// TODO ";
+            } else {
+                out << f.type;
+            }
+            if (f.isStringLength) {
+                // TODO: figure out length from string
+                const Field& f2 = fieldsMap[f.lengthFor];
+                out << f2.getterName() << "(" << getterArgs << ").length()";
+            } else {
+                out << f.getterName() << "(" << getterArgs << ")";
+            }
+            out << ");\n";
+        } else {
+            out << indent << "out.writeUnsigned(" << bits << ", 0);\n";
+        }
+    } else if (field.tagName() == "if") {
+        out << indent << "if (" << field.attribute("predicate") << ") {\n";
+        for (QDomElement e = field.firstChildElement(); !e.isNull(); e = e.nextSiblingElement())
+            processFieldElementForWrite(indent + "    ", out, e, fieldsMap);
+        out << indent << "}\n";
+    } else if (field.tagName() == "choose") {
+        bool isFirst = true;
+        for (QDomElement childField = field.firstChildElement(); !childField.isNull(); childField = childField.nextSiblingElement()) {
+            if (isFirst) out << indent; else out << " ";
+            if (!isFirst) out << "else ";
+            isFirst = false;
+            if (childField.hasAttribute("predicate"))
+                out << "if (" << childField.attribute("predicate") << ") ";
+            out << "{\n";
+
+            for (QDomElement child = childField.firstChildElement(); !child.isNull(); child = child.nextSiblingElement()) {
+                processFieldElementForWrite(indent + "    ", out, child, fieldsMap);
+            }
+
+            out << indent << "}";
+        }
+        out << "\n";
+    } else if (field.tagName() == "array") {
+        QString length = field.attribute("length");
+        if (fieldsMap.contains(length))
+            length = fieldsMap[length].getterName() + "()";
+        else
+            length = "d->" + field.firstChildElement().attribute("name") + ".size()";
+
+        out << indent << "for (unsigned i = 0, endi = " << length << "; i < endi; ++i) {\n";
+        for (QDomElement e = field.firstChildElement(); !e.isNull(); e = e.nextSiblingElement())
+            processFieldElementForWrite(indent + "    ", out, e, fieldsMap, "i");
+        out << indent << "}\n";
+    } else if (field.tagName() == "optional") {
+        out << indent << "if (has" << ucFirst(field.attribute("name")) << "()) {\n";
+        for (QDomElement e = field.firstChildElement(); !e.isNull(); e = e.nextSiblingElement())
+            processFieldElementForWrite(indent + "    ", out, e, fieldsMap);
+        out << indent << "}\n";
     }
 }
 
@@ -503,6 +692,11 @@ static void processFieldElementForDump(QString indent, QTextStream& out, QDomEle
         for (QDomElement e = field.firstChildElement(); !e.isNull(); e = e.nextSiblingElement())
             processFieldElementForDump(indent + "    ", out, e, fieldsMap, "i");
         out << indent << "}\n";
+    } else if (field.tagName() == "optional") {
+        out << indent << "if (has" << ucFirst(field.attribute("name")) << "()) {\n";
+        for (QDomElement e = field.firstChildElement(); !e.isNull(); e = e.nextSiblingElement())
+            processFieldElementForDump(indent + "    ", out, e, fieldsMap);
+        out << indent << "}\n";
     }
 }
 
@@ -545,15 +739,23 @@ void processRecordForImplementation(QDomElement e, QTextStream& out)
         else if (!f.isStringLength)
             out << "    " << f.type << " " << f.name << ";\n";
     }
+    // optional groups
+    QDomNodeList optionalNodes = e.elementsByTagName("optional");
+    for (int i = 0; i < optionalNodes.size(); i++) {
+        QDomElement f = optionalNodes.at(i).toElement();
+        out << "    bool has" << ucFirst(f.attribute("name")) << ";\n";
+    }
     out << "};\n\n";
 
     // constructor
-    out << className << "::" << className << "(Workbook *book)\n";
+    out << className << "::" << className << "(Swinder::Workbook *book)\n";
     out << "    : Record(book), d(new Private)\n{\n";
     foreach(const Field& f, fields) {
         if (f.isArray || f.isStringLength) continue;
         QString val;
-        if (f.type == "unsigned" || f.type == "int") {
+        if (!f.defaultValue.isEmpty()) {
+            val = f.defaultValue;
+        } else if (f.type == "unsigned" || f.type == "int") {
             val = "0";
         } else if (f.type == "bool") {
             val = "false";
@@ -561,6 +763,10 @@ void processRecordForImplementation(QDomElement e, QTextStream& out)
         if (!val.isEmpty())
             out << "    " << f.setterName() << "(" << val << ");\n";
     }
+    for (int i = 0; i < optionalNodes.size(); i++) {
+        QDomElement f = optionalNodes.at(i).toElement();
+        out << "    d->has" << ucFirst(f.attribute("name")) << " = false;\n";
+    }
     out << "}\n\n";
 
     // destructor
@@ -619,6 +825,37 @@ void processRecordForImplementation(QDomElement e, QTextStream& out)
         << "    return " << f.attribute("value") << ";\n}\n\n";
     }
 
+    // array lengths
+    QDomNodeList arrayNodes = e.elementsByTagName("array");
+    for (int i = 0; i < arrayNodes.size(); i++) {
+        QDomElement f = arrayNodes.at(i).toElement();
+        if (f.hasAttribute("lengthField")) {
+            QString name = f.attribute("lengthField");
+            QDomNodeList afields = f.elementsByTagName("field");
+            QDomElement af1 = afields.at(0).toElement();
+
+            out << "unsigned " << className << "::" << lcFirst(name)  << "() const\n{\n";
+            out << "    return d->" << af1.attribute("name") << ".size();\n}\n\n";
+
+            out << "void " << className << "::set" << ucFirst(name) << "( unsigned " << name << " )\n{\n";
+            for (int j = 0; j < afields.size(); j++) {
+                QDomElement af = afields.at(j).toElement();
+                if (af.attribute("name").startsWith("reserved")) continue;
+                out << "    d->" << af.attribute("name") << ".resize(" << name << ");\n";
+            }
+            out << "}\n\n";
+        }
+    }
+
+    // optional groups
+    for (int i = 0; i < optionalNodes.size(); i++) {
+        QDomElement f = optionalNodes.at(i).toElement();
+        out << "bool " << className << "::has" << ucFirst(f.attribute("name")) << "() const\n{\n";
+        out << "    return d->has" << ucFirst(f.attribute("name")) << ";\n}\n\n";
+        out << "void " << className << "::setHas" << ucFirst(f.attribute("name")) << "( bool value )\n{\n";
+        out << "    d->has" << ucFirst(f.attribute("name")) << " = value;\n}\n\n";
+    }
+
     // setData method
     bool hasFields = !fieldNodes.isEmpty();
     if (hasFields) {
@@ -627,7 +864,7 @@ void processRecordForImplementation(QDomElement e, QTextStream& out)
         out << "void " << className << "::setData( unsigned size, const unsigned char*, const unsigned int* )\n{\n";
     }
     out << "    setRecordSize(size);\n\n";
-    if (e.elementsByTagName("if").size() > 0 || e.elementsByTagName("array").size() > 0 || containsStrings)
+    if (e.elementsByTagName("if").size() > 0 || e.elementsByTagName("array").size() > 0 || containsStrings || e.elementsByTagName("optional").size() > 0)
         out << "    unsigned curOffset;\n";
     if (containsStrings) {
         out << "    bool stringLengthError = false;\n"
@@ -635,11 +872,21 @@ void processRecordForImplementation(QDomElement e, QTextStream& out)
     }
     unsigned offset = 0;
     bool dynamicOffset = false;
-    sizeCheck("    ", out, e.firstChildElement(), offset, dynamicOffset);
+    sizeCheck("    ", out, e.firstChildElement(), offset, dynamicOffset, "");
     for (QDomElement child = e.firstChildElement(); !child.isNull(); child = child.nextSiblingElement())
         processFieldElement("    ", out, child, offset, dynamicOffset, fieldsMap);
     out << "}\n\n";
 
+    // writeData method
+    if (hasFields) {
+        out << "void " << className << "::writeData( XlsRecordOutputStream& out ) const\n{\n";
+    } else {
+        out << "void " << className << "::writeData( XlsRecordOutputStream& ) const\n{\n";
+    }
+    for (QDomElement child = e.firstChildElement(); !child.isNull(); child = child.nextSiblingElement())
+        processFieldElementForWrite("    ", out, child, fieldsMap);
+    out << "}\n\n";
+
     // dump method
     out << "void " << className << "::dump( std::ostream& out ) const\n{\n"
     << "    out << \"" << e.attribute("name") << "\" << std::endl;\n";
@@ -648,7 +895,7 @@ void processRecordForImplementation(QDomElement e, QTextStream& out)
     out << "}\n\n";
 
     // creator function
-    out << "static Record* create" << className << "(Workbook *book)\n{\n    return new " << className << "(book);\n}\n\n";
+    out << "static Record* create" << className << "(Swinder::Workbook *book)\n{\n    return new " << className << "(book);\n}\n\n";
 }
 
 
@@ -695,7 +942,8 @@ int main(int argc, char** argv)
     cppOut << "// This file was automatically generated from records.xml\n"
     << "#include \"records.h\"\n"
     << "#include <vector>\n"
-    << "#include <iomanip>\n\n"
+    << "#include <iomanip>\n"
+    << "#include \"XlsRecordOutputStream.h\"\n\n"
     << "namespace Swinder {\n\n";
 
     QDomNodeList records = doc.elementsByTagName("record");
