@@ -239,6 +239,12 @@ void Record::setData(unsigned, const unsigned char*, const unsigned int*)
 {
 }
 
+void Record::writeData(XlsRecordOutputStream &out) const
+{
+    Q_UNUSED(out);
+    fprintf(stderr, "ERROR! writeData not implemented for record type %u\n", rtti());
+}
+
 void Record::dump(std::ostream&) const
 {
     // nothing to dump
