@@ -46,8 +46,6 @@
 #include "ColorConversions.h"
 //! @todo only include for TINY target
 
-const int HLSMAX = 255; //!< Used for computing tint
-
 //! @return tinted value for @a color
 //! Alpha value is left unchanged.
 /*! @param color to be converted
@@ -56,6 +54,7 @@ const int HLSMAX = 255; //!< Used for computing tint
 */
 static QColor tintedColor(const QColor& color, qreal tint)
 {
+    const int HLSMAX = 255; // Used for computing tint
 //kDebug() << "rgb:" << color.name() << "tint:" << tint;
     if (tint == 0.0 || !color.isValid()) {
         return color;
@@ -76,6 +75,18 @@ static QColor tintedColor(const QColor& color, qreal tint)
 //kDebug() << "rgb:" << r << g << b << QColor(r, g, b, color.alpha()).name();
     return QColor(r, g, b, color.alpha());
 }
+#if 0
+static QColor tintedColor( const QColor & color, qreal tintfactor )
+{
+    QColor retColor;
+    const qreal  nonTindedPart = 1.0 - tintfactor;
+    const int tintedColor = 255 * nonTindedPart;
+    retColor.setRed( tintedColor + tintfactor * color.red() );
+    retColor.setGreen( tintedColor + tintfactor * color.green() );
+    retColor.setBlue( tintedColor + tintfactor * color.blue() );
+    return retColor;
+}
+#endif
 
 /*! @return color decoded from a "rgb" attribute of the current element
             or invalid QColor when reading was not possible.
@@ -120,7 +131,7 @@ void XlsxColorStyle::clear()
     theme = -1;
 }
 
-bool XlsxColorStyle::isValid(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+bool XlsxColorStyle::isValid(const /*QMap<QString, */MSOOXML::DrawingMLTheme/**>*/ *themes) const
 {
 kDebug() << "indexed:" << indexed << "rgb:" << rgb.name() << "tint:" << tint << "theme:" << theme;
     if (theme >= 0) {
@@ -130,11 +141,11 @@ kDebug() << themeColor(themes).isValid();
     return rgb.isValid();
 }
 
-QColor XlsxColorStyle::themeColor(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+QColor XlsxColorStyle::themeColor(const/* QMap<QString,*/ MSOOXML::DrawingMLTheme*/*> **/themes) const
 {
     Q_ASSERT(themes);
 //! @todo find proper theme, not just any
-    MSOOXML::DrawingMLTheme *themeObject = themes->constBegin().value();
+    const MSOOXML::DrawingMLTheme *themeObject = themes;
 kDebug() << themeObject;
     if (themeObject) {
         MSOOXML::DrawingMLColorSchemeItemBase *colorItemBase = themeObject->colorScheme.value(theme);
@@ -148,7 +159,7 @@ kDebug() << colorItemBase;
     return QColor();
 }
 
-QColor XlsxColorStyle::value(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+QColor XlsxColorStyle::value(const /*QMap<QString,*/ MSOOXML::DrawingMLTheme/**>*/ *themes) const
 {
     QColor realColor;
 kDebug() << "theme:" << theme;
@@ -209,7 +220,7 @@ XlsxBorderStyle::XlsxBorderStyle()
 {
 }
 
-QString XlsxBorderStyle::setupCellStyle(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+QString XlsxBorderStyle::setupCellStyle(const /*QMap<QString, */MSOOXML::DrawingMLTheme/**>*/ *themes) const
 {
     QString styleString = this->style;
 
@@ -256,7 +267,7 @@ XlsxBorderStyles::XlsxBorderStyles()
 {
 }
 
-void XlsxBorderStyles::setupCellStyle(KoGenStyle* cellStyle, const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+void XlsxBorderStyles::setupCellStyle(KoGenStyle* cellStyle, const /*QMap<QString,*/ MSOOXML::DrawingMLTheme/**>*/ *themes) const
 {
 //! @todo simplify if 2 or 4 sides are the same
     QString s;
@@ -283,7 +294,7 @@ void XlsxBorderStyles::setupCellStyle(KoGenStyle* cellStyle, const QMap<QString,
     }
 }
 
-static QColor applyPatternDensity( const XlsxColorStyle& bg, const XlsxColorStyle& fg, qreal percent, const QMap<QString, MSOOXML::DrawingMLTheme*> *themes )
+static QColor applyPatternDensity( const XlsxColorStyle& bg, const XlsxColorStyle& fg, qreal percent, const /*QMap<QString, */MSOOXML::DrawingMLTheme/**>*/ *themes )
 {
     const QColor bgColor = bg.theme >= 0 ? bg.themeColor( themes ) : bg.rgb.isValid() ? bg.rgb : QColor( Qt::white );
     const QColor fgColor = fg.theme >= 0 ? fg.themeColor( themes ) : fg.rgb;//.isValid() ? fg.rgb : QColor( Qt::black );
@@ -304,7 +315,7 @@ static QColor applyPatternDensity( const XlsxColorStyle& bg, const XlsxColorStyl
     return result;
 }
 
-const XlsxColorStyle* XlsxFillStyle::realBackgroundColor( const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+const XlsxColorStyle* XlsxFillStyle::realBackgroundColor( const /*QMap<QString,*/ MSOOXML::DrawingMLTheme/**>*/ *themes) const
 {
     delete cachedRealBackgroundColor;
     cachedRealBackgroundColor = new XlsxColorStyle;
@@ -364,7 +375,7 @@ kDebug() << "patternType:" << patternType;
     return &bgColor;
 }
 
-void XlsxFillStyle::setupCellStyle(KoGenStyle* cellStyle, const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+void XlsxFillStyle::setupCellStyle(KoGenStyle* cellStyle, const /*QMap<QString, */MSOOXML::DrawingMLTheme/**>*/ *themes) const
 {
 //! @todo implement more styling;
 //!       use XlsxColorStyle::automatic, XlsxColorStyle::indexed, XlsxColorStyle::theme...
@@ -499,7 +510,7 @@ void XlsxFontStyle::setupCharacterStyle(KoCharacterStyle* characterStyle) const
 }
 
 void XlsxFontStyle::setupCellTextStyle(
-    const QMap<QString, MSOOXML::DrawingMLTheme*> *themes,
+    const /*QMap<QString,*/ MSOOXML::DrawingMLTheme/**>*/ *themes,
     KoGenStyle* cellStyle) const
 {
     if (!name.isEmpty()) {
@@ -649,7 +660,9 @@ void XlsxCellFormat::setupCellStyleAlignment(KoGenStyle* cellStyle) const
     if (verticalTtb)
         cellStyle->addProperty("style:direction", "ttb");
     else if (textRotation != 0) {
-        //@todo map other cases (to style:text-rotate-angle? that one only allows 0, 90, 270)
+        unsigned angle = textRotation;
+        if (angle > 90) angle = 360 - (angle - 90);
+        cellStyle->addProperty("style:rotation-angle", QString::number(angle));
     }
 
     if (shrinkToFit)
@@ -666,6 +679,8 @@ void XlsxCellFormat::setupCellStyleAlignment(KoGenStyle* cellStyle) const
     case GeneralHorizontalAlignment: // ok?
         if (verticalTtb) // Excel centers vertical text by default, so mimic that
             cellStyle->addProperty("fo:text-align", "center", KoGenStyle::ParagraphType);
+        if (textRotation > 90 && textRotation < 180) // Excel right aligns rotated text for some angles
+            cellStyle->addProperty("fo:text-align", "end", KoGenStyle::ParagraphType);
         break;
     case LeftHorizontalAlignment:
         cellStyle->addProperty("fo:text-align", "start", KoGenStyle::ParagraphType);
@@ -708,7 +723,7 @@ void XlsxCellFormat::setupCellStyleAlignment(KoGenStyle* cellStyle) const
 //! See http://www.w3.org/TR/2001/REC-xsl-20011015/slice7.html#text-align
 bool XlsxCellFormat::setupCellStyle(
     const XlsxStyles *styles,
-    const QMap<QString, MSOOXML::DrawingMLTheme*> *themes,
+    const /*QMap<QString, */MSOOXML::DrawingMLTheme/**>*/ *themes,
     KoGenStyle* cellStyle) const
 {
     kDebug() << "fontId:" << fontId << "fillId:" << fillId << "borderId:" << borderId;
@@ -859,6 +874,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_styleSheet()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(fonts)
             ELSE_TRY_READ_IF(fills)
@@ -867,7 +883,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_styleSheet()
             ELSE_TRY_READ_IF(borders)
 //! @todo add ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -899,11 +914,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fonts()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(font)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -932,12 +947,12 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_numFmts()
     while( !atEnd() )
     {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if( isStartElement() )
         {
             TRY_READ_IF( numFmt )
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF( CURRENT_EL )
     }
 
     READ_EPILOGUE
@@ -974,8 +989,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_numFmt()
 
     while( true )
     {
-        BREAK_IF_END_OF( CURRENT_EL );
         readNext();
+        BREAK_IF_END_OF( CURRENT_EL );
     }
 
     READ_EPILOGUE
@@ -1027,6 +1042,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_font()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(sz)
             ELSE_TRY_READ_IF(name)
@@ -1042,7 +1058,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_font()
             ELSE_TRY_READ_IF(vertAlign)
 //! @todo add ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     READ_EPILOGUE_WITHOUT_RETURN
@@ -1083,8 +1098,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_sz()
     }
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1108,8 +1123,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_vertAlign()
     m_currentFontStyle->vertAlign = ST_VerticalAlignRun( val );
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1132,8 +1147,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_name()
     TRY_READ_ATTR_WITHOUT_NS_INTO(val, m_currentFontStyle->name)
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1158,8 +1173,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_b()
     m_currentFontStyle->bold = readBooleanAttr("val", true);
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1184,8 +1199,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_i()
     m_currentFontStyle->italic = readBooleanAttr("val", true);
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1210,8 +1225,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_strike()
     m_currentFontStyle->strike = readBooleanAttr("val", true);
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1236,8 +1251,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_u()
     m_currentFontStyle->setUnderline(attrs.value("val").toString());
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1279,8 +1294,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_color()
     RETURN_IF_ERROR( m_currentColorStyle->readAttributes(attrs, "color") )
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1318,11 +1333,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_cellXfs()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(xf)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1401,11 +1416,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_xf()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(alignment)
 //! @todo add ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     READ_EPILOGUE_WITHOUT_RETURN
@@ -1451,8 +1466,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_alignment()
 //! @todo more attributes
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1485,11 +1500,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fills()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(fill)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1524,12 +1539,12 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fill()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(gradientFill)
             ELSE_TRY_READ_IF(patternFill)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE_WITHOUT_RETURN
 
@@ -1639,12 +1654,12 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_patternFill()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(bgColor)
             ELSE_TRY_READ_IF(fgColor)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1671,8 +1686,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_bgColor()
     RETURN_IF_ERROR( m_currentFillStyle->bgColor.readAttributes(attrs, "bgColor") )
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1699,8 +1714,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fgColor()
     RETURN_IF_ERROR( m_currentFillStyle->fgColor.readAttributes(attrs, "fgColor") )
 
     while (true) {
-        BREAK_IF_END_OF(CURRENT_EL);
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1728,11 +1743,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_gradientFill()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
 //! @todo            TRY_READ_IF(stop)
 //todo            ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1756,11 +1771,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_borders()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(border)
 //todo            ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1790,6 +1805,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_border()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(bottom)
             ELSE_TRY_READ_IF(diagonal)
@@ -1802,7 +1818,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_border()
 //! @todo (dxf only)            ELSE_TRY_READ_IF(vertical)
 //todo            ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     currentBorderStyleSetter.release();
@@ -1825,6 +1840,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_bottom()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             if (QUALIFIED_NAME_IS(color)) {
                 m_currentColorStyle = &m_currentBorderStyle->bottom.color;
@@ -1833,7 +1849,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_bottom()
             }
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1850,6 +1865,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_top()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             if (QUALIFIED_NAME_IS(color)) {
                 m_currentColorStyle = &m_currentBorderStyle->top.color;
@@ -1858,7 +1874,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_top()
             }
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1874,6 +1889,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_left()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             if (QUALIFIED_NAME_IS(color)) {
                 m_currentColorStyle = &m_currentBorderStyle->left.color;
@@ -1882,7 +1898,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_left()
             }
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1898,6 +1913,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_right()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             if (QUALIFIED_NAME_IS(color)) {
                 m_currentColorStyle = &m_currentBorderStyle->right.color;
@@ -1906,7 +1922,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_right()
             }
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -1922,6 +1937,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_diagonal()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             if (QUALIFIED_NAME_IS(color)) {
                 m_currentColorStyle = &m_currentBorderStyle->diagonal.color;
@@ -1930,7 +1946,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_diagonal()
             }
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
