@@ -37,6 +37,7 @@ class XlsxXmlDocumentReaderContext;
 class XlsxComments;
 class XlsxStyles;
 class XlsxImport;
+class Sheet;
 
 //! A class reading MSOOXML XLSX markup - xl/worksheets/sheet*.xml part.
 class XlsxXmlWorksheetReader : public MSOOXML::MsooXmlCommonReader
@@ -80,8 +81,6 @@ protected:
     int m_currentColumn;
     //! Used in read_f() and read_v()
     QString m_value;
-    //! Filled by read_sheetFormatPr(), measured in pt
-    QString m_defaultRowHeight;
 
     // for optimization
     static const char* officeValue;
@@ -117,18 +116,20 @@ public:
         const QString& _worksheetName,
         const QString& _state,
         const QString _path, const QString _file,
-        const QMap<QString, MSOOXML::DrawingMLTheme*>& _themes,
+        /*QMap<QString, */MSOOXML::DrawingMLTheme*/*>*/& _themes,
         const XlsxSharedStringVector& _sharedStrings,
         const XlsxComments& _comments,
         const XlsxStyles& _styles,
         MSOOXML::MsooXmlRelationships& _relationships,
         XlsxImport* _import,
         int& numberOfOleObjects);
+    virtual ~XlsxXmlWorksheetReaderContext();
 
+    Sheet* sheet;
     const uint worksheetNumber;
     QString worksheetName;
     QString state;
-    const QMap<QString, MSOOXML::DrawingMLTheme*>* themes;
+    /*QMap<QString, */MSOOXML::DrawingMLTheme*/*>**/ themes;
     const XlsxSharedStringVector *sharedStrings;
     const XlsxComments* comments;
     const XlsxStyles* styles;
