@@ -111,6 +111,7 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read(MSOOXML::MsooXmlReaderConte
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(styles)
         if (isStartElement()) {
             TRY_READ_IF(docDefaults)
             ELSE_TRY_READ_IF(style)
@@ -119,7 +120,6 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read(MSOOXML::MsooXmlReaderConte
             }
             //! @todo add ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(styles)
     }
 
     if (!expectElEnd("w:styles")) {
@@ -174,12 +174,12 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_docDefaults()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(pPrDefault)
             ELSE_TRY_READ_IF(rPrDefault)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     m_defaultStyle = m_currentTextStyle;
     m_currentTextStyle = KoGenStyle();
@@ -203,11 +203,11 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_pPrDefault()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(pPr)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -229,11 +229,11 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_rPrDefault()
     while (!atEnd()) {
         readNext();
         kDebug() << *this;
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF_IN_CONTEXT(rPr)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -302,8 +302,8 @@ static QString ST_StyleType_to_ODF(const QString& type)
  - rPr (Run Properties) §17.7.9.1
  - rsid (Revision Identifier for Style Definition) §17.7.4.15
  - semiHidden (Hide Style From Main User Interface) §17.7.4.16
- - tblPr (Style Table Properties) §17.7.6.4
- - tblStylePr (Style Conditional Table Formatting Properties) §17.7.6.6
+ - [done] tblPr (Style Table Properties) §17.7.6.4
+ - [done] tblStylePr (Style Conditional Table Formatting Properties) §17.7.6.6
  - tcPr (Style Table Cell Properties) §17.7.6.9
  - trPr (Style Table Row Properties) §17.7.6.11
  - uiPriority (Optional User Interface Sorting Order) §17.7.4.19
@@ -378,12 +378,14 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_style()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             const QXmlStreamAttributes attrs(attributes());
             TRY_READ_IF(name)
             ELSE_TRY_READ_IF_IN_CONTEXT(rPr)
             ELSE_TRY_READ_IF(pPr)
             ELSE_TRY_READ_IF(tblPr)
+            ELSE_TRY_READ_IF(tblStylePr)
             else if (QUALIFIED_NAME_IS(basedOn)) {
                 READ_ATTR(val)
                 if (type == "character") {
@@ -398,7 +400,6 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_style()
             }
             //! @todo add ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     KoGenStyles::InsertionFlags insertionFlags = KoGenStyles::DontAddNumberToName;
     if (styleName.isEmpty()) {
@@ -468,6 +469,31 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_style()
     READ_EPILOGUE
 }
 
+#undef CURRENT_EL
+#define CURRENT_EL tblStylePr
+//! tbleStylePr (table style handler)
+KoFilter::ConversionStatus DocxXmlStylesReader::read_tblStylePr()
+{
+    READ_PROLOGUE
+    const QXmlStreamAttributes attrs(attributes());
+
+    TRY_READ_ATTR(type)
+    if (!type.isEmpty()) {
+
+    }
+
+    while (!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
+        if (isStartElement()) {
+            //TRY_READ_IF(name)
+            //! @todo add ELSE_WRONG_FORMAT
+        }
+    }
+
+    READ_EPILOGUE
+}
+
 #undef CURRENT_EL
 #define CURRENT_EL name
 //! 17.7.4.9 name (Primary Style Name)
