@@ -68,6 +68,10 @@ QString pt(double v) {
 QString percent(double v) {
     return format(v) + '%';
 }
+QString mm(double v) {
+    static const QString mm("mm");
+    return format(v) + mm;
+}
 }
 
 /*
@@ -75,7 +79,7 @@ QString percent(double v) {
  * Drawing Writer
  * ************************************************
  */
-DrawingWriter::DrawingWriter(KoXmlWriter& xmlWriter, KoGenStyles& kostyles, 
+DrawingWriter::DrawingWriter(KoXmlWriter& xmlWriter, KoGenStyles& kostyles,
                              bool stylesxml_, bool inlineObj)
         : Writer(xmlWriter, kostyles, stylesxml_),
           xLeft(0),
@@ -92,22 +96,22 @@ DrawingWriter::DrawingWriter(KoXmlWriter& xmlWriter, KoGenStyles& kostyles,
     scaleY = 25.4 / 1440;
 }
 
-QString DrawingWriter::vLength()
+qreal DrawingWriter::vLength()
 {
     return Writer::vLength(yBottom - yTop);
 }
 
-QString DrawingWriter::hLength()
+qreal DrawingWriter::hLength()
 {
     return Writer::hLength(xRight - xLeft);
 }
 
-QString DrawingWriter::vOffset()
+qreal DrawingWriter::vOffset()
 {
     return Writer::vOffset(yTop);
 }
 
-QString DrawingWriter::hOffset()
+qreal DrawingWriter::hOffset()
 {
     return Writer::hOffset(xLeft);
 }
@@ -122,11 +126,11 @@ void DrawingWriter::SetRectangle(wvWare::Word97::FSPA& spa)
 
 void DrawingWriter::SetGroupRectangle(MSO::OfficeArtFSPGR& fspgr)
 {
-    if(fspgr.xRight == fspgr.xLeft) {
+    if (fspgr.xRight == fspgr.xLeft) {
         return;
     }
 
-    if(fspgr.yBottom == fspgr.yTop)
+    if (fspgr.yBottom == fspgr.yTop)
         return;
 
     xOffset = xOffset + xLeft*scaleX;
@@ -187,35 +191,32 @@ void KWordGraphicsHandler::init(Drawings * pDrawings, const wvWare::Word97::FIB
     //to access FIB first in mswordodfimport.cpp
     parseOfficeArtContainer(m_document->storage(), fib);
 
+    //create default GraphicStyle using information from OfficeArtDggContainer
+    defineDefaultGraphicStyle(m_mainStyles);
+
+    //parse and store floating pictures  
     parseFloatingPictures();
-
     m_picNames = createFloatingPictures(m_store, m_manifestWriter);
 
-//     create default draw style
-//    KoGenStyle style(KoGenStyle::GraphicStyle, "graphic");
-//    style.setDefaultStyle(true);
-//
-//    defineDefaultGraphicProperties(&style,pDrawings);
-//
-//    //add default draw style to styles
-//    m_mainStyles->insert(style, "");
-
     m_drawings = pDrawings;
     m_fib = const_cast<wvWare::Word97::FIB *>(&fib);
 
-    //provide the backgroud color information to the Document
+    //Provide the backgroud color information to the Document, if present.
     DrawStyle ds = getDrawingStyle();
-    MSO::OfficeArtCOLORREF fc = ds.fillColor();
-    QColor color(fc.red, fc.green, fc.blue);
-    m_document->addBgColor(color.name());
-
+    if (ds.fFilled()) {
+        MSO::OfficeArtCOLORREF fc = ds.fillColor();
+        QColor color = QColor(fc.red, fc.green, fc.blue);
+        QString tmp = color.name();
+        if (tmp != m_document->currentBgColor()) {
+            m_document->updateBgColor(tmp);
+        }
+    }
     return;
 }
 
 void KWordGraphicsHandler::handleInlineObject(const wvWare::PictureData& data)
 {
     kDebug(30513) ;
-
     // going to parse and process the Data stream content
     LEInputStream* in = m_document->data_stream();
     int size = (data.picf->lcb - data.picf->cbHeader);
@@ -271,14 +272,14 @@ void KWordGraphicsHandler::handleInlineObject(const wvWare::PictureData& data)
     out.m_rgbUid = ref.uid;
 
     OfficeArtSpContainer* o = &(co.shape);
-    processObjectContent(*o, out);
+    processDrawingObject(*o, out);
 }
 
 void KWordGraphicsHandler::handleFloatingObject(unsigned int globalCP)
 {
     kDebug(30513) << "globalCP" << globalCP ;
     // draw shape or group of shapes
-    if(m_drawings == NULL) {
+    if (m_drawings == NULL) {
         return;
     }
 
@@ -287,15 +288,16 @@ void KWordGraphicsHandler::handleFloatingObject(unsigned int globalCP)
         PLCFIterator<Word97::FSPA> it(fspa->at(0));
 
         //search for drawing in main body
-        for(size_t i = 0; i < fspa->count(); i++, ++it) {
+        for (size_t i = 0; i < fspa->count(); i++, ++it) {
             kDebug(30513) << "FSPA start:" << it.currentStart();
             kDebug(30513) << "FSPA spid:" << it.current()->spid;
 
-            if(it.currentStart() == globalCP) {
-                DrawingWriter out(*m_bodyWriter,*m_mainStyles,true, false);
+            if (it.currentStart() == globalCP) {
+                DrawingWriter out(*m_bodyWriter, *m_mainStyles, true, false);
                 out.m_pSpa = it.current();
                 out.m_bodyDrawing = true;
-                drawObject((uint) it.current()->spid, m_pOfficeArtBodyDgContainer, out,it.current());
+                locateDrawing((m_pOfficeArtBodyDgContainer->groupShape).data(), out,
+			      it.current(), (uint) it.current()->spid);
                 return;
             }
         }
@@ -305,15 +307,16 @@ void KWordGraphicsHandler::handleFloatingObject(unsigned int globalCP)
     if (fspa != 0) {
         PLCFIterator<Word97::FSPA> itHeader(fspa->at(0));
         //search for drawing in header
-        for(size_t i = 0; i < fspa->count(); i++, ++itHeader) {
+        for (size_t i = 0; i < fspa->count(); i++, ++itHeader) {
             kDebug(30513) << "FSPA start:" << itHeader.currentStart() + m_fib->ccpText + m_fib->ccpFtn;
             kDebug(30513) << "FSPA spid:" << itHeader.current()->spid;
 
-            if((itHeader.currentStart() + m_fib->ccpText + m_fib->ccpFtn)  == globalCP) {
-                DrawingWriter out(*m_bodyWriter,*m_mainStyles,true, false);
+            if ((itHeader.currentStart() + m_fib->ccpText + m_fib->ccpFtn)  == globalCP) {
+                DrawingWriter out(*m_bodyWriter, *m_mainStyles, true, false);
                 out.m_pSpa = itHeader.current();
                 out.m_bodyDrawing = false;
-                drawObject((uint) itHeader.current()->spid, m_pOfficeArtHeaderDgContainer, out,itHeader.current());
+                locateDrawing((m_pOfficeArtHeaderDgContainer->groupShape).data(), out,
+                              itHeader.current(), (uint) itHeader.current()->spid);
                 return;
             }
         }
@@ -327,45 +330,57 @@ void KWordGraphicsHandler::setBodyWriter(KoXmlWriter* writer)
 
 DrawStyle KWordGraphicsHandler::getDrawingStyle()
 {
-    if(m_pOfficeArtBodyDgContainer != NULL) {
-        if(m_pOfficeArtBodyDgContainer->shape.isNull() == false) {
-            if((*m_pOfficeArtBodyDgContainer->shape).shapePrimaryOptions.isNull() == false ) {
-                return DrawStyle(m_OfficeArtDggContainer,NULL,m_pOfficeArtBodyDgContainer->shape.data());
-                }
+    if (m_pOfficeArtBodyDgContainer != NULL) {
+        if (m_pOfficeArtBodyDgContainer->shape.isNull() == false) {
+            if ((*m_pOfficeArtBodyDgContainer->shape).shapePrimaryOptions.isNull() == false) {
+                return DrawStyle(m_OfficeArtDggContainer, NULL, m_pOfficeArtBodyDgContainer->shape.data());
             }
         }
+    }
     return DrawStyle(m_OfficeArtDggContainer);
 }
 
-void KWordGraphicsHandler::drawObject(uint spid, MSO::OfficeArtDgContainer * dg, DrawingWriter& out
-        , wvWare::Word97::FSPA* spa)
+void KWordGraphicsHandler::locateDrawing(const MSO::OfficeArtSpgrContainer* spgr, DrawingWriter& out, 
+                                         wvWare::Word97::FSPA* spa, uint spid)
 {
-    if(dg == NULL || dg->groupShape == NULL)
+    if (spgr == NULL) {
         return;
-
+    }
     m_zIndex = 0;
 
-    foreach(const OfficeArtSpgrContainerFileBlock& co, dg->groupShape->rgfb) {
-        //if spgr is in root, find out if his first item is sp with right spid
-        if (co.anon.is<OfficeArtSpgrContainer>()) {
-            const OfficeArtSpContainer* first =
-                    (*co.anon.get<OfficeArtSpgrContainer>()).rgfb[0].anon.get<OfficeArtSpContainer>();
+    //FIXME: combine childAnchor, shapeGroup coordinates with information from
+    //clientAnchor pointing to the SPA structure!
 
+    //NOTE: The OfficeArtSpgrContainer record specifies a container for groups
+    //(4) of shapes.  The group (4) container contains a variable number of
+    //shape containers and other group (4) containers.  Each group (4) is a
+    //shape.  The first container MUST be an OfficeArtSpContainer record, which
+    //MUST contain shape information for the group. MS-ODRAW, 2.2.16
+
+    foreach (const OfficeArtSpgrContainerFileBlock& co, spgr->rgfb) {
+
+        if (co.anon.is<OfficeArtSpgrContainer>()) {
+            const OfficeArtSpContainer* first = 
+                (*co.anon.get<OfficeArtSpgrContainer>()).rgfb[0].anon.get<OfficeArtSpContainer>();
             if (first && first->shapeProp.spid == spid) {
                 out.SetRectangle(*spa);
-                processObjectContent(*co.anon.get<OfficeArtSpgrContainer>(), out); //draw group
+                processGroup(*co.anon.get<OfficeArtSpgrContainer>(), out);
                 break;
             } else {
                 m_zIndex = m_zIndex + (*co.anon.get<OfficeArtSpgrContainer>()).rgfb.size();
             }
-        }
-        else //if sp is in root, find out if it has the right spid
-        {
-            const OfficeArtSpContainer & spCo = *co.anon.get<OfficeArtSpContainer>();
-
-            if (spCo.shapeProp.spid == spid) {
+        } else {
+            const OfficeArtSpContainer &sp = *co.anon.get<OfficeArtSpContainer>();
+            if (sp.shapeProp.fGroup) {
+		if (sp.shapeGroup) {
+                    out.SetGroupRectangle(*sp.shapeGroup);
+                }
+                if (sp.shapeProp.spid == spid) {
+                    kDebug(30513) << "An unprocessed shape storing information for the group is referred from text!";
+                }
+            } else if (sp.shapeProp.spid == spid) {
                 out.SetRectangle(*spa);
-                processObjectContent(spCo, out); //draw object
+                processDrawingObject(sp, out);
                 break;
             }
             m_zIndex++;
@@ -373,34 +388,35 @@ void KWordGraphicsHandler::drawObject(uint spid, MSO::OfficeArtDgContainer * dg,
     }
 }
 
-void KWordGraphicsHandler::processObjectContent(const MSO::OfficeArtSpgrContainer& o, DrawingWriter& out)
+void KWordGraphicsHandler::processGroup(const MSO::OfficeArtSpgrContainer& o, DrawingWriter& out)
 {
-    if (o.rgfb.size() < 2) return;
-
+    if (o.rgfb.size() < 2) {
+        return;
+    }
+    //TODO: create corresponding style and apply style properties
     out.xml.startElement("draw:g");
-
     const OfficeArtSpContainer *first = o.rgfb[0].anon.get<OfficeArtSpContainer>();
-    if(first && first->shapeGroup) {
-        out.SetGroupRectangle(*first->shapeGroup); //set group rectangle
+
+    if (first && first->shapeGroup) {
+        //process shape information for the group
+        out.SetGroupRectangle(*first->shapeGroup);
     }
 
     for (int i = 1; i < o.rgfb.size(); ++i) {
-        if(o.rgfb[i].anon.is<OfficeArtSpContainer>()) {
-
-            OfficeArtSpContainer tempSp = *o.rgfb[i].anon.get<OfficeArtSpContainer>();
-
-            if(tempSp.childAnchor) {
-                out.SetClientRectangle(*tempSp.childAnchor); //set child rectangle
+        if (o.rgfb[i].anon.is<OfficeArtSpContainer>()) {
+            OfficeArtSpContainer sp = *o.rgfb[i].anon.get<OfficeArtSpContainer>();
+            if (sp.childAnchor) {
+                out.SetClientRectangle(*sp.childAnchor); //set child rectangle
             }
-
-            processObjectContent(tempSp, out); //draw objects
-        }
+            processDrawingObject(sp, out); //draw objects
+        } 
+	//TODO: another group shape can be here! We should call locateDrawing
+	//again!
     }
-
     out.xml.endElement(); // draw:g
 }
 
-void KWordGraphicsHandler::processObjectContent(const MSO::OfficeArtSpContainer& o, DrawingWriter out)
+void KWordGraphicsHandler::processDrawingObject(const MSO::OfficeArtSpContainer& o, DrawingWriter out)
 {
     kDebug(30513);
 
@@ -413,7 +429,7 @@ void KWordGraphicsHandler::processObjectContent(const MSO::OfficeArtSpContainer&
     kDebug(30513) << "pib: " << ds.pib();
 
     // textbox can be msosptTextBox or msosptRectangle or ...
-    if(!o.clientTextbox.isNull()) {
+    if (!o.clientTextbox.isNull()) {
         kDebug(30513)<< "processing text box";
         parseTextBox(o, out);
         return;
@@ -454,7 +470,7 @@ void KWordGraphicsHandler::parseOfficeArtContainer(POLE::Storage* storage, const
 {
     kDebug(30513);
     // get OfficeArtContent
-    if(fib.lcbDggInfo != 0) {
+    if (fib.lcbDggInfo != 0) {
 
 	const std::string table = fib.fWhichTblStm ? "1Table" : "0Table";
         POLE::Stream stream(storage, table);
@@ -478,14 +494,14 @@ void KWordGraphicsHandler::parseOfficeArtContainer(POLE::Storage* storage, const
             parseOfficeArtDggContainer(in, m_OfficeArtDggContainer);
         }
         catch (IOException e) {
-            kDebug(30513) << "caught IOException while parsing parseOfficeArtDggContainer ";
+            kDebug(30513) << "caught IOException while parsing parseOfficeArtDggContainer";
             return;
         }
         catch (...) {
             kDebug(30513) << "caught unknown exception while parsing parseOfficeArtDggContainer";
             return;
         }
-        kDebug(30513) << "OfficeArtDggContainer parsed successful " ;
+        kDebug(30513) << "OfficeArtDggContainer parsed successfully" ;
 
         // parse drawingsVariable from msdoc
         // 0 - next OfficeArtDgContainer belongs to Main document;
@@ -504,10 +520,10 @@ void KWordGraphicsHandler::parseOfficeArtContainer(POLE::Storage* storage, const
         }
 
         //parse OfficeArfDgContainer from msdoc
-        OfficeArtDgContainer * pDgContainer = NULL;
+        OfficeArtDgContainer *pDgContainer = NULL;
         try {
             pDgContainer = new OfficeArtDgContainer();
-            if(drawingsVariable == 0) {
+            if (drawingsVariable == 0) {
                 m_pOfficeArtBodyDgContainer = pDgContainer;
             } else {
                 m_pOfficeArtHeaderDgContainer = pDgContainer;
@@ -543,15 +559,14 @@ void KWordGraphicsHandler::parseOfficeArtContainer(POLE::Storage* storage, const
         pDgContainer = NULL;
         try {
             pDgContainer = new OfficeArtDgContainer();
-            if(drawingsVariable == 0) {
-                if(m_pOfficeArtBodyDgContainer!= NULL){
+            if (drawingsVariable == 0) {
+                if (m_pOfficeArtBodyDgContainer != NULL){
                     delete m_pOfficeArtBodyDgContainer;
                 }
                 m_pOfficeArtBodyDgContainer = pDgContainer;
             }
-            else
-            {
-                if(m_pOfficeArtHeaderDgContainer != NULL) {
+            else {
+                if (m_pOfficeArtHeaderDgContainer != NULL) {
                     delete m_pOfficeArtHeaderDgContainer;
                 }
                 m_pOfficeArtHeaderDgContainer = pDgContainer;
@@ -577,6 +592,16 @@ void KWordGraphicsHandler::parseOfficeArtContainer(POLE::Storage* storage, const
     return;
 }
 
+void KWordGraphicsHandler::defineDefaultGraphicStyle(KoGenStyles* styles)
+{
+    // write style <style:default-style style:family="graphic">
+    KoGenStyle style(KoGenStyle::GraphicStyle, "graphic");
+    DrawStyle ds(m_OfficeArtDggContainer);
+    style.setDefaultStyle(true);
+    defineGraphicProperties(style, ds);
+    styles->insert(style);
+}
+
 void KWordGraphicsHandler::defineGraphicProperties(KoGenStyle& style, const DrawStyle& ds,
                                                   const QString& listStyle)
 {
@@ -624,21 +649,22 @@ void KWordGraphicsHandler::defineGraphicProperties(KoGenStyle& style, const Draw
     // draw:end-guide
     // draw:end-line-spacing-horizontal
     // draw:end-line-spacing-vertical
-    // draw:fill ("bitmap", "gradient", "hatch", "none" or "solid")
-    qint32 fillType = ds.fillType();
+
+    // NOTE: fFilled specifies whether fill of the shape is render based on the
+    // properties of the "fill style" property set.
     if (ds.fFilled()) {
+        qint32 fillType = ds.fillType();
+        // draw:fill ("bitmap", "gradient", "hatch", "none" or "solid")
         style.addProperty("draw:fill", getFillType(fillType), gt);
+        // NOTE: only set the color if the fill type is 'solid' because OOo
+        // ignores fill='none' if the color is set
+        if (fillType == 0) {
+            clr = ds.fillColor();
+            style.addProperty("draw:fill-color", QColor(clr.red, clr.green, clr.blue).name(), gt);
+        }
     } else {
         style.addProperty("draw:fill", "none", gt);
     }
-    // draw:fill-color
-    // NOTE: only set the color if the fill type is 'solid' because OOo ignores
-    // fill='none' if the color is set
-    if (fillType == 0) {
-        clr = ds.fillColor();
-        QColor fillColor(clr.red,clr.green,clr.blue);
-        style.addProperty("draw:fill-color", fillColor.name(), gt);
-    }
     // draw:fill-gradient-name
     // draw:fill-hatch-name
     // draw:fill-hatch-solid
@@ -691,15 +717,30 @@ void KWordGraphicsHandler::defineGraphicProperties(KoGenStyle& style, const Draw
     // draw:placing
     // draw:red
     // draw:secondary-fill-color
-    // draw:shadow
-    // draw:shadow-color
-    // draw:shadow-offset-x
-    style.addProperty("draw:shadow-offset-x", pt(ds.shadowOffsetX()/12700.),gt);
-    // draw:shadow-offset-y
-    style.addProperty("draw:shadow-offset-y", pt(ds.shadowOffsetY()/12700.),gt);
-    // draw:shadow-opacity
-    float shadowOpacity = toQReal(ds.shadowOpacity());
-    style.addProperty("draw:shadow-opacity", percent(100*shadowOpacity), gt);
+
+    // NOTE: fShadow property specifies whether the shape has a shadow.
+    if (ds.fShadow()) {
+        // draw:shadow
+        style.addProperty("draw:shadow", "visible", gt);
+        // draw:shadow-color
+        clr = ds.shadowColor();
+        style.addProperty("draw:shadow-color", QColor(clr.red, clr.green, clr.blue).name(), gt);
+        //shadowOffset* properties MUST exist if shadowType property equals
+        //msoshadowOffset or msoshadowDouble, otherwise MUST be ignored,
+        //MS-ODRAW 2.3.13.6
+        quint32 type = ds.shadowType();
+        if ((type == 0) || (type == 1)) {
+            // draw:shadow-offset-x
+            style.addProperty("draw:shadow-offset-x", pt(ds.shadowOffsetX()/12700.), gt);
+            // draw:shadow-offset-y
+            style.addProperty("draw:shadow-offset-y", pt(ds.shadowOffsetY()/12700.), gt);
+        }
+        // draw:shadow-opacity
+        float shadowOpacity = toQReal(ds.shadowOpacity());
+        style.addProperty("draw:shadow-opacity", percent(100*shadowOpacity), gt);
+    } else {
+        style.addProperty("draw:shadow", "hidden");
+    }
     // draw:show-unit
     // draw:start-guide
     // draw:start-line-spacing-horizontal
@@ -767,7 +808,6 @@ void KWordGraphicsHandler::defineGraphicProperties(KoGenStyle& style, const Draw
     // style:rel-height
     // style:rel-width
     // style:repeat
-    // style:shadow
     // svg:fill-rule
     // svg:height
     // svg:stroke-color from 2.3.8.1 lineColor
@@ -937,7 +977,7 @@ void KWordGraphicsHandler::defineWrappingProperties(KoGenStyle& style, const Dra
     // style:wrap-dynamic-treshold
 }
 
-void KWordGraphicsHandler::SetAnchorTypeAtribute(DrawingWriter& out)
+void KWordGraphicsHandler::SetAnchorTypeAttribute(DrawingWriter& out)
 {
     if (out.m_inline) {
         out.xml.addAttribute("text:anchor-type","as-char");
@@ -946,7 +986,7 @@ void KWordGraphicsHandler::SetAnchorTypeAtribute(DrawingWriter& out)
     }
 }
 
-void KWordGraphicsHandler::SetZIndexAtribute(DrawingWriter& out)
+void KWordGraphicsHandler::SetZIndexAttribute(DrawingWriter& out)
 {
     out.xml.addAttribute("draw:z-index",m_zIndex);
 }
@@ -969,29 +1009,29 @@ void KWordGraphicsHandler::parseTextBox(const MSO::OfficeArtSpContainer& o, Draw
 
     out.xml.startElement("draw:frame");
     out.xml.addAttribute("draw:style-name", styleName);
-    SetAnchorTypeAtribute(out);
-    SetZIndexAtribute(out);
+    SetAnchorTypeAttribute(out);
+    SetZIndexAttribute(out);
 
     switch(drawStyle.txflTextFlow()) {
     case 1: //msotxflTtoBA up-down
     case 3: //msotxflTtoBN up-down
     case 5: //msotxflVertN up-down
-        out.xml.addAttribute("svg:width", out.vLength());
-        out.xml.addAttribute("svg:height",out.hLength());
+        out.xml.addAttribute("svg:width", mm(out.vLength()));
+        out.xml.addAttribute("svg:height", mm(out.hLength()));
         out.xml.addAttribute("draw:transform","matrix(0 1 -1 0 " +
-                ((Writer *)&out)->hOffset(out.xRight) + " " + out.vOffset() + ")");
+                mm(((Writer *)&out)->hOffset(out.xRight)) + " " + mm(out.vOffset()) + ")");
         break;
     case 2: //msotxflBtoT down-up
-        out.xml.addAttribute("svg:width", out.vLength());
-        out.xml.addAttribute("svg:height",out.hLength());
+        out.xml.addAttribute("svg:width", mm(out.vLength()));
+        out.xml.addAttribute("svg:height", mm(out.hLength()));
         out.xml.addAttribute("draw:transform","matrix(0 -1 1 0 " +
-               out.hOffset() + " " + ((Writer *)&out)->vOffset(out.yBottom) + ")");
+               mm(out.hOffset()) + " " + mm(((Writer *)&out)->vOffset(out.yBottom)) + ")");
          break;
     default : //standard text flow
-        out.xml.addAttribute("svg:width", out.hLength());
-        out.xml.addAttribute("svg:height", out.vLength());
-        out.xml.addAttribute("svg:x", out.hOffset());
-        out.xml.addAttribute("svg:y", out.vOffset());
+        out.xml.addAttribute("svg:width", mm(out.hLength()));
+        out.xml.addAttribute("svg:height", mm(out.vLength()));
+        out.xml.addAttribute("svg:x", mm(out.hOffset()));
+        out.xml.addAttribute("svg:y", mm(out.vOffset()));
     }
 
     out.xml.startElement("draw:text-box");
@@ -999,7 +1039,7 @@ void KWordGraphicsHandler::parseTextBox(const MSO::OfficeArtSpContainer& o, Draw
     emit textBoxFound(o.shapeProp.spid , out.m_bodyDrawing);
 
     out.xml.endElement(); //draw:text-box
-    out.xml.endElement(); // draw:frame
+    out.xml.endElement(); //draw:frame
 }
 
 void KWordGraphicsHandler::processRectangle(const MSO::OfficeArtSpContainer& o,DrawingWriter& out)
@@ -1019,17 +1059,17 @@ void KWordGraphicsHandler::processRectangle(const MSO::OfficeArtSpContainer& o,D
 
     out.xml.startElement("draw:frame");
     out.xml.addAttribute("draw:style-name", styleName);
-    SetAnchorTypeAtribute(out);
-    SetZIndexAtribute(out);
+    SetAnchorTypeAttribute(out);
+    SetZIndexAttribute(out);
     out.xml.addAttribute("draw:layer", "layout");
-    out.xml.addAttribute("svg:width", out.hLength());
-    out.xml.addAttribute("svg:height", out.vLength());
-    out.xml.addAttribute("svg:x", out.hOffset());
-    out.xml.addAttribute("svg:y", out.vOffset());
+    out.xml.addAttribute("svg:width", mm(out.hLength()));
+    out.xml.addAttribute("svg:height", mm(out.vLength()));
+    out.xml.addAttribute("svg:x", mm(out.hOffset()));
+    out.xml.addAttribute("svg:y", mm(out.vOffset()));
 
     out.xml.startElement("draw:text-box");
-    out.xml.endElement(); // draw:text-box
-    out.xml.endElement(); // draw:frame
+    out.xml.endElement(); //draw:text-box
+    out.xml.endElement(); //draw:frame
 }
 
 void KWordGraphicsHandler::processInlinePictureFrame(const MSO::OfficeArtSpContainer& o, DrawingWriter& out)
@@ -1056,7 +1096,7 @@ void KWordGraphicsHandler::processInlinePictureFrame(const MSO::OfficeArtSpConta
     out.xml.startElement("draw:frame");
     if (url.isEmpty()) {
         // if the image cannot be found, just place an empty frame
-        out.xml.endElement(); // frame
+        out.xml.endElement(); //draw:frame
         return;
     }
     out.xml.addAttribute("draw:style-name", styleName);
@@ -1075,8 +1115,8 @@ void KWordGraphicsHandler::processInlinePictureFrame(const MSO::OfficeArtSpConta
     out.xml.addAttribute("xlink:type", "simple");
     out.xml.addAttribute("xlink:show", "embed");
     out.xml.addAttribute("xlink:actuate", "onLoad");
-    out.xml.endElement(); // image
-    out.xml.endElement(); // frame
+    out.xml.endElement(); //draw:image
+    out.xml.endElement(); //draw:frame
     return;
 }
 
@@ -1113,24 +1153,24 @@ void KWordGraphicsHandler::processFloatingPictureFrame(const MSO::OfficeArtSpCon
     }
     out.xml.startElement("draw:frame");
     if (url.isEmpty()) {
-        // if the image cannot be found, just place an empty frame
-        out.xml.endElement(); // frame
+        //if the image cannot be found, just place an empty frame
+        out.xml.endElement(); //draw:frame
         return;
     }
     out.xml.addAttribute("draw:style-name", styleName);
     out.xml.addAttribute("text:anchor-type","char");
-    SetZIndexAtribute(out);
-    out.xml.addAttribute("svg:width", out.hLength());
-    out.xml.addAttribute("svg:height", out.vLength());
-    out.xml.addAttribute("svg:x", out.hOffset());
-    out.xml.addAttribute("svg:y", out.vOffset());
+    SetZIndexAttribute(out);
+    out.xml.addAttribute("svg:width", mm(out.hLength()));
+    out.xml.addAttribute("svg:height", mm(out.vLength()));
+    out.xml.addAttribute("svg:x", mm(out.hOffset()));
+    out.xml.addAttribute("svg:y", mm(out.vOffset()));
 
     out.xml.startElement("draw:image");
     out.xml.addAttribute("xlink:href", url);
     out.xml.addAttribute("xlink:type", "simple");
     out.xml.addAttribute("xlink:show", "embed");
     out.xml.addAttribute("xlink:actuate", "onLoad");
-    out.xml.endElement(); // image
+    out.xml.endElement(); //draw:image
 
     //check for user edited wrap points
     if (ds.fEditedWrap()) {
@@ -1143,7 +1183,7 @@ void KWordGraphicsHandler::processFloatingPictureFrame(const MSO::OfficeArtSpCon
             LEInputStream in(&buf);
             PWrapPolygonVertices_complex _v;
             parsePWrapPolygonVertices_complex(in, _v);
-                buf.close();
+            buf.close();
 
             //_v.data is an array of POINTs, MS-ODRAW, page 89
             QByteArray a, a2;
@@ -1166,10 +1206,10 @@ void KWordGraphicsHandler::processFloatingPictureFrame(const MSO::OfficeArtSpCon
         }
         out.xml.startElement("draw:contour-polygon");
         out.xml.addAttribute("draw:points", points);
-        out.xml.endElement(); // contour-polygon
+        out.xml.endElement(); //draw:contour-polygon
         delete data;
     }
-    out.xml.endElement(); // frame
+    out.xml.endElement(); //draw:frame
     return;
 }
 
@@ -1222,8 +1262,8 @@ void KWordGraphicsHandler::processLineShape(const MSO::OfficeArtSpContainer& o,
     //create a custom shape
     out.xml.startElement("draw:custom-shape");
     out.xml.addAttribute("draw:style-name", styleName);
-    SetAnchorTypeAtribute(out);
-    SetZIndexAtribute(out);
+    SetAnchorTypeAttribute(out);
+    SetZIndexAttribute(out);
 
     QString height = QString::number(ds.dxHeightHR() / 1440.0f).append("in");
     out.xml.addAttribute("svg:height", height);
