@@ -102,6 +102,12 @@ public:
     virtual void bookmarkStart( const wvWare::BookmarkData& data );
     virtual void bookmarkEnd( const wvWare::BookmarkData& data );
 
+    /**
+     * Paragraph can be present in {header, footer, footnote, endnote,
+     * annotation, body}.  @return the actual writer.
+     */
+    KoXmlWriter* currentWriter() const;
+
     ///////// Our own interface, also used by processStyles
 
     Document* document() const {
@@ -270,7 +276,7 @@ private:
     QList<QString> fld_snippets;
 
     //field type enumeration as defined in MS-DOC page 354/609
-    typedef enum
+    enum fldType
     {
         UNSUPPORTED = 0,
         REF = 3,
@@ -279,8 +285,10 @@ private:
         PAGE = 33,
         PAGEREF = 37,
         EQ = 49,
+        MERGEFIELD = 59,
         HYPERLINK = 88,
-    } fldType;
+        SHAPE = 95
+    };
 
     //set to 0 for a field we can't handle, anything else is the field type
     int m_fieldType;
