@@ -29,6 +29,7 @@
 #include <styles/KoListLevelProperties.h>
 
 #include "MsooXmlReader.h"
+#include "MsooXmlUtils.h"
 
 namespace MSOOXML
 {
@@ -63,7 +64,11 @@ protected:
     KoCharacterStyle* m_currentTextStyleProperties;
 
     KoGenStyle m_currentListStyle;
-    KoListLevelProperties* m_currentListStyleProperties;
+    MSOOXML::Utils::ParagraphBulletProperties m_currentBulletProperties;
+    // Properties for potentially all 9 lvls which are possible
+    QMap<int, MSOOXML::Utils::ParagraphBulletProperties> m_currentCombinedBulletProperties;
+    QMap<int, KoGenStyle> m_currentCombinedParagraphStyles;
+    QMap<int, KoGenStyle> m_currentCombinedTextStyles;
 
     //! Style (from styles.xml) to apply to the current paragraph or similar element, set by read_pStyle()
     QString m_currentStyleName;
@@ -100,6 +105,8 @@ protected:
 //    uint m_currentParagraphStyleNumber;
 //    QString currentParagraphStyleName() const;
 private:
+    Q_DISABLE_COPY(MsooXmlCommonReader)
+
     void init();
 };
 
