@@ -22,6 +22,8 @@
 * the table starts at tbl §21.1.3.13
 */
 
+#include <MsooXmlDrawingTableStyleReader.h>
+
 #undef CURRENT_EL
 #define CURRENT_EL tbl
 //! tbl (Table) §21.1.3.13
@@ -31,15 +33,6 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tbl()
     MSOOXML::Utils::XmlWriteBuffer tableBuf;
     body = tableBuf.setWriter(body);
 
-    //const QXmlStreamAttributes attrs(attributes());
-    //TRY_READ_ATTR(bandDol)
-    //TRY_READ_ATTR(bandRow)
-    //TRY_READ_ATTR(firstCol)
-    //TRY_READ_ATTR(firstRow)
-    //TRY_READ_ATTR(lastCol)
-    //TRY_READ_ATTR(lastRow)
-    //TRY_READ_ATTR(rtl)
-
     m_currentTableName = QLatin1String("Table") + QString::number(m_currentTableNumber + 1);
     m_currentTableStyle = KoGenStyle(KoGenStyle::TableAutoStyle, "table");
     m_currentTableWidth = 0.0;
@@ -47,22 +40,23 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tbl()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(tblPr)
             ELSE_TRY_READ_IF(tblGrid)
             ELSE_TRY_READ_IF(tr)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     body = tableBuf.originalWriter();
-
-    body->startElement("draw:frame");
-    body->addAttribute("svg:x", QString("%1pt").arg(EMU_TO_POINT(m_svgX)));
-    body->addAttribute("svg:y", QString("%1pt").arg(EMU_TO_POINT(m_svgY)));
-    body->addAttribute("svg:width", QString("%1pt").arg(EMU_TO_POINT(m_svgWidth)));
-    body->addAttribute("svg:height", QString("%1pt").arg(EMU_TO_POINT(m_svgHeight)));
+//TODO this seems to not be needed anymore, it seems it's taken care up in the hierarchy.
+//However, I'm not absolutely sure so I'll just comment it for now.
+//     body->startElement("draw:frame");
+//     body->addAttribute("svg:x", QString("%1pt").arg(EMU_TO_POINT(m_svgX)));
+//     body->addAttribute("svg:y", QString("%1pt").arg(EMU_TO_POINT(m_svgY)));
+//     body->addAttribute("svg:width", QString("%1pt").arg(EMU_TO_POINT(m_svgWidth)));
+//     body->addAttribute("svg:height", QString("%1pt").arg(EMU_TO_POINT(m_svgHeight)));
 
     body->startElement("table:table");
     body->addAttribute("table:name", m_currentTableName);
@@ -91,6 +85,8 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tbl()
         const QString columnStyleName = mainStyles->insert(columnStyle, "col");
 
         body->addAttribute("table:style-name", columnStyleName);
+        body->addAttribute("table:default-cell-style-name", m_defaultCellStyle);
+
         body->endElement(); // table:table-column
     }
     m_columnsWidth.clear();
@@ -98,7 +94,7 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tbl()
     (void)tableBuf.releaseWriter();
     body->endElement(); // table:table
 
-    body->endElement(); // draw:frame
+//     body->endElement(); // draw:frame
 
     m_currentTableNumber++;
 
@@ -111,8 +107,18 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tbl()
 KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tblPr()
 {
     READ_PROLOGUE
+
+    const QXmlStreamAttributes attrs(attributes());
+    TRY_READ_ATTR(bandCol)
+    TRY_READ_ATTR(bandRow)
+    TRY_READ_ATTR(firstCol)
+    TRY_READ_ATTR(firstRow)
+    TRY_READ_ATTR(lastCol)
+    TRY_READ_ATTR(lastRow)
+
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
 //             TRY_READ_IF(blipFill)
 //             ELSE_TRY_READ_IF(effectDrag)
@@ -128,9 +134,37 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tblPr()
             /*ELSE_*/TRY_READ_IF(tableStyleId)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
+    // Needed temporarily as there are no lists necessarily
+    if (m_context->tableStyleList) {
+        MSOOXML::TableStyle tableStyle = m_context->tableStyleList->tableStyle(m_styleId);
+
+        m_defaultCellStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::WholeTbl).saveStyle(*mainStyles);
+
+        if(bandCol == "1") {
+            m_oddColumnStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::Band1Horizontal).saveStyle(*mainStyles);
+            m_evenColumnStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::Band2Horizontal).saveStyle(*mainStyles);
+        }
+        if(bandRow == "1") {
+            m_oddRowStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::Band1Vertical).saveStyle(*mainStyles);
+            m_evenRowStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::Band2Vertical).saveStyle(*mainStyles);
+        }
+        if(firstCol == "1") {
+            m_firstColStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::FirstCol).saveStyle(*mainStyles);
+        }
+        if(firstRow == "1") {
+            m_firstRowStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::FirstRow).saveStyle(*mainStyles);
+        }
+        if(lastCol == "1") {
+            m_lastColStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::LastCol).saveStyle(*mainStyles);
+        }
+        if(firstCol == "1") {
+            m_lastRowStyle = tableStyle.propertiesForType(MSOOXML::TableStyleProperties::LastRow).saveStyle(*mainStyles);
+        }
+    }
+//     TRY_READ_ATTR(rtl)
+
     READ_EPILOGUE
 }
 
@@ -143,11 +177,11 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tblGrid()
     READ_PROLOGUE
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(gridCol)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -168,10 +202,10 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_gridCol()
 
     while(!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
 //         if(isStartElement()) {
 //             TRY_READ_IF(extLst)
 //         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     READ_EPILOGUE
@@ -195,12 +229,12 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tr()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             TRY_READ_IF(tc)
 //             ELSE_TRY_READ_IF(extLst)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     body = rowBuf.originalWriter();
@@ -237,6 +271,7 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tc()
 
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
             if(qualifiedName() == "a:txBody") {
                 TRY_READ(DrawingML_txBody);
@@ -245,8 +280,6 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tc()
             ELSE_TRY_READ_IF(tcPr)
 //             ELSE_WRONG_FORMAT
         }
-
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     body = cellBuf.originalWriter();
@@ -335,6 +368,7 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tcPr()
     READ_PROLOGUE
     while (!atEnd()) {
         readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if (isStartElement()) {
 //            TRY_READ_IF(blipFill)
 //            ELSE_TRY_READ_IF(cell3D)
@@ -352,7 +386,6 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tcPr()
 //            ELSE_TRY_READ_IF(solidFill)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
     READ_EPILOGUE
 }
@@ -365,7 +398,7 @@ KoFilter::ConversionStatus MSOOXML_CURRENT_CLASS::read_tableStyleId()
     READ_PROLOGUE
 
     readNext();
-    //text().toString();
+    m_styleId = text().toString();
     readNext();
 
     READ_EPILOGUE
