@@ -176,35 +176,43 @@ Border TableStyleProperties::borderForSide(Border::Side side) const
 QString TableStyleProperties::stringFromType(Type type)
 {
     switch(type) {
-        case NoType:
-            return QString();
-        case FirstRow:
-            return "firstRow";
-            break;
-        case FirstCol:
-            return "firstCol";
-            break;
-        case LastCol:
-            return "lastCol";
-            break;
-        case LastRow:
-            return "lastRow";
-            break;
-        case NeCell:
-            return "neCell";
-            break;
-        case NwCell:
-            return "nwCell";
-            break;
-        case SeCell:
-            return "seCell";
-            break;
-        case SwCell:
-            return "swCell";
-            break;
-        case WholeTbl:
-            return "wholeTbl";
-            break;
+    case NoType:
+        return QString();
+    case FirstRow:
+        return "firstRow";
+        break;
+    case FirstCol:
+        return "firstCol";
+        break;
+    case LastCol:
+        return "lastCol";
+        break;
+    case LastRow:
+        return "lastRow";
+        break;
+    case NeCell:
+        return "neCell";
+        break;
+    case NwCell:
+        return "nwCell";
+        break;
+    case SeCell:
+        return "seCell";
+        break;
+    case SwCell:
+        return "swCell";
+        break;
+    case WholeTbl:
+        return "wholeTbl";
+        break;
+    case Band1Horizontal:
+    case Band2Horizontal:
+    case Band1Vertical:
+    case Band2Vertical:
+    default:
+        qFatal("Encountered unhandled table style type");
+        return QString();
+
     }
 
     //Shut up the compiler about not returning from a non-void function
@@ -252,42 +260,25 @@ QString TableStyleProperties::saveStyle(KoGenStyles& styles)
         return QString();
     }
 
-    KoGenStyle::Type styleType;
-    QLatin1String styleBaseName("");
-    switch(m_type) {
-        //shut up compiler about not handled enum value in a switch
-        case NoType:
-            Q_ASSERT(false);
-            break;
-        case FirstRow:
-        case LastRow:
-            styleType = KoGenStyle::TableRowAutoStyle;
-            styleBaseName = QLatin1String("row");
-            break;
-        case FirstCol:
-        case LastCol:
-            styleType = KoGenStyle::TableColumnAutoStyle;
-            styleBaseName = QLatin1String("col");
-            break;
-        case NeCell:
-        case NwCell:
-        case SeCell:
-        case SwCell:
-            styleType = KoGenStyle::TableCellAutoStyle;
-            styleBaseName = QLatin1String("cell");
-            break;
-        case WholeTbl:
-            styleType =KoGenStyle::TableAutoStyle;
-            styleBaseName = QLatin1String("table");
-            break;
-    }
-    KoGenStyle style = KoGenStyle(styleType);
+    KoGenStyle style = KoGenStyle(KoGenStyle::TableCellAutoStyle, "table-cell");
+
+    //FIXME support actual background
+    //I just added it to play nice to OOo which adds a blue background of questionable taste
+    style.addProperty("draw:fill", "none", KoGenStyle::GraphicType);
 
     foreach(const Border& border, m_borders) {
-        style.addAttribute(border.odfBorderName(), border.odfStyleProperties());
+        style.addProperty(border.odfBorderName(), border.odfStyleProperties(), KoGenStyle::ParagraphType);
     }
 
-    return styles.insert(style, styleBaseName.latin1());
+    return styles.insert(style, "cell");
+}
+
+TableStyle::TableStyle()
+{
+}
+
+TableStyle::~TableStyle()
+{
 }
 
 void TableStyle::setId(const QString& id)
@@ -320,6 +311,14 @@ void TableStyle::addProperties(TableStyleProperties properties)
     m_properties.insert(properties.type(), properties);
 }
 
+TableStyleList::TableStyleList()
+{
+}
+
+TableStyleList::~TableStyleList()
+{
+}
+
 TableStyle TableStyleList::tableStyle(const QString& id) const
 {
     return m_styles.value(id);
@@ -339,6 +338,19 @@ MsooXmlDrawingTableStyleReader::~MsooXmlDrawingTableStyleReader()
 {
 }
 
+MsooXmlDrawingTableStyleContext::MsooXmlDrawingTableStyleContext(MsooXmlImport* _import, const QString& _path, const QString& _file, DrawingMLTheme* _themes, TableStyleList* _styleList)
+{
+    import = _import;
+    path = _path;
+    file = _file;
+    themes = _themes;
+    styleList = _styleList;
+}
+
+MsooXmlDrawingTableStyleContext::~MsooXmlDrawingTableStyleContext()
+{
+}
+
 KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read(MsooXmlReaderContext* context)
 {
     m_context = dynamic_cast<MsooXmlDrawingTableStyleContext*>(context);
@@ -363,10 +375,11 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tblStyleLst()
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
             TRY_READ_IF(tblStyle)
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     READ_EPILOGUE
@@ -378,7 +391,12 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tblStyle()
 {
     READ_PROLOGUE
 
+    QXmlStreamAttributes attrs(attributes());
+    READ_ATTR_WITHOUT_NS(styleId)
+
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
 //             TRY_READ_IF(band1H)
 //             ELSE_TRY_READ_IF(band1V)
@@ -397,11 +415,8 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tblStyle()
             /*ELSE_*/TRY_READ_IF(wholeTbl)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
-    QXmlStreamAttributes attrs(attributes());
-    READ_ATTR_WITHOUT_NS(styleId)
     m_context->styleList->insertStyle(styleId, m_currentStyle);
     m_currentStyle = TableStyle();
 
@@ -417,12 +432,13 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_wholeTbl()
     m_currentStyleProperties.setType(TableStyleProperties::WholeTbl);
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
             TRY_READ_IF(tcStyle)
             ELSE_TRY_READ_IF(tcTxStyle)
             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     m_currentStyle.addProperties(m_currentStyleProperties);
@@ -438,14 +454,15 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tcStyle()
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
 //             TRY_READ_IF(cell3D)
 //             ELSE_TRY_READ_IF(fill)
 //             ELSE_TRY_READ_IF(fillRef)
-            /*ELSE_*/TRY_READ_IF(tcBrd)
+            /*ELSE_*/TRY_READ_IF(tcBdr)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     READ_EPILOGUE
@@ -460,12 +477,14 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tcTxStyle()
 }
 
 #undef CURRENT_EL
-#define CURRENT_EL tcBrd
-KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tcBrd()
+#define CURRENT_EL tcBdr
+KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tcBdr()
 {
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
             TRY_READ_IF(bottom)
 //             ELSE_TRY_READ_IF(extLst)
@@ -477,9 +496,7 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tcBrd()
             ELSE_TRY_READ_IF(top)
 //             ELSE_TRY_READ_IF(tr2bl)
 //             ELSE_WRONG_FORMAT
-            m_currentPen = QPen();
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     READ_EPILOGUE
@@ -492,17 +509,20 @@ KoFilter::ConversionStatus MSOOXML::MsooXmlDrawingTableStyleReader::read_bottom(
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
-            TRY_READ_IF(ln)
+//             TRY_READ_IF(ln)
 //             ELSE_TRY_READ_IF(lnRef)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     Border border;
     border.setSide(Border::Bottom);
-    border.setColor(m_currentPen.color());
+//     border.setColor(m_currentPen.color());
+    //FIXME don't asume it's solid
+    border.setStyle(MSOOXML::Border::Solid);
     m_currentStyleProperties.addBorder(border);
 
     READ_EPILOGUE
@@ -515,17 +535,20 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_top()
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
-            TRY_READ_IF(ln)
+//             TRY_READ_IF(ln)
 //             ELSE_TRY_READ_IF(lnRef)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     Border border;
     border.setSide(Border::Top);
-    border.setColor(m_currentPen.color());
+//     border.setColor(m_currentPen.color());
+    //FIXME don't asume it's solid
+    border.setStyle(MSOOXML::Border::Solid);
     m_currentStyleProperties.addBorder(border);
 
     READ_EPILOGUE
@@ -538,17 +561,20 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_left()
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
-            TRY_READ_IF(ln)
+//             TRY_READ_IF(ln)
 //             ELSE_TRY_READ_IF(lnRef)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     Border border;
     border.setSide(Border::Left);
-    border.setColor(m_currentPen.color());
+//     border.setColor(m_currentPen.color());
+    //FIXME don't asume it's solid
+    border.setStyle(MSOOXML::Border::Solid);
     m_currentStyleProperties.addBorder(border);
 
     READ_EPILOGUE
@@ -561,17 +587,20 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_right()
     READ_PROLOGUE
 
     while(!atEnd()) {
+        readNext();
+        BREAK_IF_END_OF(CURRENT_EL);
         if(isStartElement()) {
-            TRY_READ_IF(ln)
+//             TRY_READ_IF(ln)
 //             ELSE_TRY_READ_IF(lnRef)
 //             ELSE_WRONG_FORMAT
         }
-        BREAK_IF_END_OF(CURRENT_EL);
     }
 
     Border border;
     border.setSide(Border::Right);
-    border.setColor(m_currentPen.color());
+//     border.setColor(m_currentPen.color());
+    //FIXME don't asume it's solid
+    border.setStyle(MSOOXML::Border::Solid);
     m_currentStyleProperties.addBorder(border);
 
     READ_EPILOGUE
@@ -582,7 +611,7 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_right()
 // KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tl2br()
 // {
 //     READ_PROLOGUE
-// 
+//
 //     while(!atEnd()) {
 //         if(isStartElement()) {
 //             TRY_READ_IF(ln)
@@ -591,11 +620,11 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_right()
 //         }
 //         BREAK_IF_END_OF(CURRENT_EL);
 //     }
-// 
+//
 //     Border border;
 //     border.setColor(m_currentPen.color());
 //     m_currentStyleProperties.addBorder(border, Border::TopLeftToBottomRight);
-// 
+//
 //     READ_EPILOGUE
 // }
 
@@ -604,7 +633,7 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_right()
 // KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_tr2bl()
 // {
 //     READ_PROLOGUE
-// 
+//
 //     while(!atEnd()) {
 //         if(isStartElement()) {
 //             TRY_READ_IF(ln)
@@ -613,11 +642,11 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_right()
 //         }
 //         BREAK_IF_END_OF(CURRENT_EL);
 //     }
-// 
+//
 //     Border border;
 //     border.setColor(m_currentPen.color());
 //     m_currentStyleProperties.addBorder(border, Border::TopRightToBottomLeft);
-// 
+//
 //     READ_EPILOGUE
 // }
 
@@ -629,4 +658,4 @@ KoFilter::ConversionStatus MsooXmlDrawingTableStyleReader::read_right()
 #define DRAWINGML_NS "a"
 #define DRAWINGML_PIC_NS "p" // DrawingML/Picture
 
-#include <MsooXmlCommonReaderDrawingMLImpl.h>
\ No newline at end of file
+#include <MsooXmlCommonReaderDrawingMLImpl.h>
