@@ -101,6 +101,7 @@ const char* const MSOOXML::ContentTypes::presentationComments =      "applicatio
 
 // spreadsheetml-specific content types
 const char* const MSOOXML::ContentTypes::spreadsheetDocument =        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml";
+const char* const MSOOXML::ContentTypes::spreadsheetMacroDocument =   "application/vnd.ms-excel.sheet.macroEnabled.main+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetPrinterSettings = "application/vnd.openxmlformats-officedocument.spreadsheetml.printerSettings";
 const char* const MSOOXML::ContentTypes::spreadsheetStyles =          "application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml";
 const char* const MSOOXML::ContentTypes::spreadsheetWorksheet =       "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml";
@@ -236,7 +237,14 @@ QIODevice* Utils::openDeviceForFile(const KZip* zip, QString& errorMessage, cons
     const KZipFileEntry* f = static_cast<const KZipFileEntry *>(entry);
     kDebug() << "Entry" << fileName << "has size" << f->size();
     status = KoFilter::OK;
-    return f->createDevice();
+    // There seem to be some problems with kde/zlib when trying to read
+    // multiple streams, this functionality is needed in the filter
+    // Until there's another solution for this, this avoids the problem
+    //return f->createDevice();
+    QBuffer *device = new QBuffer();
+    device->setData(f->data());
+    device->open(QIODevice::ReadOnly);
+    return device;
 }
 
 #define BLOCK_SIZE 4096
@@ -925,6 +933,10 @@ public:
                new UnderlineStyle(KoCharacterStyle::SolidLine, KoCharacterStyle::DoubleLine,
                                   KoCharacterStyle::AutoLineWeight)
               );
+        insert("dbl",
+               new UnderlineStyle(KoCharacterStyle::SolidLine, KoCharacterStyle::DoubleLine,
+                                  KoCharacterStyle::AutoLineWeight)
+              );
         insert("words",
                new UnderlineStyle(KoCharacterStyle::SolidLine, KoCharacterStyle::SingleLine,
                                   KoCharacterStyle::AutoLineWeight, KoCharacterStyle::SkipWhiteSpaceLineMode)
@@ -961,6 +973,10 @@ public:
                new UnderlineStyle(KoCharacterStyle::WaveLine, KoCharacterStyle::DoubleLine,
                                   KoCharacterStyle::AutoLineWeight)
               );
+        insert("wavyDbl",
+               new UnderlineStyle(KoCharacterStyle::WaveLine, KoCharacterStyle::DoubleLine,
+                                  KoCharacterStyle::AutoLineWeight)
+              );
         insert("wavyHeavy",
                new UnderlineStyle(KoCharacterStyle::WaveLine, KoCharacterStyle::SingleLine,
                                   KoCharacterStyle::BoldLineWeight)
@@ -1001,17 +1017,19 @@ public:
     }
 };
 
-QString Utils::rotateString(const qreal rotation, const qreal xPos, const qreal yPos)
+void Utils::rotateString(const qreal rotation, const qreal width, const qreal height, qreal& angle, qreal& xDiff, qreal& yDiff,
+    bool flipH, bool flipV)
 {
-    qreal angle = -(qreal)rotation * ((qreal)(M_PI) / (qreal)180.0)
-                                                    / (qreal)60000.0;
-    qreal oldX = xPos / 360000.0;
-    qreal oldY = yPos / 360000.0;
-    qreal hypo = sqrt(oldX * oldX + oldY * oldY);
-    qreal newX = hypo * sin((qreal)(M_PI)/4 + angle);
-    qreal newY = hypo * cos((qreal)(M_PI)/4 + angle);
-
-    return QString("rotate(%1) translate(%2cm %3cm)").arg(angle).arg(oldX - newX).arg(oldY - newY);
+    angle = -(qreal)rotation * ((qreal)(M_PI) / (qreal)180.0)/ (qreal)60000.0;
+    // Angle seems to be negative if one of the flips is active
+    if ((flipH && !flipV) || (!flipH && flipV)) {
+        angle = -angle;
+    }
+    //position change is calculated based on the fact that center point stays in the same location
+    // Width/2 = Xnew + cos(angle)*Width/2 - sin(angle)*Height/2
+    // Height/2 = Ynew + sin(angle)*Width/2 + cos(angle)*Height/2
+    xDiff = width/2 - cos(-angle)*width/2 + sin(-angle)*height/2;
+    yDiff = height/2 - sin(-angle)*width/2 - cos(-angle)*height/2;
 }
 
 void Utils::setupUnderLineStyle(const QString& msooxmlName, KoCharacterStyle* textStyleProperties)
@@ -1260,8 +1278,86 @@ MSOOXML_EXPORT QString Utils::ST_PositiveUniversalMeasure_to_cm(const QString& v
 
 // </units> -------------------
 
+MSOOXML_EXPORT Utils::ParagraphBulletProperties::ParagraphBulletProperties() :
+    m_startValue(0), m_type(ParagraphBulletProperties::BulletType), m_bulletChar(""), 
+    m_align(""), m_indent(0)
+{
+}
+
+MSOOXML_EXPORT bool Utils::ParagraphBulletProperties::isEmpty() const
+{
+    if (m_bulletChar == "" && m_type == ParagraphBulletProperties::BulletType) {
+        return true;
+    }
+    return false;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::clear()
+{
+    m_startValue = 0;
+    setBulletChar("");
+    m_align = "";
+    m_indent = 0;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setAlign(const QString& align)
+{
+    m_align = align;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setBulletChar(const QString& bulletChar)
+{
+    m_bulletChar = bulletChar;
+    m_type = ParagraphBulletProperties::BulletType;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setIndent(const qreal indent)
+{
+    m_indent = indent;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setSuffix(const QString& suffixChar)
+{
+    m_suffix = suffixChar;
+}
+
+MSOOXML_EXPORT void Utils::ParagraphBulletProperties::setNumFormat(const QString& numFormat)
+{
+    m_numFormat = numFormat;
+    m_type = ParagraphBulletProperties::NumberType;
+}
+
+MSOOXML_EXPORT QString Utils::ParagraphBulletProperties::convertToListProperties() const
+{
+    QString returnValue;
+    if (m_type == ParagraphBulletProperties::NumberType) {
+        returnValue = QString("<text:list-level-style-number text:level=\"%1\" ").arg(m_level);
+        returnValue += QString("style:num-suffix=\"%1\" style:num-format=\"%2\" ").arg(m_suffix).arg(m_numFormat);
+        if (m_startValue != 0) {
+            returnValue += QString("style:start-value=\"%1\" ").arg(m_startValue);
+        }
+    }
+    else {
+        returnValue = QString("<text:list-level-style-bullet text:level=\"%1\" ").arg(m_level);
+        returnValue += QString("text:bullet-char=\"%1\" ").arg(m_bulletChar);
+    }
+    if (!m_align.isEmpty()) {
+        returnValue += QString("fo:text-align=\"%1\" ").arg(m_align);
+    }
+    if (m_indent > 0) {
+        returnValue += QString("text:space-before=\"%1pt\" ").arg(m_indent);
+    }
+    returnValue += "/>";
+
+    return returnValue;
+}
+
 MSOOXML_EXPORT void Utils::copyPropertiesFromStyle(const KoGenStyle& sourceStyle, KoGenStyle& targetStyle, KoGenStyle::PropertyType type)
 {
+    if (sourceStyle.isEmpty()) {
+        return;
+    }
+
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter tempWriter(&buffer);
@@ -1296,3 +1392,45 @@ MSOOXML_EXPORT void Utils::copyPropertiesFromStyle(const KoGenStyle& sourceStyle
         targetStyle.addProperty(propertyName, propertyValue, type);
     }
 }
+
+MSOOXML_EXPORT void Utils::modifyColor(QColor& color, qreal tint, qreal shade, qreal satMod)
+{
+    int red = color.red();
+    int green = color.green();
+    int blue = color.blue();
+
+    if (tint > 0) {
+        red = tint * red + (1 - tint) * 255;
+        green = tint * green + (1 - tint) * 255;
+        blue = tint * blue + (1 - tint) * 255;
+    }
+    if (shade > 0) {
+        red = shade * red;
+        green = shade * green;
+        blue = shade * blue;
+    }
+
+    // FIXME: This calculation for sure is incorrect,
+    // According to MS forums, RGB should first be converted to linear RGB
+    // Then to HSL and then multiply saturation value by satMod
+    // SatMod can be for example 3.5 so converting RGB -> HSL is not an option
+    // ADD INFO: MS document does not say that when calculating TINT and SHADE
+    // That whether one should use normal RGB or linear RGB, check it!
+    if (satMod > 0) {
+        red = red * satMod;
+        green = green * satMod;
+        blue = blue * satMod;
+        if (red > 255) {
+            red = 255;
+        }
+        if (green > 255) {
+            green = 255;
+        }
+        if (blue > 255) {
+            blue = 255;
+        }
+    }
+
+    color = QColor(red, green, blue);
+}
+
