@@ -30,7 +30,7 @@
 #include "IdentifierElement.h"
 #include "TableElement.h"
 #include "TableRowElement.h"
-#include "TableEntryElement.h"
+#include "TableDataElement.h"
 #include "MultiscriptElement.h"
 #include "NumberElement.h"
 #include "OperatorElement.h"
@@ -52,7 +52,7 @@
 BasicElement* ElementFactory::createElement( const QString& tagName,
                                              BasicElement* parent )
 {
-    kWarning( DEBUGID ) << "Creating element: " << tagName;
+//    kWarning( DEBUGID ) << "Creating element: " << tagName;
     if( tagName == "mi" )
         return new IdentifierElement( parent );
     else if ( tagName == "mo" )
@@ -83,14 +83,28 @@ BasicElement* ElementFactory::createElement( const QString& tagName,
           return new PaddedElement( parent );
     else if ( tagName == "mphantom" )
           return new PhantomElement( parent );
+    else if ( tagName == "mtable" )
+          return new TableElement( parent );
+    else if ( tagName == "mtr" )
+          return new TableRowElement( parent );
+    else if ( tagName == "mtd" )
+          return new TableDataElement( parent );
     else if ( tagName == "mfenced" )
           return new FencedElement( parent );
     else if ( tagName == "menclose" )
           return new EncloseElement( parent );
-    else if ( tagName == "msub" || tagName == "msup" || tagName == "msubsup" )
-          return new SubSupElement( parent );
-    else if ( tagName == "munder" || tagName == "mover" || tagName == "munderover" )
-          return new UnderOverElement( parent );
+    else if ( tagName == "msub")
+          return new SubSupElement( parent, SubScript );
+    else if ( tagName == "msup")
+          return new SubSupElement( parent, SupScript );
+    else if ( tagName == "msubsup")
+          return new SubSupElement( parent, SubSupScript );
+    else if ( tagName == "munder")
+          return new UnderOverElement( parent, Under );
+    else if ( tagName == "mover")
+          return new UnderOverElement( parent, Over );
+    else if ( tagName == "munderover")
+          return new UnderOverElement( parent, UnderOver );
     else if ( tagName == "mmultiscripts" )
           return new MultiscriptElement( parent );
     
@@ -149,11 +163,11 @@ QString ElementFactory::elementName( ElementType type )
             return "mroot";
         case SquareRoot:
             return "msqrt";
-        case Matrix:
+        case Table:
             return "mtable";
-        case MatrixRow:
+        case TableRow:
             return "mtr"; // TODO: Check for mlabeledtr
-        case MatrixEntry:
+        case TableData:
             return "mtd";
         case Action:
             return "maction";
