@@ -36,9 +36,9 @@ enum ElementType {
     Operator,
     Space,
     Fraction,
-    Matrix,
-    MatrixRow,
-    MatrixEntry,
+    Table,
+    TableRow,
+    TableData,
     Under,
     Over,
     UnderOver,
@@ -58,7 +58,8 @@ enum ElementType {
     Enclose,
     Phantom,
     Action,
-    Unknown
+    Unknown,
+    Empty
 };
 
 
