@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2005 David Faure <faure@kde.org>
- * Copyright (C) 2007-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2007 Pierre Ducroquet <pinaraf@gmail.com>
  * Copyright (C) 2007-2009 Thorsten Zachmann <zachmann@kde.org>
@@ -28,12 +28,7 @@
 #include "frames/KWTextFrame.h"
 #include "frames/KWCopyShape.h"
 
-#include <KoShape.h>
-#include <KoTextAnchor.h>
-#include <KoTextDocument.h>
 #include <KoTextShapeData.h>
-#include <KoTextDocumentLayout.h>
-#include <KoInlineTextObjectManager.h>
 #include <KoOdfLoadingContext.h>
 #include <KoShapeLoadingContext.h>
 #include <KoXmlNS.h>
@@ -41,7 +36,7 @@
 #include <QTextCursor>
 #include <kdebug.h>
 
-KWOdfSharedLoadingData::KWOdfSharedLoadingData(KWOdfLoader* loader)
+KWOdfSharedLoadingData::KWOdfSharedLoadingData(KWOdfLoader *loader)
         : KoTextSharedLoadingData()
         , m_loader(loader)
 {
@@ -53,7 +48,7 @@ KWOdfSharedLoadingData::KWOdfSharedLoadingData(KWOdfLoader* loader)
             KoXmlNS::text, "anchor-page-number", "text:anchor-page-number"));
 }
 
-void KWOdfSharedLoadingData::shapeInserted(KoShape* shape, const KoXmlElement &element, KoShapeLoadingContext &context)
+void KWOdfSharedLoadingData::shapeInserted(KoShape *shape, const KoXmlElement &element, KoShapeLoadingContext &context)
 {
     int pageNumber = -1;
     if (shape->hasAdditionalAttribute("text:anchor-type")) {
@@ -66,7 +61,7 @@ void KWOdfSharedLoadingData::shapeInserted(KoShape* shape, const KoXmlElement &e
         }
     }
 
-    //kDebug(32001) << "text:anchor-type =" << shape->additionalAttribute("text:anchor-type") << shape->additionalAttribute( "text:anchor-page-number" ) << pageNumber;
+    //kDebug(32001) << "text:anchor-type =" << shape->additionalAttribute("text:anchor-type") << shape->additionalAttribute("text:anchor-page-number") << pageNumber;
     shape->removeAdditionalAttribute("text:anchor-type");
     const KoXmlElement *style = 0;
     if (element.hasAttributeNS(KoXmlNS::draw, "style-name")) {
@@ -105,11 +100,18 @@ void KWOdfSharedLoadingData::shapeInserted(KoShape* shape, const KoXmlElement &e
                 m_nextFrames.insert(nextFrame, frame);
             }
 
-            if (textBox.hasAttributeNS(KoXmlNS::fo, "min-height"))
+            if (textBox.hasAttributeNS(KoXmlNS::fo, "min-height")) {
                 frame->setMinimumFrameHeight(KoUnit::parseValue(textBox.attributeNS(KoXmlNS::fo, "min-height")));
+                KoShape *shape = frame->shape();
+                QSizeF newSize = shape->size();
+                if (newSize.height() < frame->minimumFrameHeight()) {
+                    newSize.setHeight(frame->minimumFrameHeight());
+                    shape->setSize(newSize);
+                }
+            }
         }
     } else {
-        KWFrameSet* fs = new KWFrameSet();
+        KWFrameSet *fs = new KWFrameSet();
         fs->setName(m_loader->document()->uniqueFrameSetName(shape->name()));
         KWFrame *frame = new KWFrame(shape, fs, pageNumber);
         if (style)
@@ -130,7 +132,7 @@ bool KWOdfSharedLoadingData::fillFrameProperties(KWFrame *frame, const KoXmlElem
         // untested... No app saves this currently..
         foreach (KWFrame *f, frame->frameSet()->frames()) {
             if (f->shape()->name() == copy) {
-                KWCopyShape *shape= new KWCopyShape(f->shape());
+                KWCopyShape *shape = new KWCopyShape(f->shape());
                 new KWFrame(shape, frame->frameSet(), frame->loadingPageNumber());
                 delete frame;
                 return false;
@@ -164,12 +166,27 @@ bool KWOdfSharedLoadingData::fillFrameProperties(KWFrame *frame, const KoXmlElem
         margin = properties.attributeNS(KoXmlNS::fo, "margin-right");
     frame->setRunAroundDistance(KoUnit::parseValue(margin));
 
-    frame->setTextRunAround(KWord::RunThrough); // Default
-    QString wrap = properties.attributeNS(KoXmlNS::style, "wrap");
+    QString wrap;
+    if (properties.hasAttributeNS(KoXmlNS::style, "wrap")) {
+        wrap = properties.attributeNS(KoXmlNS::style, "wrap");
+    } else {
+        // no value given in the file, and for compatibility reasons we do some suggestion on
+        // what to use.
+        KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(frame->frameSet());
+        if (tfs == 0)
+            wrap = "none";
+        else
+            wrap = "biggest";
+    }
     if (wrap == "none") {
         frame->setTextRunAround(KWord::NoRunAround);
     } else if (wrap == "run-through") {
-        frame->setTextRunAround(KWord::RunThrough);
+        QString runTrought = properties.attributeNS(KoXmlNS::style, "run-through", "background");
+        if (runTrought == "background") {
+            frame->setTextRunAround(KWord::RunThrough, KWord::Background);
+        } else {
+            frame->setTextRunAround(KWord::RunThrough, KWord::Foreground);
+        }
     } else {
         frame->setTextRunAround(KWord::RunAround);
         if (wrap == "biggest")
