@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2001 David Faure <faure@kde.org>
  * Copyright (C) 2005-2007, 2009, 2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -52,6 +53,7 @@
 #include "commands/KWRemoveFrameClipCommand.h"
 
 // koffice libs includes
+#include <kofficeversion.h>
 #include <KoCopyController.h>
 #include <KoTextDocument.h>
 #include <KoTextShapeData.h>
@@ -62,6 +64,7 @@
 #include <KoStandardAction.h>
 #include <KoPasteController.h>
 #include <KoShape.h>
+#include <KoTextOnShapeContainer.h>
 #include <KoText.h>
 #include <KoFind.h>
 #include <KoShapeContainer.h>
@@ -73,7 +76,6 @@
 #include <KoMainWindow.h>
 #include <KoTextEditor.h>
 #include <KoToolProxy.h>
-#include <KoResourceManager.h>
 #include <KoTextAnchor.h>
 #include <KoShapeGroupCommand.h>
 #include <KoZoomController.h>
@@ -84,7 +86,7 @@
 
 // KDE + Qt includes
 #include <QHBoxLayout>
-#include <QMenu>
+#include <KMenu>
 #include <QTimer>
 #include <klocale.h>
 #include <kdebug.h>
@@ -120,6 +122,7 @@ static KWFrame *frameForShape(KoShape *shape)
 
 KWView::KWView(const QString &viewMode, KWDocument *document, QWidget *parent)
         : KoView(document, parent)
+        , m_canvas(0), m_actionMenu(0)
 {
     m_document = document;
     m_snapToGrid = m_document->gridData().snapToGrid();
@@ -180,9 +183,10 @@ KWView::KWView(const QString &viewMode, KWDocument *document, QWidget *parent)
 
 KWView::~KWView()
 {
+    m_canvas = 0;
 }
 
-KWCanvas *KWView::kwcanvas() const
+KoCanvasBase *KWView::canvasBase() const
 {
     return m_canvas;
 }
@@ -305,18 +309,19 @@ void KWView::setupActions()
     actionCollection()->addAction("send_toback_frame", m_actionSendBackward);
     connect(m_actionSendBackward, SIGNAL(triggered()), this, SLOT(sendToBack()));
 
-    KActionMenu *actionMenu = new KActionMenu(i18n("Variable"), this);
-    foreach (QAction *action, m_document->inlineTextObjectManager()->createInsertVariableActions(kwcanvas()))
-        actionMenu->addAction(action);
-    actionCollection()->addAction("insert_variable", actionMenu);
+    m_actionMenu = new KActionMenu(i18n("Variable"), this);
+    foreach (QAction *action, m_document->inlineTextObjectManager()->createInsertVariableActions(canvasBase()))
+        m_actionMenu->addAction(action);
+    actionCollection()->addAction("insert_variable", m_actionMenu);
+    connect(m_document->inlineTextObjectManager()->variableManager(), SIGNAL(valueChanged()), this, SLOT(variableChanged()));
 
 #ifdef SHOULD_BUILD_RDF
     if (KoDocumentRdf* rdf = m_document->documentRdf()) {
-        KAction* createRef = rdf->createInsertSemanticObjectReferenceAction(kwcanvas());
+        KAction* createRef = rdf->createInsertSemanticObjectReferenceAction(canvasBase());
         actionCollection()->addAction("insert_semanticobject_ref", createRef);
         KActionMenu *subMenu = new KActionMenu(i18n("Create"), this);
         actionCollection()->addAction("insert_semanticobject_new", subMenu);
-        foreach(KAction *action, rdf->createInsertSemanticObjectNewActions(kwcanvas())) {
+        foreach(KAction *action, rdf->createInsertSemanticObjectNewActions(canvasBase())) {
             subMenu->addAction(action);
         }
     }
@@ -376,11 +381,11 @@ void KWView::setupActions()
 
     // -------------- Edit actions
     action = actionCollection()->addAction(KStandardAction::Cut,  "edit_cut", 0, 0);
-    new KoCutController(kwcanvas(), action);
+    new KoCutController(canvasBase(), action);
     action = actionCollection()->addAction(KStandardAction::Copy,  "edit_copy", 0, 0);
-    new KoCopyController(kwcanvas(), action);
+    new KoCopyController(canvasBase(), action);
     action = actionCollection()->addAction(KStandardAction::Paste,  "edit_paste", 0, 0);
-    new KoPasteController(kwcanvas(), action);
+    new KoPasteController(canvasBase(), action);
 
     action  = new KAction(i18n("Statistics"), this);
     actionCollection()->addAction("file_statistics", action);
@@ -426,7 +431,7 @@ if (false) { // TODO move this to the text tool as soon as  a) the string freeze
     action = new KAction(KIcon("edit-delete"), i18n("Delete"), this);
     action->setShortcut(QKeySequence("Del"));
     connect(action, SIGNAL(triggered()), this, SLOT(editDeleteSelection()));
-    connect(kwcanvas()->toolProxy(), SIGNAL(selectionChanged(bool)), action, SLOT(setEnabled(bool)));
+    connect(canvasBase()->toolProxy(), SIGNAL(selectionChanged(bool)), action, SLOT(setEnabled(bool)));
     actionCollection()->addAction("edit_delete", action);
 
     action = m_document->gridData().gridToggleAction(m_canvas);
@@ -456,11 +461,15 @@ if (false) { // TODO move this to the text tool as soon as  a) the string freeze
     actionCollection()->addAction("remove_clipped_frame", action);
     connect(action, SIGNAL(triggered()), this, SLOT(removeFrameClipping()));
 
-    //------------------------ Settings menu
-    action = new KToggleAction(i18n("Status Bar"), this);
-    action->setToolTip(i18n("Shows or hides the status bar"));
-    actionCollection()->addAction("showStatusBar", action);
-    connect(action, SIGNAL(toggled(bool)), this, SLOT(showStatusBar(bool)));
+    action = new KAction(i18n("Add Text on Shape"), this);
+    actionCollection()->addAction("add_text_on_shape", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(createTextOnShape()));
+
+    KToggleAction *tAction = new KToggleAction(i18n("Show Status Bar"), this);
+    tAction->setCheckedState(KGuiItem(i18n("Hide Status Bar")));
+    tAction->setToolTip(i18n("Shows or hides the status bar"));
+    actionCollection()->addAction("showStatusBar", tAction);
+    connect(tAction, SIGNAL(toggled(bool)), this, SLOT(showStatusBar(bool)));
 
     // -------------- Insert menu
     /* ********** From old kwview ****
@@ -926,7 +935,7 @@ if (false) { // TODO move this to the text tool as soon as  a) the string freeze
 QList<KWFrame*> KWView::selectedFrames() const
 {
     QList<KWFrame*> frames;
-    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes()) {
+    foreach (KoShape *shape, canvasBase()->shapeManager()->selection()->selectedShapes()) {
         KWFrame *frame = frameForShape(shape);
         Q_ASSERT(frame);
         frames.append(frame);
@@ -944,16 +953,17 @@ void KWView::editFrameProperties()
 
 KoPrintJob *KWView::createPrintJob()
 {
-    KWPrintingDialog *dia = new KWPrintingDialog(this);
+    KWPrintingDialog *dia = new KWPrintingDialog(m_document, m_canvas->shapeManager(), this);
     dia->printer().setResolution(600);
-    dia->printer().setCreator("KWord 2.0");
+    dia->printer().setCreator(QString("KWord %1.%2.$3").arg(KOffice::versionMajor(),
+        KOffice::versionMinor(), KOffice::versionRelease()));
     dia->printer().setFullPage(true); // ignore printer margins
     return dia;
 }
 
 void KWView::insertFrameBreak()
 {
-    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (canvasBase()->toolProxy()->selection());
     if (handler) {
         // this means we have the text tool selected right now.
         handler->insertFrameBreak();
@@ -967,7 +977,7 @@ void KWView::addBookmark()
 {
     QString name, suggestedName;
 
-    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
     KoShape *shape = 0;
     shape = selection->firstSelectedShape();
     if (shape == 0) return; // no shape selected
@@ -979,7 +989,7 @@ void KWView::addBookmark()
 
     QString tool = KoToolManager::instance()->preferredToolForSelection(selection->selectedShapes());
     KoToolManager::instance()->switchToolRequested(tool);
-    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (canvasBase()->toolProxy()->selection());
     Q_ASSERT(handler);
 
     KoBookmarkManager *manager = m_document->inlineTextObjectManager()->bookmarkManager();
@@ -1018,7 +1028,7 @@ void KWView::selectBookmark()
 
     KoBookmark *bookmark = manager->retrieveBookmark(name);
     KoShape *shape = bookmark->shape();
-    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
     selection->deselectAll();
     selection->select(shape);
 
@@ -1057,7 +1067,7 @@ void KWView::deleteBookmark(const QString &name)
 void KWView::editDeleteFrame()
 {
     QList<KoShape*> frames;
-    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+    foreach (KoShape *shape, canvasBase()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
         KWFrame *frame = frameForShape(shape);
         if (frame) {
             KWTextFrameSet *fs = dynamic_cast<KWTextFrameSet*>(frame->frameSet());
@@ -1066,7 +1076,7 @@ void KWView::editDeleteFrame()
         }
         frames.append(shape);
     }
-    QUndoCommand *cmd = kwcanvas()->shapeController()->removeShapes(frames);
+    QUndoCommand *cmd = canvasBase()->shapeController()->removeShapes(frames);
     m_document->addCommand(cmd);
 }
 
@@ -1097,16 +1107,16 @@ void KWView::toggleSnapToGrid()
 void KWView::adjustZOrderOfSelectedFrames(KoShapeReorderCommand::MoveShapeType direction)
 {
     // TODO we should not allow any shapes to fall behind the main text frame.
-    QUndoCommand *cmd = KoShapeReorderCommand::createCommand(kwcanvas()->shapeManager()->selection()->selectedShapes(),
-                        kwcanvas()->shapeManager(), direction);
+    QUndoCommand *cmd = KoShapeReorderCommand::createCommand(canvasBase()->shapeManager()->selection()->selectedShapes(),
+                        canvasBase()->shapeManager(), direction);
     if (cmd)
         m_document->addCommand(cmd);
 }
 
 void KWView::toggleViewFrameBorders(bool on)
 {
-    kwcanvas()->resourceManager()->setResource(KoText::ShowTextFrames, on);
-    kwcanvas()->update();
+    m_canvas->resourceManager()->setResource(KoText::ShowTextFrames, on);
+    m_canvas->update();
     m_document->config().setViewFrameBorders(on);
 }
 
@@ -1138,7 +1148,7 @@ void KWView::editSemanticStylesheets()
 void KWView::inlineFrame()
 {
     Q_ASSERT(kwdocument()->mainFrameSet());
-    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
 
     KoShape *targetShape = 0;
     foreach (KoShape *shape, selection->selectedShapes(KoFlake::TopLevelSelection)) {
@@ -1182,7 +1192,7 @@ void KWView::inlineFrame()
 
     QString tool = KoToolManager::instance()->preferredToolForSelection(selection->selectedShapes());
     KoToolManager::instance()->switchToolRequested(tool);
-    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (canvasBase()->toolProxy()->selection());
     Q_ASSERT(handler);
     KoTextAnchor *anchor = new KoTextAnchor(targetShape);
     anchor->setOffset(QPointF(0, -targetShape->size().height()));
@@ -1205,7 +1215,7 @@ void KWView::showRulers(bool visible)
 
 void KWView::createLinkedFrame()
 {
-    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
     QList<KoShape*> oldSelection = selection->selectedShapes(KoFlake::TopLevelSelection);
     if (oldSelection.count() == 0)
         return;
@@ -1218,7 +1228,7 @@ void KWView::createLinkedFrame()
         KWCopyShape *copy = new KWCopyShape(frame->shape(), m_document->pageManager());
         copy->setPosition(frame->shape()->position());
         QPointF offset(40, 40);
-        kwcanvas()->clipToDocument(copy, offset);
+        canvasBase()->clipToDocument(copy, offset);
         copy->setPosition(frame->shape()->position() + offset);
         copy->setZIndex(frame->shape()->zIndex() + 1);
         KWFrame *newFrame = new KWFrame(copy, frame->frameSet());
@@ -1268,7 +1278,7 @@ void KWView::setShowFormattingChars(bool on)
 
 void KWView::editSelectAllFrames()
 {
-    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
     foreach (KWFrameSet *fs, m_document->frameSets()) {
         foreach (KWFrame *frame, fs->frames()) {
             if (frame->shape()->isVisible())
@@ -1279,7 +1289,7 @@ void KWView::editSelectAllFrames()
 
 void KWView::editDeleteSelection()
 {
-    kwcanvas()->toolProxy()->deleteSelection();
+    canvasBase()->toolProxy()->deleteSelection();
 }
 
 void KWView::createCustomOutline()
@@ -1296,7 +1306,7 @@ void KWView::createCustomOutline()
         m_document->addCommand(cmd);
     }
 
-    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
     selection->deselectAll();
     foreach (KWFrame *frame, frames) {
         KoShapeContainer *group = frame->shape()->parent();
@@ -1310,7 +1320,7 @@ void KWView::createCustomOutline()
 void KWView::createFrameClipping()
 {
     QSet<KWFrame *> clipFrames;
-    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+    foreach (KoShape *shape, canvasBase()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
         KWFrame *frame = frameForShape(shape);
         Q_ASSERT(frame);
         if (frame->shape()->parent() == 0)
@@ -1325,7 +1335,7 @@ void KWView::createFrameClipping()
 void KWView::removeFrameClipping()
 {
     QSet<KWFrame *> unClipFrames;
-    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+    foreach (KoShape *shape, canvasBase()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
         KWFrame *frame = frameForShape(shape);
         Q_ASSERT(frame);
         if (frame->shape()->parent())
@@ -1373,7 +1383,34 @@ void KWView::insertImage()
 void KWView::setGuideVisibility(bool on)
 {
     m_document->guidesData().setShowGuideLines(on);
-    kwcanvas()->update();
+    m_canvas->update();
+}
+
+void KWView::createTextOnShape()
+{
+    QSet<KoShape *> frameShapes;
+    KoSelection *selection = canvasBase()->shapeManager()->selection();
+    foreach (KoShape *shape, selection->selectedShapes(KoFlake::TopLevelSelection)) {
+        KWFrame *frame = frameForShape(shape);
+        Q_ASSERT(frame);
+        if (frame->shape()->parent() == 0) {
+            frameShapes << frame->shape();
+        }
+    }
+#if 0
+    if (!frameShapes.isEmpty()) {
+        KoAddTextOnShapeCommand *cmd = new KoAddTextOnShapeCommand(frameShapes.toList(), m_document);
+        m_document->addCommand(cmd);
+    }
+#else
+    foreach (KoShape *shape, frameShapes) {
+        selection->deselect(shape);
+        KoTextOnShapeContainer *decorator = new KoTextOnShapeContainer(shape, m_document->resourceManager());
+        decorator->setPlainText("dummy text, la la la enzo");
+        m_document->addShape(decorator);
+        selection->select(decorator);
+    }
+#endif
 }
 
 // end of actions
@@ -1397,7 +1434,7 @@ void KWView::zoomChanged(KoZoomMode::Mode mode, qreal zoom)
 
 void KWView::selectionChanged()
 {
-    KoShape *shape = kwcanvas()->shapeManager()->selection()-> firstSelectedShape();
+    KoShape *shape = canvasBase()->shapeManager()->selection()-> firstSelectedShape();
     m_actionFormatFrameSet->setEnabled(shape != 0);
     m_actionAddBookmark->setEnabled(shape != 0);
     if (shape) {
@@ -1409,10 +1446,13 @@ void KWView::selectionChanged()
         else
             m_actionAddBookmark->setEnabled(false);
     }
-    // actions need at least one shape selected
-    actionCollection()->action("create_linked_frame")->setEnabled(shape);
+    // actions that need at least one shape selected
+    QAction *action = actionCollection()->action("create_linked_frame");
+    if (action) action->setEnabled(shape);
+    action = actionCollection()->action("inline_frame");
+    if (action) action->setEnabled(shape && kwdocument()->mainFrameSet());
 
-    foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+    foreach (KoShape *shape, canvasBase()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
         KWFrame *frame = frameForShape(shape);
         Q_ASSERT(frame);
         QVariant variant;
@@ -1508,7 +1548,7 @@ void KWView::semanticObjectViewSiteUpdated(KoRdfSemanticItem* item, const QStrin
 {
 #ifdef SHOULD_BUILD_RDF
     kDebug(30015) << "xmlid:" << xmlid << " reflow item:" << item->name();
-    KoTextEditor *editor = qobject_cast<KoTextEditor*>(kwcanvas()->toolProxy()->selection());
+    KoTextEditor *editor = qobject_cast<KoTextEditor*>(canvasBase()->toolProxy()->selection());
     if (!editor) {
         kDebug(30015) << "no editor, not reflowing rdf semantic item.";
         return;
@@ -1519,3 +1559,8 @@ void KWView::semanticObjectViewSiteUpdated(KoRdfSemanticItem* item, const QStrin
 #endif
 }
 
+void KWView::variableChanged(){
+    m_actionMenu->menu()->clear();
+    foreach (QAction *action, m_document->inlineTextObjectManager()->createInsertVariableActions(canvasBase()))
+        m_actionMenu->addAction(action);
+}
