@@ -22,24 +22,25 @@
 #include "KoShape.h"
 
 #include <QList>
-#include <QMatrix>
+#include <QTransform>
 
 #include <KDebug>
 
-class KoShapeTransformCommand::Private {
+class KoShapeTransformCommand::Private
+{
 public:
     Private(const QList<KoShape*> &list) : shapes(list) { }
     QList<KoShape*> shapes;
-    QList<QMatrix> oldState;
-    QList<QMatrix> newState;
+    QList<QTransform> oldState;
+    QList<QTransform> newState;
 };
 
-KoShapeTransformCommand::KoShapeTransformCommand( const QList<KoShape*> &shapes, const QList<QMatrix> &oldState, const QList<QMatrix> &newState, QUndoCommand * parent )
-    : QUndoCommand(parent),
-    d( new Private(shapes))
+KoShapeTransformCommand::KoShapeTransformCommand(const QList<KoShape*> &shapes, const QList<QTransform> &oldState, const QList<QTransform> &newState, QUndoCommand * parent)
+        : QUndoCommand(parent),
+        d(new Private(shapes))
 {
-    Q_ASSERT( shapes.count() == oldState.count() );
-    Q_ASSERT( shapes.count() == newState.count() );
+    Q_ASSERT(shapes.count() == oldState.count());
+    Q_ASSERT(shapes.count() == newState.count());
     d->oldState = oldState;
     d->newState = newState;
 }
@@ -54,11 +55,10 @@ void KoShapeTransformCommand::redo()
     QUndoCommand::redo();
 
     const int shapeCount = d->shapes.count();
-    for( int i = 0; i < shapeCount; ++i )
-    {
+    for (int i = 0; i < shapeCount; ++i) {
         KoShape * shape = d->shapes[i];
         shape->update();
-        shape->setTransformation( d->newState[i] );
+        shape->setTransformation(d->newState[i]);
         shape->update();
     }
 }
@@ -68,11 +68,10 @@ void KoShapeTransformCommand::undo()
     QUndoCommand::undo();
 
     const int shapeCount = d->shapes.count();
-    for( int i = 0; i < shapeCount; ++i )
-    {
+    for (int i = 0; i < shapeCount; ++i) {
         KoShape * shape = d->shapes[i];
         shape->update();
-        shape->setTransformation( d->oldState[i] );
+        shape->setTransformation(d->oldState[i]);
         shape->update();
     }
 }
