@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2004-2006 David Faure <faure@kde.org>
+ * Copyright (C) 2001-2006 David Faure <faure@kde.org>
  * Copyright (C) 2007,2009 Thomas Zander <zander@kde.org>
  * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2007 Pierre Ducroquet <pinaraf@gmail.com>
@@ -7,7 +7,7 @@
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
  * Copyright (C) 2009 Pierre Stirnweiss <pstirnweiss@googlemail.com>
-   Copyright (C) 2010 KO GmbH <ben.martin@kogmbh.com>
+ * Copyright (C) 2010 KO GmbH <ben.martin@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -250,7 +250,7 @@ KoTextLoader::KoTextLoader(KoShapeLoadingContext &context, KoDocumentRdfBase *rd
         d->textSharedData = dynamic_cast<KoTextSharedLoadingData *>(sharedData);
     }
 
-    kDebug(32500) << "sharedData" << sharedData << "textSharedData" << d->textSharedData;
+    //kDebug(32500) << "sharedData" << sharedData << "textSharedData" << d->textSharedData;
 
     if (!d->textSharedData) {
         d->textSharedData = new KoTextSharedLoadingData();
@@ -427,7 +427,8 @@ void KoTextLoader::loadParagraph(const KoXmlElement &element, QTextCursor &curso
         if (!styleName.isEmpty())
             kWarning(32500) << "paragraph style " << styleName << "not found - using default style";
         paragraphStyle = d->styleManager->defaultParagraphStyle();
-        kWarning(32500) << "defaultParagraphStyle not found - using default style";
+        if (paragraphStyle == 0)
+            kWarning(32500) << "defaultParagraphStyle not found - using default style";
     }
 
     QTextCharFormat cf = cursor.charFormat(); // store the current cursor char format
@@ -847,20 +848,16 @@ QString KoTextLoader::createUniqueBookmarkName(KoBookmarkManager* bmm, QString b
 void KoTextLoader::loadText(const QString &fulltext, QTextCursor &cursor,
                             bool *stripLeadingSpace, bool isLastNode)
 {
-#ifdef KOOPENDOCUMENTLOADER_DEBUG
-    kDebug(32500) << "  <text> localName=" << localName << " parent.localName=" << element.localName() << " text=" << text
-    << text.length();
-#endif
     QString text = KoTextLoaderP::normalizeWhitespace(fulltext, *stripLeadingSpace);
+#ifdef KOOPENDOCUMENTLOADER_DEBUG
+    kDebug(32500) << "  <text> text=" << text << text.length();
+#endif
 
     if (!text.isEmpty()) {
         // if present text ends with a space,
         // we can remove the leading space in the next text
         *stripLeadingSpace = text[text.length() - 1].isSpace();
 
-        if (d->changeTracker && d->changeStack.count())
-        qDebug() << "CT " << d->changeTracker << " " <<  d->changeStack.count();
-
         if (d->changeTracker && d->changeStack.count()) {
             QTextCharFormat format;
             format.setProperty(KoCharacterStyle::ChangeTrackerId, d->changeStack.top());
@@ -931,7 +928,7 @@ void KoTextLoader::loadSpan(const KoXmlElement &element, QTextCursor &cursor, bo
             KoCharacterStyle *characterStyle = d->textSharedData->characterStyle(styleName, d->stylesDotXml);
             if (characterStyle) {
                 characterStyle->applyStyle(&cursor);
-            } else {
+            } else if (!styleName.isEmpty()) {
                 kWarning(32500) << "character style " << styleName << " not found";
             }
 
@@ -1028,7 +1025,11 @@ void KoTextLoader::loadSpan(const KoXmlElement &element, QTextCursor &cursor, bo
                 } else if (localName == "bookmark-end") {
                     bookmark->setType(KoBookmark::EndBookmark);
                     KoBookmark *startBookmark = textObjectManager->bookmarkManager()->retrieveBookmark(uniqBookmarkName);
-                    startBookmark->setEndBookmark(bookmark);
+                    if (startBookmark) {        // set end bookmark only if we got start bookmark (we might not have in case of broken document)
+                        startBookmark->setEndBookmark(bookmark);
+                    } else {
+                        kWarning(32500) << "bookmark-end of non-existing bookmark - broken document?";
+                    }
                 }
                 textObjectManager->insertInlineObject(cursor, bookmark);
             }
@@ -1199,6 +1200,15 @@ void KoTextLoader::loadTable(const KoXmlElement &tableElem, QTextCursor &cursor)
                             }
                         }
                     }
+
+                    QString defaultCellStyleName = tblTag.attributeNS(KoXmlNS::table, "default-cell-style-name", "");
+                    if (!defaultCellStyleName.isEmpty()) {
+                        KoTableCellStyle *cellStyle = d->textSharedData->tableCellStyle(defaultCellStyleName, d->stylesDotXml);
+                        for (int c = columns; c < columns + repeatColumn; c++) {
+                            tcarManager->setDefaultColumnCellStyle(c, cellStyle);
+                        }
+                    }
+
                     columns = columns + repeatColumn;
                     if (rows > 0)
                         tbl->resize(rows, columns);
@@ -1212,6 +1222,13 @@ void KoTextLoader::loadTable(const KoXmlElement &tableElem, QTextCursor &cursor)
                             tcarManager->setRowStyle(rows, *rowStyle);
                         }
                     }
+
+                    QString defaultCellStyleName = tblTag.attributeNS(KoXmlNS::table, "default-cell-style-name", "");
+                    if (!defaultCellStyleName.isEmpty()) {
+                        KoTableCellStyle *cellStyle = d->textSharedData->tableCellStyle(defaultCellStyleName, d->stylesDotXml);
+                        tcarManager->setDefaultRowCellStyle(rows, cellStyle);
+                    }
+
                     rows++;
                     if (columns > 0)
                         tbl->resize(rows, columns);
@@ -1236,14 +1253,20 @@ void KoTextLoader::loadTable(const KoXmlElement &tableElem, QTextCursor &cursor)
 
                                     if (cell.isValid()) {
                                         QString cellStyleName = rowTag.attributeNS(KoXmlNS::table, "style-name", "");
+                                        KoTableCellStyle *cellStyle = 0;
                                         if (!cellStyleName.isEmpty()) {
-                                            KoTableCellStyle *cellStyle = d->textSharedData->tableCellStyle(cellStyleName, d->stylesDotXml);
-                                            QTextTableCellFormat cellFormat = cell.format().toTableCellFormat();
-                                            if (cellStyle)
-                                                cellStyle->applyStyle(cellFormat);
-                                            cell.setFormat(cellFormat);
+                                            cellStyle = d->textSharedData->tableCellStyle(cellStyleName, d->stylesDotXml);
+                                        } else if (tcarManager->defaultRowCellStyle(currentRow)) {
+                                            cellStyle = tcarManager->defaultRowCellStyle(currentRow);
+                                        } else if (tcarManager->defaultColumnCellStyle(currentCell)) {
+                                            cellStyle = tcarManager->defaultColumnCellStyle(currentCell);
                                         }
 
+                                        QTextTableCellFormat cellFormat = cell.format().toTableCellFormat();
+                                        if (cellStyle)
+                                            cellStyle->applyStyle(cellFormat);
+                                        cell.setFormat(cellFormat);
+
                                         // handle inline Rdf
                                         // rowTag is the current table cell.
                                         if (rowTag.hasAttributeNS(KoXmlNS::xhtml, "property")
