@@ -57,7 +57,6 @@ public:
     enum Property {
         StyleId = QTextFormat::UserProperty + 1, ///< The id stored in the charFormat to link the text to this style.
         HasHyphenation,
-        FontId,
         StrikeOutStyle,
         StrikeOutType,
         StrikeOutColor,
@@ -73,12 +72,10 @@ public:
         Language,
         Country,
         FontCharset,
-        DropCapsAdditionalFormatting, // to identify the purpose of the additionalFormatting
-        // set on the layout. is not copied during applyStyle().
         TextRotationAngle,
         TextRotationScale,
         TextScale,
-        InlineRdf,  ///< KoTextInlineRdf for this block
+        InlineRdf,  ///< KoTextInlineRdf pointer
         InlineInstanceId = 577297549, // Internal: Reserved for KoInlineTextObjectManager
         ChangeTrackerId = 577297550 // Internal: Reserved for ChangeTracker
     };
@@ -263,7 +260,7 @@ public:
     void setTextRotationAngle(RotationAngle angle);
     /// Get the current text rotation angle of this KoCharacterStyle
     RotationAngle textRotationAngle() const;
-    /** 
+    /**
      *  RotationScale pecifies whether for rotated text the width of the text
      *  should be scaled to fit into the current line height or the width of the text
      *  should remain fixed, therefore changing the current line height
