@@ -63,6 +63,7 @@
 #include <KDChartThreeDPieAttributes>
 #include <KDChartThreeDLineAttributes>
 #include <KDChartDataValueAttributes>
+#include <KDChartBackgroundAttributes>
 
 // KChart
 #include "PlotArea.h"
@@ -73,6 +74,7 @@
 #include "ChartProxyModel.h"
 #include "TextLabelDummy.h"
 #include "Layout.h"
+#include "OdfLoadingHelper.h"
 
 
 using namespace KChart;
@@ -107,8 +109,6 @@ public:
 
     // Pointer to Axis that owns this Private instance
     Axis * const q;
-    QMap< DataSet*, KDChart::MarkerAttributes > markerAttributesBackup;//QVector< KDChart::MarkerAttributes > > markerAttributesBackup;
-    QMap< DataSet*, KDChart::MarkerAttributes > markerAttributesGlobalBackup;//QVector< KDChart::MarkerAttributes > > markerAttributesBackup;
 
     PlotArea *plotArea;
 
@@ -118,6 +118,7 @@ public:
     KoShape *title;
     TextLabelData *titleData;
 
+    /// FIXME: Unused variable 'id', including id() getter
     QString id;
     QList<DataSet*> dataSets;
     qreal majorInterval;
@@ -132,6 +133,8 @@ public:
     bool useAutomaticMajorInterval;
     bool useAutomaticMinorInterval;
 
+    /// Font used for axis labels
+    /// TODO: Save to ODF
     QFont font;
 
     KDChart::CartesianAxis            *kdAxis;
@@ -175,7 +178,7 @@ public:
     ChartType     plotAreaChartType;
     ChartSubtype  plotAreaChartSubType;
 
-    QString categoryDataRegionString;
+    CellRegion categoryDataRegion;
 
     // If KDChart::LineDiagram::centerDataPoints() property is set to true,
     // the data points drawn in a line (i.e., also an area) diagram start at
@@ -246,75 +249,40 @@ Axis::Private::~Private()
 {
     Q_ASSERT( plotArea );
 
-    if ( kdPlane ) {
-        plotArea->kdChart()->takeCoordinatePlane( kdPlane );
-        delete kdPlane;
-    }
+    delete kdPlane;
+    delete kdPolarPlane;
 
-    if ( kdPolarPlane ) {
-        plotArea->kdChart()->takeCoordinatePlane( kdPolarPlane );
-        delete kdPolarPlane;
-    }
+    delete kdBarDiagram;
+    delete kdBarDiagramModel;
 
-    if ( kdBarDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdBarDiagram );
-        delete kdBarDiagram;
-        delete kdBarDiagramModel;
-    }
+    delete kdAreaDiagram;
+    delete kdAreaDiagramModel;
 
-    if ( kdAreaDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdAreaDiagram );
-        delete kdAreaDiagram;
-        delete kdAreaDiagramModel;
-    }
+    delete kdCircleDiagram;
+    delete kdCircleDiagramModel;
 
-    if ( kdCircleDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdCircleDiagram );
-        delete kdCircleDiagram;
-        delete kdCircleDiagramModel;
-    }
+    delete kdRingDiagram;
+    delete kdRingDiagramModel;
 
-    if ( kdRingDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdRingDiagram );
-        delete kdRingDiagram;
-        delete kdRingDiagramModel;
-    }
+    delete kdRadarDiagram;
+    delete kdRadarDiagramModel;
 
-    if ( kdRadarDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdRadarDiagram );
-        delete kdRadarDiagram;
-        delete kdRadarDiagramModel;
-    }
+    delete kdScatterDiagram;
+    delete kdScatterDiagramModel;
 
-    if ( kdScatterDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdScatterDiagram );
-        delete kdScatterDiagram;
-        delete kdScatterDiagramModel;
-    }
+    delete kdStockDiagram;
+    delete kdStockDiagramModel;
 
-    if ( kdStockDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdStockDiagram );
-        delete kdStockDiagram;
-        delete kdStockDiagramModel;
-    }
+    delete kdBubbleDiagram;
+    delete kdBubbleDiagramModel;
 
-    if ( kdBubbleDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdBubbleDiagram );
-        delete kdBubbleDiagram;
-        delete kdBubbleDiagramModel;
-    }
+    delete kdSurfaceDiagram;
+    delete kdSurfaceDiagramModel;
 
-    if ( kdSurfaceDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdSurfaceDiagram );
-        delete kdSurfaceDiagram;
-        delete kdSurfaceDiagramModel;
-    }
+    delete kdGanttDiagram;
+    delete kdGanttDiagramModel;
 
-    if ( kdGanttDiagram ) {
-        plotArea->parent()->legend()->kdLegend()->removeDiagram( kdGanttDiagram );
-        delete kdGanttDiagram;
-        delete kdGanttDiagramModel;
-    }
+    delete kdAxis;
 }
 
 void Axis::Private::registerDiagram( KDChart::AbstractDiagram *diagram )
@@ -909,7 +877,7 @@ void Axis::Private::createSurfaceDiagram()
         ...
     }
 #else // fallback to bar diagram for now
-    createBarDiagram();
+    if (!kdBarDiagramModel) createBarDiagram();
     kdSurfaceDiagramModel = kdBarDiagramModel;
     kdBarDiagramModel = 0;
     kdSurfaceDiagram = kdBarDiagram;
@@ -932,10 +900,10 @@ void Axis::Private::createGanttDiagram()
         ...
     }
 #else // fallback to bar diagram for now
-    createBarDiagram();
-    kdSurfaceDiagramModel = kdBarDiagramModel;
+    if (!kdBarDiagramModel) createBarDiagram();
+    kdGanttDiagramModel = kdBarDiagramModel;
     kdBarDiagramModel = 0;
-    kdSurfaceDiagram = kdBarDiagram;
+    kdGanttDiagram = kdBarDiagram;
     kdBarDiagram = 0;
 #endif
 }
@@ -967,8 +935,16 @@ Axis::Axis( PlotArea *parent )
 
     d->plotArea = parent;
     d->kdAxis       = new KDChart::CartesianAxis();
+    KDChart::BackgroundAttributes batt( d->kdAxis->backgroundAttributes() );
+    batt.setBrush( QBrush( Qt::white ) );
+    d->kdAxis->setBackgroundAttributes( batt );
     d->kdPlane      = new KDChart::CartesianCoordinatePlane();
     d->kdPolarPlane = new KDChart::PolarCoordinatePlane();
+    
+    //disable non circular axis
+    KDChart::GridAttributes gridAttributesNonCircular = d->kdPolarPlane->gridAttributes( false );
+    gridAttributesNonCircular.setGridVisible( false );
+    d->kdPolarPlane->setGridAttributes( false, gridAttributesNonCircular );
 
     d->plotAreaChartType    = d->plotArea->chartType();
     d->plotAreaChartSubType = d->plotArea->chartSubType();
@@ -983,13 +959,13 @@ Axis::Axis( PlotArea *parent )
     gridAttributes.setGridGranularitySequence( KDChartEnums::GranularitySequence_10_50 );
     d->kdPlane->setGridAttributes( Qt::Vertical, gridAttributes );
 
-    gridAttributes = d->kdPolarPlane->gridAttributes( Qt::Horizontal );
+    gridAttributes = d->kdPolarPlane->gridAttributes( true );
     gridAttributes.setGridVisible( false );
-    d->kdPolarPlane->setGridAttributes( Qt::Horizontal, gridAttributes );
+    d->kdPolarPlane->setGridAttributes( true, gridAttributes );
     
-    gridAttributes = d->kdPolarPlane->gridAttributes( Qt::Vertical );
-    gridAttributes.setGridVisible( false );
-    d->kdPolarPlane->setGridAttributes( Qt::Vertical, gridAttributes );
+//     gridAttributes = d->kdPolarPlane->gridAttributes( plotArea()->chartType() != RadarChartType );
+//     gridAttributes.setGridVisible( false );
+//     d->kdPolarPlane->setGridAttributes( plotArea()->chartType() != RadarChartType, gridAttributes );
 
     d->title = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShape(parent->parent()->resourceManager());
     if ( d->title ) {
@@ -1018,8 +994,6 @@ Axis::Axis( PlotArea *parent )
              this,        SLOT( setGapBetweenBars( int ) ) );
     connect( d->plotArea, SIGNAL( gapBetweenSetsChanged( int ) ),
              this,        SLOT( setGapBetweenSets( int ) ) );
-    connect( d->plotArea, SIGNAL( pieExplodeFactorChanged( DataSet*, int ) ),
-             this,        SLOT( setPieExplodeFactor( DataSet*, int ) ) );
     connect( d->plotArea, SIGNAL( pieAngleOffsetChanged( qreal ) ),
              this,        SLOT( setPieAngleOffset( qreal ) ) );
 }
@@ -1145,7 +1119,7 @@ bool Axis::attachDataSet( DataSet *dataSet, bool silent )
     d->dataSets.append( dataSet );
 
     if ( dimension() == XAxisDimension ) {
-        dataSet->setCategoryDataRegionString( d->categoryDataRegionString );
+        dataSet->setCategoryDataRegion( d->categoryDataRegion );
     }
     else if ( dimension() == YAxisDimension ) {
         dataSet->setAttachedAxis( this );
@@ -1161,7 +1135,6 @@ bool Axis::attachDataSet( DataSet *dataSet, bool silent )
         if( !model )
             return false;
 
-        dataSet->setKdDiagram( diagram );
         if ( model )
             model->addDataSet( dataSet, silent );
 
@@ -1182,11 +1155,9 @@ bool Axis::detachDataSet( DataSet *dataSet, bool silent )
     d->dataSets.removeAll( dataSet );
 
     if ( dimension() == XAxisDimension ) {
-        dataSet->setCategoryDataRegionString( "" );
+        dataSet->setCategoryDataRegion( CellRegion() );
     }
     else if ( dimension() == YAxisDimension ) {
-        dataSet->setAttachedAxis( 0 );
-
         ChartType chartType = dataSet->chartType();
         if ( chartType == LastChartType )
             chartType = d->plotAreaChartType;
@@ -1221,9 +1192,8 @@ bool Axis::detachDataSet( DataSet *dataSet, bool silent )
                 oldModel->removeDataSet( dataSet, silent );
         }
 
-        dataSet->setKdDiagram( 0 );
         dataSet->setKdChartModel( 0 );
-        dataSet->setKdDataSetNumber( -1 );
+        dataSet->setAttachedAxis( 0 );
 
         if ( !silent ) {
             layoutPlanes();
@@ -1234,6 +1204,13 @@ bool Axis::detachDataSet( DataSet *dataSet, bool silent )
     return true;
 }
 
+void Axis::clearDataSets()
+{
+    QList<DataSet*> list = d->dataSets;
+    foreach( DataSet *dataSet, list )
+        detachDataSet( dataSet, true );
+}
+
 qreal Axis::majorInterval() const
 {
     return d->majorInterval;
@@ -1254,9 +1231,9 @@ void Axis::setMajorInterval( qreal interval )
     attributes.setGridStepWidth( interval );
     d->kdPlane->setGridAttributes( orientation(), attributes );
 
-    attributes = d->kdPolarPlane->gridAttributes( orientation() );
+    attributes = d->kdPolarPlane->gridAttributes( true );
     attributes.setGridStepWidth( interval );
-    d->kdPolarPlane->setGridAttributes( orientation(), attributes );
+    d->kdPolarPlane->setGridAttributes( true, attributes );
 
     // FIXME: Hide minor tick marks more appropriately
     if ( !d->showMinorGrid && interval != 0.0 )
@@ -1297,9 +1274,9 @@ void Axis::setMinorIntervalDivisor( int divisor )
     attributes.setGridSubStepWidth( (divisor != 0) ? (d->majorInterval / divisor) : 0.0 );
     d->kdPlane->setGridAttributes( orientation(), attributes );
     
-    attributes = d->kdPolarPlane->gridAttributes( orientation() );
+    attributes = d->kdPolarPlane->gridAttributes( true );
     attributes.setGridSubStepWidth( (divisor != 0) ? (d->majorInterval / divisor) : 0.0 );
-    d->kdPolarPlane->setGridAttributes( orientation(), attributes );
+    d->kdPolarPlane->setGridAttributes( true, attributes );
 
     requestRepaint();
 }
@@ -1384,9 +1361,9 @@ void Axis::setShowMajorGrid( bool showGrid )
     attributes.setGridVisible( d->showMajorGrid );
     d->kdPlane->setGridAttributes( orientation(), attributes );
     
-    attributes = d->kdPolarPlane->gridAttributes( orientation() );
+    attributes = d->kdPolarPlane->gridAttributes( true );
     attributes.setGridVisible( d->showMajorGrid );
-    d->kdPolarPlane->setGridAttributes( orientation(), attributes );
+    d->kdPolarPlane->setGridAttributes( true, attributes );
 
     requestRepaint();
 }
@@ -1405,9 +1382,9 @@ void Axis::setShowMinorGrid( bool showGrid )
     attributes.setSubGridVisible( d->showMinorGrid );
     d->kdPlane->setGridAttributes( orientation(), attributes );
     
-    attributes = d->kdPolarPlane->gridAttributes( orientation() );
+    attributes = d->kdPolarPlane->gridAttributes( true );
     attributes.setSubGridVisible( d->showMinorGrid );
-    d->kdPolarPlane->setGridAttributes( orientation(), attributes );
+    d->kdPolarPlane->setGridAttributes( true, attributes );
 
     requestRepaint();
 }
@@ -1433,17 +1410,17 @@ Qt::Orientation Axis::orientation()
     return Qt::Vertical;
 }
 
-QString Axis::categoryDataRegionString() const
+CellRegion Axis::categoryDataRegion() const
 {
-    return d->categoryDataRegionString;
+    return d->categoryDataRegion;
 }
 
-void Axis::setCategoryDataRegionString( const QString &region )
+void Axis::setCategoryDataRegion( const CellRegion &region )
 {
-    d->categoryDataRegionString = region;
+    d->categoryDataRegion = region;
 
     foreach( DataSet *dataSet, d->dataSets )
-        dataSet->setCategoryDataRegionString( region );
+        dataSet->setCategoryDataRegion( region );
 }
 
 bool Axis::loadOdf( const KoXmlElement &axisElement, KoShapeLoadingContext &context )
@@ -1451,6 +1428,8 @@ bool Axis::loadOdf( const KoXmlElement &axisElement, KoShapeLoadingContext &cont
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
+    OdfLoadingHelper *helper = (OdfLoadingHelper*)context.sharedData( OdfLoadingHelperId );
+
     d->title->setVisible( false );
     
     QPen gridPen(Qt::NoPen);
@@ -1533,7 +1512,7 @@ bool Axis::loadOdf( const KoXmlElement &axisElement, KoShapeLoadingContext &cont
                     styleStack.setTypeProperties( "graphic" );
                     if ( styleStack.hasProperty( KoXmlNS::svg, "stroke-color" ) ) {
                         const QString strokeColor = styleStack.property( KoXmlNS::svg, "stroke-color" );
-                        d->showMajorGrid = true;
+                        //d->showMajorGrid = true;
                         if ( major )
                             gridPen = QPen( QColor( strokeColor ) );
                         else
@@ -1542,8 +1521,10 @@ bool Axis::loadOdf( const KoXmlElement &axisElement, KoShapeLoadingContext &cont
                 }
             }
             else if ( n.localName() == "categories" ) {
-                if ( n.hasAttributeNS( KoXmlNS::table, "cell-range-address" ) )
-                    setCategoryDataRegionString( n.attributeNS( KoXmlNS::table, "cell-range-address" ) );
+                if ( n.hasAttributeNS( KoXmlNS::table, "cell-range-address" ) ) {
+                    const CellRegion region = CellRegion( helper->tableSource, n.attributeNS( KoXmlNS::table, "cell-range-address" ) );
+                    setCategoryDataRegion( region );
+                }
             }
         }
 
@@ -1607,7 +1588,10 @@ bool Axis::loadOdf( const KoXmlElement &axisElement, KoShapeLoadingContext &cont
         gridAttr.setGridPen( gridPen );
     if ( subGridPen.style() != Qt::NoPen )
         gridAttr.setSubGridPen( subGridPen );
-    d->kdPolarPlane->setGridAttributes( orientation(), gridAttr );
+//     if ( plotArea()->chartType() == RadarChartType )
+//         d->kdPolarPlane->setGridAttributes( false, gridAttr );
+//     else
+    d->kdPolarPlane->setGridAttributes( true, gridAttr );
 
     if ( !loadOdfChartSubtypeProperties( axisElement, context ) )
         return false;
@@ -1757,6 +1741,9 @@ KDChart::AbstractCoordinatePlane *Axis::kdPlane() const
 
 void Axis::plotAreaChartTypeChanged( ChartType newChartType )
 {
+    if ( dimension() != YAxisDimension )
+        return;
+
     // Return if there's nothing to do
     if ( newChartType == d->plotAreaChartType )
         return;
@@ -1900,15 +1887,13 @@ void Axis::plotAreaChartTypeChanged( ChartType newChartType )
         if ( !d->kdSurfaceDiagram )
            d->createSurfaceDiagram();
         newModel = d->kdSurfaceDiagramModel;
-        //newDiagram = d->kdSurfaceDiagram;
-        newDiagram = 0;
+        newDiagram = d->kdSurfaceDiagram;
         break;
     case GanttChartType:
         if ( !d->kdGanttDiagram )
            d->createGanttDiagram();
         newModel = d->kdGanttDiagramModel;
-        //newDiagram = d->kdGanttDiagram;
-        newDiagram = 0;
+        newDiagram = d->kdGanttDiagram;
         break;
     default:;
         // FIXME: Implement more chart types
@@ -1934,11 +1919,16 @@ void Axis::plotAreaChartTypeChanged( ChartType newChartType )
         if ( dataSet->chartType() != LastChartType )
             continue;
 
+// FIXME: What does this do? Only the user may set a data set's pen through
+// a proper UI, in any other case the pen falls back to a default
+// which depends on the chart type, so setting it here will break the default
+// for other chart types.
+#if 0
         Qt::PenStyle newPenStyle = newDiagram->pen().style();
         QPen newPen = dataSet->pen();
         newPen.setStyle( newPenStyle );
         dataSet->setPen(  newPen );
-        dataSet->setKdDiagram( newDiagram );
+#endif
         newModel->addDataSet( dataSet );        
         if ( oldModel && *oldModel ) {
             const int dataSetCount = (*oldModel)->dataDirection() == Qt::Vertical
@@ -2180,18 +2170,6 @@ void Axis::setGapBetweenSets( int percent )
     requestRepaint();
 }
 
-void Axis::setPieExplodeFactor( DataSet *dataSet, int percent )
-{
-    if ( d->kdCircleDiagram ) {
-        KDChart::PieAttributes attributes = d->kdCircleDiagram->pieAttributes();
-        attributes.setExplodeFactor( (float)percent / 100.0 );
-        d->kdCircleDiagram->setPieAttributes( dataSet->kdDataSetNumber(),
-                                              attributes );
-    }
-
-    requestRepaint();
-}
-
 void Axis::setPieAngleOffset( qreal angle )
 {
     // only set if we already have a diagram else the value will be picked up on creating the diagram
