@@ -103,6 +103,8 @@
 #include "ChartDocument.h"
 #include "ChartTableModel.h"
 #include "Layout.h"
+#include "TableSource.h"
+#include "OdfLoadingHelper.h"
 
 
 // Define the protocol used here for embedded documents' URL
@@ -255,8 +257,11 @@ public:
     PlotArea  *plotArea;
 
     // Data
-    QAbstractItemModel  *internalModel; // The actual data
-    ChartProxyModel     *proxyModel;	// What's presented to KDChart
+    ChartProxyModel     *proxyModel;	 /// What's presented to KDChart
+    QAbstractItemModel  *internalModel;
+    TableSource          tableSource;
+
+    bool usesInternalModelOnly; /// @see usesInternalModelOnly()
 
     ChartDocument *document;
 
@@ -267,7 +272,8 @@ public:
 
 
 ChartShape::Private::Private( ChartShape *shape )
-    : resourceManager(0)
+    : internalModel(0),
+    resourceManager(0)
 {
     // Register the owner.
     this->shape = shape;
@@ -280,9 +286,11 @@ ChartShape::Private::Private( ChartShape *shape )
     plotArea = 0;
 
     // Data
-    internalModel = 0;
     proxyModel    = 0;
 
+    // If not explicitly set otherwise, this chart provides its own data.
+    usesInternalModelOnly = true;
+
     document = 0;
 }
 
@@ -366,7 +374,7 @@ ChartShape::ChartShape(KoResourceManager *resourceManager)
     setShapeId( ChartShapeId );
 
     // Instantiated all children first
-    d->proxyModel = new ChartProxyModel();
+    d->proxyModel = new ChartProxyModel( &d->tableSource );
 
     d->plotArea = new PlotArea( this );
     d->document = new ChartDocument( this );
@@ -504,22 +512,12 @@ ChartShape::~ChartShape()
     delete d->plotArea;
 
     delete d->proxyModel;
-    delete d->internalModel;    // Ok to call even when 0.
 
     delete d->document;
 
     delete d;
 }
 
-
-QAbstractItemModel *ChartShape::model() const
-{
-    // Can't return d->internalModel because the data may come from
-    // the outside, e.g. a spreadsheet.  We only use d->internalModel
-    // for a model that we own ourselves.
-    return d->proxyModel->sourceModel();
-}
-
 ChartProxyModel *ChartShape::proxyModel() const
 {
     return d->proxyModel;
@@ -606,55 +604,25 @@ void ChartShape::showFooter(bool doShow)
     d->showLabel( d->footer, doShow );
 }
 
-void ChartShape::setModel( QAbstractItemModel *model,
-                           bool takeOwnershipOfModel )
+QAbstractItemModel *ChartShape::internalModel() const
 {
-    Q_ASSERT( model );
-    //kDebug(35001) << "Setting" << model << "as chart model.";
-
-    // Only do something if we are not already using the new model.
-    if ( model == d->internalModel )
-        return;
-
-    // If we already have an old internal model, delete it first.
-    if ( d->internalModel ) {
-        delete d->internalModel;
-    }
-
-    d->internalModel = ( takeOwnershipOfModel ? model : 0 );
-
-    d->proxyModel->setSourceModel( model );
-
-    requestRepaint();
+    return d->internalModel;
 }
 
-void ChartShape::setModel( QAbstractItemModel *model,
-                           const QVector<QRect> &selection )
+void ChartShape::setInternalModel( QAbstractItemModel *model )
 {
-    Q_ASSERT( model );
-    kDebug(35001) << "Setting" << model << "as chart model.";
-    kDebug(35001) << "Selection:" << selection;
-
-    d->proxyModel->setSourceModel( model, selection );
-
-    if ( d->internalModel ) {
-        delete d->internalModel;
-        d->internalModel = 0;
-    }
-
-    requestRepaint();
+    Q_ASSERT( !d->internalModel );
+    d->internalModel = model;
 }
 
-bool ChartShape::addAxis( Axis *axis )
+TableSource *ChartShape::tableSource() const
 {
-    Q_ASSERT( d->plotArea );
-    return d->plotArea->addAxis( axis );
+    return &d->tableSource;
 }
 
-bool ChartShape::removeAxis( Axis *axis )
+bool ChartShape::usesInternalModelOnly() const
 {
-    Q_ASSERT( d->plotArea );
-    return d->plotArea->removeAxis( axis );
+    return d->usesInternalModelOnly;
 }
 
 
@@ -680,24 +648,23 @@ bool ChartShape::isThreeD() const
     return d->plotArea->isThreeD();
 }
 
-void ChartShape::setFirstRowIsLabel( bool isLabel )
+void ChartShape::setSheetAccessModel( QAbstractItemModel *model )
 {
-    d->proxyModel->setFirstRowIsLabel( isLabel );
-
-    requestRepaint();
+    d->tableSource.setSheetAccessModel( model );
 }
 
-void ChartShape::setFirstColumnIsLabel( bool isLabel )
+void ChartShape::reset( const QString &region,
+                        bool firstRowIsLabel,
+                        bool firstColumnIsLabel,
+                        Qt::Orientation dataDirection )
 {
-    d->proxyModel->setFirstColumnIsLabel( isLabel );
-
-    requestRepaint();
-}
-
-void ChartShape::setDataDirection( Qt::Orientation orientation )
-{
-    Q_ASSERT( d->proxyModel );
-    d->proxyModel->setDataDirection( orientation );
+    // This method is provided via KoChartInterface, which is
+    // used by embedding applications.
+    d->usesInternalModelOnly = false;
+    d->proxyModel->setFirstRowIsLabel( firstRowIsLabel );
+    d->proxyModel->setFirstColumnIsLabel( firstColumnIsLabel );
+    d->proxyModel->setDataDirection( dataDirection );
+    d->proxyModel->reset( CellRegion( &d->tableSource, region ) );
 }
 
 void ChartShape::setChartType( ChartType type )
@@ -906,21 +873,14 @@ bool ChartShape::loadEmbeddedDocument( KoStore *store,
 bool ChartShape::loadOdf( const KoXmlElement &element,
                           KoShapeLoadingContext &context )
 {
-    // Invalidate all data sets before we new load data sets from ODF
-    // and *before* we delete existing axes.
-    proxyModel()->invalidateDataSets();
-
-    // When loading from ODF, all data sets are added explicitly.
-    bool autoCreation = proxyModel()->automaticDataSetCreation();
-    proxyModel()->setAutomaticDataSetCreation( false );
+    proxyModel()->beginLoading();
 
     // Load common attributes of (frame) shapes.  If you change here,
     // don't forget to also change in saveOdf().
     loadOdfAttributes( element, context, OdfAllAttributes );
     bool result = loadOdfFrame( element, context );
 
-    // Restore previous setting
-    proxyModel()->setAutomaticDataSetCreation( autoCreation );
+    proxyModel()->endLoading();
 
     return result;
 }
@@ -942,6 +902,26 @@ bool ChartShape::loadOdfFrameElement( const KoXmlElement &element,
 bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
                                   KoShapeLoadingContext &context )
 {
+    // The shared data will automatically be deleted in the destructor
+    // of KoShapeLoadingContext
+    OdfLoadingHelper *helper = new OdfLoadingHelper;
+    helper->tableSource = &d->tableSource;
+    helper->chartUsesInternalModelOnly = d->usesInternalModelOnly;
+
+    // Get access to sheets in KSpread
+    QAbstractItemModel *sheetAccessModel = 0;
+    if ( resourceManager()->hasResource( 75751149 ) ) { // duplicated from kspread
+        QVariant var = resourceManager()->resource( 75751149 );
+        sheetAccessModel = static_cast<QAbstractItemModel*>( var.value<void*>() );
+        if ( sheetAccessModel ) {
+            // We're embedded in KSpread, which means KSpread provides the data
+            d->usesInternalModelOnly = false;
+            d->tableSource.setSheetAccessModel( sheetAccessModel );
+            helper->chartUsesInternalModelOnly = d->usesInternalModelOnly;
+        }
+    }
+    context.addSharedData( OdfLoadingHelperId, helper );
+
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
@@ -1093,11 +1073,24 @@ bool ChartShape::loadOdfData( const KoXmlElement &tableElement,
     if ( tableElement.isNull() || !tableElement.isElement() )
         return true;
 
-    // FIXME: Make model->loadOdf() return a bool, and use it here.
-    ChartTableModel *model = new ChartTableModel;
-    model->loadOdf( tableElement, context );
+    Table *oldInternalTable = d->tableSource.get( d->internalModel );
+    d->tableSource.remove( oldInternalTable->name() );
+    delete d->internalModel;
 
-    setModel( model, true );
+    // FIXME: Make model->loadOdf() return a bool, and use it here.
+    // Create a table with data from document, add it as table source
+    // and reset the proxy only with data from this new table.
+    ChartTableModel *internalModel = new ChartTableModel;
+    d->internalModel = internalModel;
+    internalModel->loadOdf( tableElement, context );
+    int rows = internalModel->rowCount();
+    int cols = internalModel->columnCount();
+
+    QString tableName = tableElement.attributeNS( KoXmlNS::table, "name" );
+    Table *table = d->tableSource.add( tableName, internalModel );
+    // TODO: d->tableSource.setAvoidNameClash( tableName )
+    CellRegion region( table, QRect( 1, 1, cols, rows ) );
+    d->proxyModel->reset( region );
 
     return true;
 }
@@ -1176,12 +1169,15 @@ void ChartShape::saveOdfData( KoXmlWriter &bodyWriter, KoGenStyles &mainStyles )
 {
     Q_UNUSED( mainStyles );
 
+    // FIXME: Move this method to a sane place
+    QAbstractItemModel *internalModel = d->internalModel;
+
     // Only save the data if we actually have some.
-    if ( !d->internalModel )
+    if ( !internalModel )
         return;
 
-    const int cols = d->internalModel->columnCount();
-    const int rows = d->internalModel->rowCount();
+    const int cols = internalModel->columnCount();
+    const int rows = internalModel->rowCount();
 
     bodyWriter.startElement( "table:table" );
     bodyWriter.addAttribute( "table:name", "local-table" );
@@ -1220,9 +1216,9 @@ void ChartShape::saveOdfData( KoXmlWriter &bodyWriter, KoGenStyles &mainStyles )
     for ( int row = 0; row < rows ; ++row ) {
         bodyWriter.startElement( "table:table-row" );
         for ( int col = 0; col < cols; ++col ) {
-            //QVariant value( d->internalModel.cellVal( row, col ) );
-            QModelIndex  index = d->internalModel->index( row, col );
-            QVariant     value = d->internalModel->data( index );
+            //QVariant value( internalModel.cellVal( row, col ) );
+            QModelIndex  index = internalModel->index( row, col );
+            QVariant     value = internalModel->data( index );
 
             QString  valType;
             QString  valStr;
