@@ -60,7 +60,7 @@ class KDChartModel;
 class CHARTSHAPELIB_EXPORT DataSet
 {
 public:
-    DataSet( ChartProxyModel *model, int dataSetNr );
+    DataSet( int dataSetNr );
     ~DataSet();
 
     // Getter methods
@@ -69,8 +69,6 @@ public:
     ChartSubtype  chartSubType() const;
     Axis         *attachedAxis() const;
 
-    ChartProxyModel   *model() const;
-
     /**
      * Describes ODF attribute chart:data-label-number from §15.32.3
      */
@@ -162,7 +160,6 @@ public:
     void setPen( int section, const QPen &pen );
     void setBrush( int section, const QBrush &brush );
     void setColor( const QColor &color );
-    void setNumber( int num );
 
     void setPieExplodeFactor( int factor );
     void setPieExplodeFactor( int section, int factor );
@@ -193,33 +190,15 @@ public:
     CellRegion labelDataRegion() const;
     // TODO: Region for custom colors
 
-    QString xDataRegionString() const;
-    QString yDataRegionString() const;
-    QString customDataRegionString() const;
-    QString categoryDataRegionString() const;
-    QString labelDataRegionString() const;
-
     void setXDataRegion( const CellRegion &region );
     void setYDataRegion( const CellRegion &region );
     void setCustomDataRegion( const CellRegion &region );
     void setCategoryDataRegion( const CellRegion &region );
     void setLabelDataRegion( const CellRegion &region );
 
-    void setXDataRegionString( const QString &region );
-    void setYDataRegionString( const QString &region );
-    void setCustomDataRegionString( const QString &region );
-    void setCategoryDataRegionString( const QString &region );
-    void setLabelDataRegionString( const QString &region );
-
     int size() const;
     int dimension() const;
 
-    void setKdDiagram( KDChart::AbstractDiagram *diagram );
-    void setKdDataSetNumber( int number );
-
-    KDChart::AbstractDiagram *kdDiagram() const;
-    int kdDataSetNumber() const;
-
     // Called by the proxy model
     void yDataChanged( int start, int end ) const;
     void xDataChanged( int start, int end ) const;
@@ -227,6 +206,7 @@ public:
     void labelDataChanged() const;
     void categoryDataChanged( int start, int end ) const;
 
+    // FIXME: The parameter 'region' is unused in the methods below
     void yDataChanged( const QRect &region ) const;
     void xDataChanged( const QRect &region ) const;
     void customDataChanged( const QRect &region ) const;
