@@ -76,6 +76,7 @@
 #include "ChartProxyModel.h"
 #include "ScreenConversions.h"
 #include "Layout.h"
+#include "OdfLoadingHelper.h"
 
 using namespace KChart;
 
@@ -128,7 +129,6 @@ public:
     int   gapBetweenSets;
     
     // 2. Pie charts
-    int   pieExplodeFactor;     // in percents
     // TODO: Load+Save
     qreal pieAngleOffset;       // in degrees
 
@@ -178,15 +178,20 @@ PlotArea::Private::Private( PlotArea *q, ChartShape *parent )
 
 PlotArea::Private::~Private()
 {
+    foreach( Axis *axis, axes )
+        delete axis;
     delete kdPlane;
     delete kdChart;
+    delete wall;
+    delete floor;
+    delete threeDScene;
 }
 
 void PlotArea::Private::initAxes()
 {
     // The category data region is anchored to an axis and will be set on addAxis if the
-    // axis defines the Axis::categoryDataRegionString(). So, clear it now.
-    q->proxyModel()->setCategoryDataRegion(QString());
+    // axis defines the Axis::categoryDataRegion(). So, clear it now.
+    q->proxyModel()->setCategoryDataRegion( CellRegion() );
     // Remove all old axes
     while( !axes.isEmpty() ) {
         Axis *axis = axes.takeLast();
@@ -217,11 +222,15 @@ PlotArea::PlotArea( ChartShape *parent )
     Q_ASSERT( d->shape->proxyModel() );
 
     connect( d->shape->proxyModel(), SIGNAL( modelReset() ),
-             this,                   SLOT( dataSetCountChanged() ) );
+             this,                   SLOT( proxyModelStructureChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( rowsInserted( const QModelIndex, int, int ) ),
-             this,                   SLOT( dataSetCountChanged() ) );
+             this,                   SLOT( proxyModelStructureChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( rowsRemoved( const QModelIndex, int, int ) ),
-             this,                   SLOT( dataSetCountChanged() ) );
+             this,                   SLOT( proxyModelStructureChanged() ) );
+    connect( d->shape->proxyModel(), SIGNAL( columnsInserted( const QModelIndex, int, int ) ),
+             this,                   SLOT( proxyModelStructureChanged() ) );
+    connect( d->shape->proxyModel(), SIGNAL( columnsRemoved( const QModelIndex, int, int ) ),
+             this,                   SLOT( proxyModelStructureChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( columnsInserted( const QModelIndex, int, int ) ),
              this,                   SLOT( plotAreaUpdate() ) );
     connect( d->shape->proxyModel(), SIGNAL( columnsRemoved( const QModelIndex, int, int ) ),
@@ -251,15 +260,35 @@ void PlotArea::plotAreaInit()
     d->initAxes();
 }
 
-void PlotArea::dataSetCountChanged()
+void PlotArea::proxyModelStructureChanged()
 {
-    if ( !yAxis() )
-        return;
+    Q_ASSERT( xAxis() && yAxis() );
+    QMap<DataSet*, Axis*> attachedAxes;
+    QList<DataSet*> dataSets = proxyModel()->dataSets();
 
-    foreach( DataSet *dataSet, proxyModel()->dataSets() ) {
-        if ( !dataSet->attachedAxis() ) {
+    // Remember to what y axis each data set belongs
+    foreach( DataSet *dataSet, dataSets )
+        attachedAxes.insert( dataSet, dataSet->attachedAxis() );
+
+    // Proxy structure and thus data sets changed, drop old state and
+    // clear all axes of data sets
+    foreach( Axis *axis, axes() )
+        axis->clearDataSets();
+
+    // FIXME: Maybe this shouldn't be a property of an axis. After all
+    // there should be exactly one x axis and one region for categories.
+    // See note in Axis::setCategoryDataRegion()
+    // Categories might have been inserted or removed from the proxy model.
+//     xAxis()->setCategoryDataRegion( proxyModel()->categoryDataRegion() );
+
+    // Now add the new list of data sets to the axis they belong to
+    foreach( DataSet *dataSet, dataSets ) {
+        xAxis()->attachDataSet( dataSet );
+        // If they weren't assigned to a y axis before, use default y axis
+        if ( attachedAxes[dataSet] )
+            attachedAxes[dataSet]->attachDataSet( dataSet );
+        else
             yAxis()->attachDataSet( dataSet );
-        }
     }
 }
 
@@ -279,11 +308,6 @@ QList<DataSet*> PlotArea::dataSets() const
     return proxyModel()->dataSets();
 }
 
-int PlotArea::dataSetCount() const
-{
-    return proxyModel()->dataSets().size();
-}
-
 Axis *PlotArea::xAxis() const
 {
     foreach( Axis *axis, d->axes ) {
@@ -397,8 +421,8 @@ bool PlotArea::addAxis( Axis *axis )
     if ( axis->dimension() == XAxisDimension ) {
         // set the categoryDataRegion of the proxyModel. This will then be used on
         // ChartProxyModel::createDataSetsFromRegion to create the dataSets.
-        if ( proxyModel()->categoryDataRegion().isEmpty() && ! axis->categoryDataRegionString().isEmpty() )
-            proxyModel()->setCategoryDataRegion(axis->categoryDataRegionString());
+        if ( !proxyModel()->categoryDataRegion().isValid() && axis->categoryDataRegion().isValid() )
+            proxyModel()->setCategoryDataRegion( axis->categoryDataRegion() );
 
         // let each axis know about the other axis
         foreach ( Axis *_axis, d->axes ) {
@@ -431,11 +455,11 @@ bool PlotArea::removeAxis( Axis *axis )
     
     if ( axis->dimension() == XAxisDimension ) {
         // If the axis is removed we probably need to update the used categoryDataRegion too.
-        if ( ! proxyModel()->categoryDataRegion().isEmpty() && proxyModel()->categoryDataRegion() == axis->categoryDataRegionString() ) {
-            proxyModel()->setCategoryDataRegion(QString());
+        if ( proxyModel()->categoryDataRegion().isValid() && proxyModel()->categoryDataRegion() == axis->categoryDataRegion() ) {
+            proxyModel()->setCategoryDataRegion( CellRegion() );
             foreach ( Axis *_axis, d->axes ) {
-                 if ( _axis->dimension() == XAxisDimension && ! _axis->categoryDataRegionString().isEmpty()) {
-                     proxyModel()->setCategoryDataRegion( _axis->categoryDataRegionString() );
+                 if ( _axis->dimension() == XAxisDimension && _axis->categoryDataRegion().isValid()) {
+                     proxyModel()->setCategoryDataRegion( _axis->categoryDataRegion() );
                      break;
                  }
             }
@@ -553,6 +577,8 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
+    OdfLoadingHelper *helper = (OdfLoadingHelper*)context.sharedData( OdfLoadingHelperId );
+
     styleStack.clear();
 
     // First step is to load the axis. Datasets are attached to an
@@ -562,6 +588,8 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
     while ( !d->axes.isEmpty() ) {
         Axis *axis = d->axes.takeLast();
         Q_ASSERT( axis );
+        // Clear this axis of all data sets, deleting any diagram associated with it.
+        axis->clearDataSets();
         if ( axis->title() )
             d->automaticallyHiddenAxisTitles.removeAll( axis->title() );
         delete axis;
@@ -595,7 +623,8 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
 
     CellRegion cellRangeAddress;
     if ( plotAreaElement.hasAttributeNS( KoXmlNS::table, "cell-range-address" ) ) {
-        cellRangeAddress = CellRegion( plotAreaElement.attributeNS( KoXmlNS::table, "cell-range-address" ) );
+        cellRangeAddress = CellRegion( helper->tableSource, plotAreaElement.attributeNS( KoXmlNS::table, "cell-range-address" ) );
+//         setCellRangeAddress( cellRangeAddress );
     }
 
     // Find out about things that are in the plotarea style.
@@ -707,30 +736,7 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
         proxyModel()->setFirstColumnIsLabel( false );
     }
 
-    QAbstractItemModel *sheetAccessModel = 0;
-    if (d->shape->resourceManager()->hasResource(75751149)) { // duplicated from kspread
-        QVariant var = d->shape->resourceManager()->resource(75751149);
-        sheetAccessModel = static_cast<QAbstractItemModel*>(var.value<void*>());
-    }
-
     setCellRangeAddress( cellRangeAddress );
-
-    if ( sheetAccessModel ) {
-        const QString sheetName = cellRangeAddress.sheetName();
-        int sheetIndex = 0;
-        // Find sheet that this cell range address is associated with
-        if ( !sheetName.isEmpty() ) {
-            while ( sheetIndex + 1 < sheetAccessModel->columnCount() &&
-                    sheetAccessModel->headerData( sheetIndex, Qt::Horizontal ) != sheetName )
-                sheetIndex++;
-        }
-        QPointer<QAbstractItemModel> sheet = sheetAccessModel->data( sheetAccessModel->index( 0, sheetIndex ) ).value< QPointer<QAbstractItemModel> >();
-
-        // If sheet can't be found, we'll stay with the back-up model loaded from the
-        // chart document.
-        if ( sheet )
-            d->shape->setModel( sheet.data() );
-    }
     
     // Now, after the axes, load the datasets.
     // Note that this only contains properties of the datasets, the
@@ -979,13 +985,6 @@ void PlotArea::setGapBetweenSets( int percent )
     emit gapBetweenSetsChanged( percent );
 }
 
-void PlotArea::setPieExplodeFactor( DataSet *dataSet, int percent )
-{
-    d->pieExplodeFactor = percent;
-
-    emit pieExplodeFactorChanged( dataSet, percent );
-}
-
 void PlotArea::setPieAngleOffset( qreal angle )
 {
     d->pieAngleOffset = angle;
