@@ -655,6 +655,13 @@ void TextTool::mousePressEvent(KoPointerEvent *event)
     }
     updateActions();
 
+    //activate context-menu for spelling-suggestions
+    if (event->button() == Qt::RightButton) {
+        KoTextEditingPlugin *plugin = m_textEditingPlugins->spellcheck();
+        if (plugin)
+            plugin->setCurrentCursorPosition(m_textEditor.data()->document(), m_textEditor.data()->position());
+    }
+
     if (event->button() ==  Qt::MidButton) // Paste
         paste();
     else
@@ -923,9 +930,9 @@ void TextTool::mouseReleaseEvent(KoPointerEvent *event)
 
             event->accept();
             new KRun(m_textEditor.data()->charFormat().anchorHref(), 0);
-                m_textEditor.data()->setPosition(0);
-                ensureCursorVisible();
-                return;
+            m_textEditor.data()->setPosition(0);
+            ensureCursorVisible();
+            return;
         } else {
             QStringList anchorList = m_textEditor.data()->charFormat().anchorNames();
             QString anchorName;
@@ -1433,10 +1440,10 @@ QRectF TextTool::textRect(int startPosition, int endPosition) const
     if (startPosition > endPosition)
         qSwap(startPosition, endPosition);
     QTextBlock block = m_textShapeData->document()->findBlock(startPosition);
-    if (!block.isValid() || block.length() == 1) // length of 1 means only linefeed. I.e. empty
+    if (!block.isValid())
         return QRectF();
     QTextLine line1 = block.layout()->lineForTextPosition(startPosition - block.position());
-    if (! line1.isValid())
+    if (!line1.isValid())
         return QRectF();
     qreal startX = line1.cursorToX(startPosition - block.position());
     if (startPosition == endPosition)
