@@ -27,9 +27,11 @@
 
 #include "KoShape.h"
 #include <KoShapeContainer.h>
+#include <KoTextOnShapeContainer.h>
 #include "KoConnectionShape.h"
 #include "kdebug.h"
 
+
 Layout::Layout(KoShapeContainer *container)
     : m_container(container)
     , m_structure(TreeShape::FollowParent)
@@ -90,6 +92,7 @@ void Layout::attachConnector(KoShape* shape, KoConnectionShape *connector)
 void Layout::setRoot(KoShape *shape, TreeShape::RootType type)
 {
     m_root = shape;
+    m_rootShape = dynamic_cast<KoTextOnShapeContainer *>(shape)->shapes().first();
     m_rootType = type;
     m_container->setSize(m_root->size());
     m_lastWidth = m_container->size().width();
@@ -292,9 +295,9 @@ QList<KoShape*> Layout::shapes() const
 {
     //kDebug() << "";
     QList<KoShape*> all;
+    all.append(m_root);
     all.append(m_children);
     all.append(m_connectors);
-    all.append(m_root);
     return all;
 }
 
@@ -438,9 +441,10 @@ void Layout::buildOrgUp()
                     -fromParentToChild;
         child->setPosition(QPointF(x, y));
         KoConnectionShape *connector = m_bonds[child];
-        connector->connectFirst(m_root,0);
+        connector->connectFirst(m_rootShape,0);
         TreeShape *tree = dynamic_cast<TreeShape*>(child);
-        connector->connectSecond(tree->root(),2);
+        KoShape *secondShape = dynamic_cast<KoTextOnShapeContainer*>(tree->root())->shapes().first();
+        connector->connectSecond(secondShape,2);
         connector->updateConnections();
         x += child->size().width() + fromChildToChild;
     }
@@ -477,9 +481,10 @@ void Layout::buildOrgDown()
         qreal y = fromParentToChild+m_root->size().height();
         child->setPosition(QPointF(x, y));
         KoConnectionShape *connector = m_bonds[child];
-        connector->connectFirst(m_root,2);
+        connector->connectFirst(m_rootShape,2);
         TreeShape *tree = dynamic_cast<TreeShape*>(child);
-        connector->connectSecond(tree->root(),0);
+        KoShape *secondShape = dynamic_cast<KoTextOnShapeContainer*>(tree->root())->shapes().first();
+        connector->connectSecond(secondShape,0);
         connector->updateConnections();
         x += child->size().width() + fromChildToChild;
     }
@@ -520,9 +525,10 @@ void Layout::buildOrgLeft()
                    -fromParentToChild;
         child->setPosition(QPointF(x, y));
         KoConnectionShape *connector = m_bonds[child];
-        connector->connectFirst(m_root,3);
+        connector->connectFirst(m_rootShape,3);
         TreeShape *tree = dynamic_cast<TreeShape*>(child);
-        connector->connectSecond(tree->root(),1);
+        KoShape *secondShape = dynamic_cast<KoTextOnShapeContainer*>(tree->root())->shapes().first();
+        connector->connectSecond(secondShape,1);
         connector->updateConnections();
         y += child->size().height() + fromChildToChild;
     }
@@ -559,9 +565,10 @@ void Layout::buildOrgRight()
         qreal x = fromParentToChild+m_root->size().width();
         child->setPosition(QPointF(x, y));
         KoConnectionShape *connector = m_bonds[child];
-        connector->connectFirst(m_root,1);
+        connector->connectFirst(m_rootShape,1);
         TreeShape *tree = dynamic_cast<TreeShape*>(child);
-        connector->connectSecond(tree->root(),3);
+        KoShape *secondShape = dynamic_cast<KoTextOnShapeContainer*>(tree->root())->shapes().first();
+        connector->connectSecond(secondShape,3);
         connector->updateConnections();
         y += child->size().height() + fromChildToChild;
     }
