@@ -188,7 +188,8 @@ private:
     bool m_doingLayout;
     bool m_relayoutScheduled;
 
-    KoShape *m_root;
+    KoShape *m_root; // container
+    KoShape *m_rootShape; // background shape
     QList<KoShape*> m_children;
     QList<KoShape*> m_connectors;
     QMap<KoShape*, KoConnectionShape*> m_bonds;
