@@ -22,6 +22,7 @@
 #define TREESHAPE_H
 
 #include "KoShapeContainer.h"
+#include "KoResourceManager.h"
 #include <KoConnectionShape.h>
 
 #define TREESHAPEID "TreeShape"
@@ -45,15 +46,16 @@ public:
         FollowParent
     };
 
+    // ConnectionType is defined in KoConnectionShape
+
     enum RootType {
         Rectangle, Ellipse,
         None
     };
 
-    // ConnectionType is defined in KoConnectionShape
 
-    TreeShape();
-    TreeShape(KoShape *root);
+    TreeShape(KoResourceManager *documentResources=0);
+    TreeShape(KoShape *root, KoResourceManager *documentResources=0);
     virtual ~TreeShape();
     virtual void setZIndex(int zIndex);
     virtual void paintComponent(QPainter &painter, const KoViewConverter &converter);
@@ -80,6 +82,7 @@ private:
 //     virtual void shapeChanged(ChangeType type, KoShape *shape = 0);
     virtual Layout *layout() const;
 
+    KoResourceManager *m_documentResources;
     KoShape *m_nextShape;
 };
 
