@@ -143,13 +143,13 @@ void OutputDebugStrategy::rectangle( const QRect &box )
 void OutputDebugStrategy::modifyWorldTransform( quint32 mode, float M11, float M12,
 					float M21, float M22, float Dx, float Dy )
 {
-    kDebug(33100) << "EMR_MODIFYWORLDTRANSFORM:" << mode << QMatrix ( M11, M12, M21, M22, Dx, Dy );
+    kDebug(33100) << "EMR_MODIFYWORLDTRANSFORM:" << mode << QTransform ( M11, M12, M21, M22, Dx, Dy );
 }
 
 void OutputDebugStrategy::setWorldTransform( float M11, float M12, float M21,
 				     float M22, float Dx, float Dy )
 {
-    kDebug(33100) << "EMR_SETWORLDTRANSFORM:" << QMatrix ( M11, M12, M21, M22, Dx, Dy );
+    kDebug(33100) << "EMR_SETWORLDTRANSFORM:" << QTransform ( M11, M12, M21, M22, Dx, Dy );
 }
 
 void OutputDebugStrategy::setMapMode( quint32 mapMode )
@@ -260,6 +260,11 @@ void OutputDebugStrategy::createBrushIndirect( quint32 ihBrush, quint32 BrushSty
              << "Colour:" << QColor( red, green, blue ) << ", Hatch:" << BrushHatch;
 }
 
+void OutputDebugStrategy::createMonoBrush( quint32 ihBrush, Bitmap *bitmap )
+{
+    kDebug(33100) << "EMR_CREATEMONOBRUSH:" << ihBrush << "bitmap:" << bitmap;
+}
+
 void OutputDebugStrategy::selectObject( const quint32 ihObject )
 {
     kDebug(33100) << "EMR_SELECTOBJECT" << ihObject;
