@@ -32,15 +32,15 @@
 
 NotifyDialog::NotifyDialog(QWidget *parent)
         : QDialog(parent),
-        m_layout(NULL),
-        m_label(NULL)
+        m_layout(0),
+        m_label(0)
 {
     m_label = new QLabel(this);
     m_label->setText(i18n("File type not supported"));
     m_layout = new QVBoxLayout(this);
     m_layout->addWidget(m_label);
     this->setLayout(m_layout);
-    this->setWindowTitle(i18n("Office Viewer"));
+    this->setWindowTitle(i18n("FreOffice"));
     m_label->setAlignment(Qt::AlignCenter);
 }
 
